--*********************************************************************************
--*********************************************************************************
--
-- (c) Countersoft Gemini
--
-- DATA MIGRATION SCRIPT from Gemini 4.2 to Gemini 5.0
---
--*********************************************************************************
--*********************************************************************************
DELETE FROM gemini_install WHERE attribute_key = N'VERSION'
INSERT INTO gemini_install (attribute_key,attribute_value) VALUES (N'VERSION',N'5.0')
GO

ALTER TABLE gemini_applicationsettings DROP COLUMN settingcategory
ALTER TABLE gemini_applicationsettings DROP COLUMN settingdesc
GO
UPDATE gemini_applicationsettings SET settingname='AllowAnonymousAccess' WHERE settingname COLLATE DATABASE_DEFAULT  = 'AllowAnonymousUsers' COLLATE DATABASE_DEFAULT 
GO

TRUNCATE TABLE gemini_systemlog 
GO

/**********************************************************************************************************************
* New Tables
**********************************************************************************************************************/

CREATE TABLE gemini_issuewidgetdata
(
	id INT IDENTITY(1,1) CONSTRAINT gemini_issuewidgetdata_id_pk PRIMARY KEY NONCLUSTERED,
	issueid NUMERIC(10,0) NOT NULL CONSTRAINT gemini_issuewidgetdata_issueid_fk FOREIGN KEY REFERENCES gemini_issues(issueid),
	appid varchar(100) NOT NULL,
	controlid varchar(100) NOT NULL,
	value NVARCHAR(MAX) NOT NULL,
	created DATETIME NOT NULL CONSTRAINT gemini_issuewidgetdata_created_def DEFAULT GETUTCDATE()
)
CREATE UNIQUE INDEX ind_gemini_issuewidgetdata_ids ON gemini_issuewidgetdata(issueid, appid, controlid)
CREATE INDEX ind_gemini_issuewidgetdata_issueid ON gemini_issuewidgetdata(issueid)
GO

CREATE TABLE gemini_userwidgetdata
(
	id INT IDENTITY(1,1) CONSTRAINT gemini_userwidgetdata_watchid_pk PRIMARY KEY NONCLUSTERED,
	userid NUMERIC(10,0) NOT NULL CONSTRAINT gemini_userwidgetdata_userid_fk FOREIGN KEY REFERENCES gemini_users(userid),
	appid VARCHAR(38) NOT NULL,
	controlid VARCHAR(38) NOT NULL,
	value NVARCHAR(MAX) not null,
	created DATETIME NOT NULL CONSTRAINT gemini_userwidgetdata_created_def DEFAULT GETUTCDATE()
)
CREATE UNIQUE INDEX uind_gemini_userwidgetdata_id ON gemini_userwidgetdata(userid, appid, controlid)
CREATE INDEX ind_gemini_userwidgetdata_userid ON gemini_userwidgetdata(userid)
GO

CREATE TABLE gemini_issueaudit
(
	historyid INT IDENTITY(1,1) CONSTRAINT gemini_issueaudit_historyid_pk PRIMARY KEY NONCLUSTERED,
	issueid NUMERIC(10,0) NOT NULL CONSTRAINT gemini_issueaudit_issueid_fk FOREIGN KEY REFERENCES gemini_issues(issueid),
	projectid NUMERIC(10,0) NOT NULL CONSTRAINT gemini_issueaudit_projectid_fk FOREIGN KEY REFERENCES gemini_projects(projectid),
	userid NUMERIC(10,0) NULL CONSTRAINT gemini_issueaudit_userid_fk FOREIGN KEY REFERENCES gemini_users(userid),
	username NVARCHAR(255) NOT NULL,
	attributechanged INT NOT NULL,
	fieldchanged NVARCHAR(255) NOT NULL,
	iscustom BIT NOT NULL,
	valuebefore NVARCHAR(MAX) NOT NULL CONSTRAINT gemini_issueaudit_valuebefore_def DEFAULT(''),
	valueafter NVARCHAR(MAX) NOT NULL CONSTRAINT gemini_issueaudit_valueafter_def DEFAULT(''),
	idbefore NVARCHAR(255) NOT NULL CONSTRAINT gemini_issueaudit_idbeforedef DEFAULT(''),
	idafter NVARCHAR(255) NOT NULL CONSTRAINT gemini_issueaudit_idafter_def DEFAULT(''),
	created DATETIME NOT NULL CONSTRAINT gemini_issueaudit_created_def DEFAULT GETUTCDATE()
)
GO
CREATE INDEX ind_gemini_issueaudit_issueid ON gemini_issueaudit(issueid)
CREATE INDEX ind_gemini_issueaudit_projectid ON gemini_issueaudit(projectid)
GO
CREATE TABLE gemini_projectaudit
(
	historyid NUMERIC(10,0) IDENTITY(1,1) CONSTRAINT gemini_projectaudit_historyid_pk PRIMARY KEY NONCLUSTERED,
	projectid NUMERIC(10,0) NOT NULL CONSTRAINT gemini_projectaudit_projectid_fk FOREIGN KEY REFERENCES gemini_projects(projectid),
	userid NUMERIC(10,0) NULL CONSTRAINT gemini_projectaudit_userid_fk FOREIGN KEY REFERENCES gemini_users(userid),
	username NVARCHAR(255) NOT NULL,
	projectarea NVARCHAR(255) NOT NULL,
	useraction NVARCHAR(255) NOT NULL CONSTRAINT gemini_projectaudit_useraction_def DEFAULT(''),
	touched NVARCHAR(255) NOT NULL CONSTRAINT gemini_projectaudit_touched_def DEFAULT(''),
	created DATETIME NOT NULL CONSTRAINT gemini_projectaudit_created_def DEFAULT GETUTCDATE()
)
GO
CREATE INDEX ind_gemini_projectaudit_userid ON gemini_projectaudit(userid)
CREATE INDEX ind_gemini_projectaudit_projectid ON gemini_projectaudit(projectid)
GO

CREATE TABLE gemini_projecttemplate
(
	templateid INT IDENTITY(1,1) CONSTRAINT gemini_projecttemplate_templateid_pk PRIMARY KEY NONCLUSTERED,
	templatekey VARCHAR(100) NOT NULL,
	title NVARCHAR(500) NOT NULL CONSTRAINT gemini_projecttemplate_title_def DEFAULT(''),
	descrip NVARCHAR(2000) NOT NULL CONSTRAINT gemini_projecttemplate_descrip_def DEFAULT(''),
	author NVARCHAR(500) NOT NULL CONSTRAINT gemini_projecttemplate_author_def DEFAULT(''),
	published DATETIME NOT NULL CONSTRAINT gemini_projecttemplate_published_def DEFAULT GETUTCDATE(),
	versionnumber VARCHAR(50) NOT NULL CONSTRAINT gemini_projecttemplate_versionnumber_def DEFAULT(''),
	template NVARCHAR(MAX) NOT NULL CONSTRAINT gemini_projecttemplate_template_def DEFAULT (''),	
	created DATETIME NOT NULL CONSTRAINT gemini_projecttemplate_created_def DEFAULT GETUTCDATE()
)
GO
CREATE INDEX ind_gemini_projecttemplate_templateid ON gemini_projecttemplate(templateid)
CREATE INDEX ind_gemini_projecttemplate_templatekey ON gemini_projecttemplate(templatekey)
GO

SET IDENTITY_INSERT gemini_projecttemplate ON
INSERT gemini_projecttemplate ([templateid], [templatekey], [title], [descrip], [author], [versionnumber], [created], [template], [published]) 
	VALUES (1, N'Migration', N'Migration Template', N'Migration template for v4 to v5 Gemini projects', N'Countersoft', N'1.0.0', CAST(0x0000A00C0096DE08 AS DateTime), N'{"Name":"Migration Template","Description":"Migration template for v4 to v5 Gemini projects","Publisher":"Countersoft","Published":"2012-07-03T11:54:07.9423439+01:00","Version":"1.0.0","HomePage":1,"TemplateKey":"Migration","Menu":[{"PageType":1,"SeperateAfter":true},{"PageType":2,"SeperateAfter":true},{"PageType":3,"SeperateAfter":false},{"PageType":4,"SeperateAfter":false},{"PageType":5,"SeperateAfter":true},{"PageType":9,"SeperateAfter":false},{"PageType":19,"SeperateAfter":false},{"PageType":20,"SeperateAfter":true},{"PageType":8,"SeperateAfter":false},{"PageType":7,"SeperateAfter":false}],"IsNew":true,"IsExisting":false,"Active":true,"Archived":false,"Deleted":false,"Created":"2012-07-03T10:54:06.1482413Z","Revised":"2012-07-03T10:54:06.1482413Z","Errors":[],"HasErrors":false,"Id":0}', CAST(0x0000A08300C42354 AS DateTime))
SET IDENTITY_INSERT gemini_projecttemplate OFF
GO

CREATE TABLE gemini_navigationcard
(
	cardid INT IDENTITY(1,1) CONSTRAINT gemini_navigationcard_templateid_pk PRIMARY KEY NONCLUSTERED,
	projectid NUMERIC(10,0) CONSTRAINT gemini_navigationcard_projectid_fk FOREIGN KEY REFERENCES gemini_projects(projectid),
	issueid NUMERIC(10,0) CONSTRAINT gemini_navigationcard_issueid_fk FOREIGN KEY REFERENCES gemini_issues(issueid),
	userid NUMERIC(10,0) CONSTRAINT gemini_navigationcard_userid_fk FOREIGN KEY REFERENCES gemini_users(userid),
	cardtype NVARCHAR(255) NOT NULL CONSTRAINT gemini_navigationcard_cardtype_def DEFAULT('1'),
	cardkey NVARCHAR(100) NOT NULL CONSTRAINT gemini_navigationcard_cardkey_def DEFAULT(''),
	cardtitle NVARCHAR(500) NOT NULL CONSTRAINT gemini_navigationcard_cardtitle_def DEFAULT(''),
	carddata NVARCHAR(MAX) NOT NULL CONSTRAINT gemini_navigationcard_carddata_def DEFAULT(''),
	cardoptions NVARCHAR(MAX) NOT NULL CONSTRAINT gemini_navigationcard_cardoptions_def DEFAULT(''),
	cardurl NVARCHAR(1000) NOT NULL CONSTRAINT gemini_navigationcard_cardurl_def DEFAULT(''),
	originatorid INT CONSTRAINT gemini_navigationcard_originatorid_fk FOREIGN KEY REFERENCES gemini_navigationcard(cardid),
	lastseen DATETIME,
	badgecount INT NOT NULL CONSTRAINT gemini_navigationcard_badgecount_def DEFAULT(0),
	sharedgroups NVARCHAR(1000) NOT NULL CONSTRAINT gemini_navigationcard_sharedgroups_def DEFAULT(''),
	color VARCHAR(20) NOT NULL CONSTRAINT gemini_navigationcard_color_def DEFAULT(''),
	visibletoanonuser BIT NOT NULL CONSTRAINT gemini_navigationcard_visibletoanonuser_def DEFAULT(0),
	visibletoauthuser BIT NOT NULL CONSTRAINT gemini_navigationcard_visibletoauthuser_def DEFAULT(0),
	generated BIT NOT NULL CONSTRAINT gemini_navigationcard_generated_def DEFAULT(0),
	sequence INT NOT NULL CONSTRAINT gemini_navigationcard_sequence_def DEFAULT(1),
	created DATETIME NOT NULL CONSTRAINT gemini_navigationcard_created_def DEFAULT GETUTCDATE(),
	tstamp TIMESTAMP 	
)
GO

CREATE INDEX ind_gemini_navigationcards_cardid ON gemini_navigationcard(cardid)
CREATE INDEX ind_gemini_navigationcards_userid ON gemini_navigationcard(userid)
CREATE INDEX ind_gemini_navigationcards_projectid ON gemini_navigationcard(projectid)
CREATE INDEX ind_gemini_navigationcards_issueid ON gemini_navigationcard(issueid)
CREATE INDEX ind_gemini_navigationcards_cardtype ON gemini_navigationcard(cardtype)
CREATE INDEX ind_gemini_navigationcards_originatorid ON gemini_navigationcard(originatorid)
GO

INSERT INTO gemini_navigationcard (generated, cardtype, cardkey, cardtitle, cardurl, color, visibletoanonuser, visibletoauthuser, badgecount, lastseen)
	VALUES (1, 0, 'WHY GEMINI?', 'See Killer Features', 'http://www.countersoft.com', '#562366', 1, 0, 0, GETUTCDATE())
INSERT INTO gemini_navigationcard (generated, cardtype, cardkey, cardtitle, cardurl, color, visibletoanonuser, visibletoauthuser, badgecount, lastseen)
	VALUES (1, 0, 'HELP', 'Docs & Videos', 'http://docs.countersoft.com', '#562366', 0, 1, 0, GETUTCDATE())
INSERT INTO gemini_navigationcard (generated, cardtype, cardkey, cardtitle, cardurl, color, visibletoanonuser, visibletoauthuser, badgecount, lastseen)
	VALUES (1, 0, '', '', 'projects', '#562366', 0, 0, 0, GETUTCDATE())
GO
CREATE TABLE gemini_sharedviews
(
   sharedviewid INT IDENTITY (1,1) NOT NULL CONSTRAINT gemini_sharedviews_sharedviewid_pk PRIMARY KEY,
   userid NUMERIC (10,0) NULL CONSTRAINT gemini_sharedviews_userid_fk FOREIGN KEY REFERENCES gemini_users(userid),
   projectid NUMERIC (10,0) NULL CONSTRAINT gemini_sharedviews_projectid_fk FOREIGN KEY REFERENCES gemini_projects(projectid),
   pagetype INT NOT NULL,
   pageid NVARCHAR(50),
   name NVARCHAR(50) NOT NULL,
   visibility NVARCHAR(1000) NOT NULL,
   value NVARCHAR(MAX) NOT NULL CONSTRAINT gemini_sharedviews_value_def DEFAULT(''),
   created DATETIME NOT NULL CONSTRAINT gemini_sharedviews_created_def DEFAULT (GETUTCDATE()),
   tstamp TIMESTAMP NOT NULL
)
GO

CREATE INDEX ind_gemini_sharedviews_userid ON gemini_sharedviews(userid)
CREATE INDEX ind_gemini_sharedviews_projectid ON gemini_sharedviews(projectid)
GO
CREATE TABLE gemini_plannerviews
(
   plannerviewid INT IDENTITY (1,1) NOT NULL CONSTRAINT gemini_plannerviews_plannerviewid_pk PRIMARY KEY,
   userid NUMERIC (10,0) NULL CONSTRAINT gemini_plannerviews_userid_fk FOREIGN KEY REFERENCES gemini_users(userid),
   issuesfilter NVARCHAR(MAX) NOT NULL,
   issuessequence VARCHAR(MAX) NOT NULL,
   axis1property VARCHAR(50) NOT NULL,
   axis1sortdirection VARCHAR(5) NOT NULL,
   axis2property VARCHAR(50) NOT NULL,
   axis2sortdirection VARCHAR(5) NOT NULL,
   orderbyproperty VARCHAR(50) NOT NULL,
   orderbysortdirection VARCHAR(5) NOT NULL,
   thenbyproperty VARCHAR(50) NOT NULL,
   thenbysortdirection VARCHAR(5) NOT NULL,
   colorbyproperty VARCHAR(50) NOT NULL,
   swimlanelimit INT NOT NULL,
   laneslimit INT NOT NULL,
   zoomlevel INT NOT NULL,
   tottingmodeproperty VARCHAR(50) NOT NULL,
   autolimit INT NOT NULL,
   daterangeproperty VARCHAR(50) NOT NULL,
   paginginfo VARCHAR(MAX) NOT NULL,
   created DATETIME NOT NULL CONSTRAINT gemini_plannerviews_created_def DEFAULT (GETUTCDATE()),
   tstamp TIMESTAMP NOT NULL
)
GO

CREATE INDEX ind_gemini_plannerviews_userid ON gemini_plannerviews(userid)
GO
CREATE TABLE gemini_projectissuesequence
(
	id INT IDENTITY(1,1) CONSTRAINT gemini_projectissuesequence_id_pk PRIMARY KEY,
	projectid NUMERIC(10,0) NULL CONSTRAINT gemini_projectissuesequence_projectid_fk FOREIGN KEY REFERENCES gemini_projects(projectid),
	sequence VARCHAR(MAX) NOT NULL,
	created DATETIME NOT NULL CONSTRAINT gemini_projectissuesequence_created_def DEFAULT GETUTCDATE(),
	tstamp TIMESTAMP NOT NULL
)
CREATE INDEX ind_gemini_projectissuesequence_projectid ON gemini_projectissuesequence(projectid)
GO
CREATE TABLE gemini_alerttemplates
(
	alerttemplateid INT IDENTITY(1,1) CONSTRAINT gemini_alerttemplates_alerttemplateid_pk PRIMARY KEY NONCLUSTERED,
	alerttype INT CONSTRAINT gemini_alerttemplates_alerttype_def DEFAULT(0),
	label NVARCHAR(100) NOT NULL CONSTRAINT gemini_alerttemplates_alertlabel_def DEFAULT(''),
	alertcontent NVARCHAR(MAX) NOT NULL CONSTRAINT gemini_alerttemplates_alertcontent_def DEFAULT(''),
	projects NVARCHAR(MAX) NOT NULL CONSTRAINT gemini_alerttemplates_projects_def DEFAULT(''),
	created DATETIME NOT NULL CONSTRAINT ggemini_alerttemplates_created_def DEFAULT GETUTCDATE(),
	tstamp TIMESTAMP 	
)
GO
INSERT gemini_alerttemplates(alerttype,label,alertcontent,projects) VALUES('4','When something in the AppNav Card changes',
'<table style="width: 100%; font-family: arial; font-size: 16px; font-weight: bold; border: none; background-color: #8f0929;" cellpadding="20">
<tbody>
<tr>
<td style="color: white;">GEMINI TRACKER</td>
</tr>
</tbody>
</table>

<table style="width: 100%; font-family: arial; font-size: 13px; font-weight: normal; border: none; background-color: white;" cellpadding="10">
<tbody>
<tr>
<td style="color: #4c4c4c;">Just to let you know that there are <strong>@(Model.ChangeCount) changes </strong> to your <strong>@(Model.CardTitle)</strong> AppNav card.</td>
</tr>
</tbody>
</table>

@if (Model.TheItemsCreated.Count > 0)
{
	<h2 style="margin: 10px 0 10px 20px;">New</h2>
	<table style="margin-left: 40px; font-family: arial; font-size: 13px; font-weight: normal; border: none; background-color: white; border-collapse: collapse;" cellpadding="10">
	<tbody>	
	@foreach(var item in Model.TheItemsCreated)
	{
		<tr>
			<td style="color: #4c4c4c; border: 1px solid #f5f5f5; background-color: #f5f5f5; font-weight: bold; font-size: 13px;"><a href="@(Model.GeminiUrl)project/@(item.ProjectCode)/@(item.Entity.ProjectId)/item/@(item.Entity.Id)">@(item.IssueKey)</a></td>
			<td style="color: #4c4c4c; border: 1px solid #f5f5f5; font-weight: normal; font-size: 12px;">@(item.Title)</td>
		</tr>
	}
	</tbody>
	</table>
}

@if (Model.TheItemsUpdated.Count > 0)
{
	<h2 style="margin: 10px 0 10px 20px;">Changed</h2>	
	<table style="margin-left: 40px; font-family: arial; font-size: 13px; font-weight: normal; border: none; background-color: white; border-collapse: collapse;" cellpadding="10">
	<tbody>	
	@foreach(var item in Model.TheItemsUpdated)
	{
		<tr>
			<td style="color: #4c4c4c; border: 1px solid #f5f5f5; background-color: #f5f5f5; font-weight: bold; font-size: 13px;"><a href="@(Model.GeminiUrl)project/@(item.ProjectCode)/@(item.Entity.ProjectId)/item/@(item.Entity.Id)">@(item.IssueKey)</a></td>
			<td style="color: #4c4c4c; border: 1px solid #f5f5f5; font-weight: normal; font-size: 12px;">@(item.Title)</td>
		</tr>
	}
	</tbody>
	</table>	
}

@if (Model.TheItemsCommented.Count > 0)
{
	<h2 style="margin: 10px 0 10px 20px;">Comments</h2>	
	<table style="margin-left: 40px; font-family: arial; font-size: 13px; font-weight: normal; border: none; background-color: white; border-collapse: collapse;" cellpadding="10">
	<tbody>	
	@foreach(var item in Model.TheItemsCommented)
	{
		<tr>
			<td style="color: #4c4c4c; border: 1px solid #f5f5f5; background-color: #f5f5f5; font-weight: bold; font-size: 13px;">@(item.Comments[0].Entity.Fullname)</td>
			<td style="color: #4c4c4c; border: 1px solid #f5f5f5; font-weight: normal; font-size: 12px;">
			<a href="@(Model.GeminiUrl)project/@(item.ProjectCode)/@(item.Entity.ProjectId)/item/@(item.Entity.Id)#comments">@(item.IssueKey)</a>
			&nbsp;@(item.Title)</td>
		</tr>
	}
	</tbody>
	</table>}

<table style="font-family: arial; font-size: 13px; font-weight: normal; border: none; background-color: white;" cellpadding="10">
<tbody>
<tr>
<td style="color: #4c4c4c;">Thanks.</td>
</tr>
</tbody>
</table>
<table style="width: 100%; font-family: arial; font-size: 11px; font-weight: normal; border: none; background-color: #8f0929;" cellpadding="5">
<tbody>
<tr>
<td style="color: white;">Sent by Countersoft Gemini v@(Model.Version</td>
</tr>
</tbody>
</table>'
,'')

INSERT gemini_alerttemplates(alerttype,label,alertcontent,projects) VALUES('2','When watched items change',
'<table style="width: 100%; font-family: arial; font-size: 16px; font-weight: bold; border: none; background-color: #8f0929;" cellpadding="20">
<tbody>
<tr>
<td style="color: white;">GEMINI TRACKER</td>
</tr>
</tbody>
</table>

<table style="width: 100%; font-family: arial; font-size: 13px; font-weight: normal; border: none; background-color: white;" cellpadding="10">
<tbody>
<tr>
<td style="color: #4c4c4c;">Just to let you know that there are <strong>@(Model.ChangeCount) changes </strong> to your watched items.</td>
</tr>
</tbody>
</table>

@if (Model.TheItemsUpdated.Count > 0)
{
	<h2 style="margin: 10px 0 10px 20px;">Changed</h2>	
	<table style="margin-left: 40px; font-family: arial; font-size: 13px; font-weight: normal; border: none; background-color: white; border-collapse: collapse;" cellpadding="10">
	<tbody>	
	@foreach(var item in Model.TheItemsUpdated)
	{
		<tr>
			<td style="color: #4c4c4c; border: 1px solid #f5f5f5; background-color: #f5f5f5; font-weight: bold; font-size: 13px;"><a href="@(Model.GeminiUrl)project/@(item.ProjectCode)/@(item.Entity.ProjectId)/item/@(item.Entity.Id)">@(item.IssueKey)</a></td>
			<td style="color: #4c4c4c; border: 1px solid #f5f5f5; font-weight: normal; font-size: 12px;">@(item.Title)</td>
		</tr>
	}
	</tbody>
	</table>	
}

<table style="font-family: arial; font-size: 13px; font-weight: normal; border: none; background-color: white;" cellpadding="10">
<tbody>
<tr>
<td style="color: #4c4c4c;">Thanks.</td>
</tr>
</tbody>
</table>
<table style="width: 100%; font-family: arial; font-size: 11px; font-weight: normal; border: none; background-color: #8f0929;" cellpadding="5">
<tbody>
<tr>
<td style="color: white;">Sent by Countersoft Gemini v@(Model.Version</td>
</tr>
</tbody>
</table>','')


INSERT gemini_alerttemplates(alerttype,label,alertcontent,projects) VALUES('1','When new things are added',
'<table style="width: 100%; font-family: arial; font-size: 16px; font-weight: bold; border: none; background-color: #8f0929;" cellpadding="20">
<tbody>
<tr>
<td style="color: white;">GEMINI TRACKER</td>
</tr>
</tbody>
</table>
<table style="width: 100%; font-family: arial; font-size: 13px; font-weight: normal; border: none; background-color: white;" cellpadding="10">
<tbody>
<tr>
<td style="color: #4c4c4c;">Just to let you know that a new <strong>@(Model.TheItem.Type)</strong> has been added to <strong>@(Model.TheItem.ProjectName).</strong></td>
</tr>
</tbody>
</table>
<table style="margin-left: 20px; font-family: arial; font-size: 13px; font-weight: normal; border: none; background-color: white; border-collapse: collapse;" cellpadding="10">
<tbody>
<tr>
<td style="color: #4c4c4c; border: 1px solid #f5f5f5; background-color: #f5f5f5; font-weight: bold; font-size: 13px;">@(Model.TheItem.IssueKey)</td>
<td style="color: #4c4c4c; border: 1px solid #f5f5f5; background-color: #f5f5f5; font-weight: bold; font-size: 13px;">@(Model.TheItem.Title)</td>
</tr>
<tr>
<td style="color: #4c4c4c; border: 1px solid #f5f5f5;"><strong>Status:</strong></td>
<td style="border: 1px solid #f5f5f5;">@(Model.TheItem.Status)</td>
</tr>
<tr>
<td style="color: #4c4c4c; border: 1px solid #f5f5f5;"><strong>Resourced:</strong></td>
<td style="color: #4c4c4c; border: 1px solid #f5f5f5;">@(Model.TheItem.ResourceNames)</td>
</tr>
</tbody>
</table>
<table style="margin-left: 20px; font-family: arial; font-size: 13px; font-weight: normal; border: none; background-color: white; border-collapse: collapse;" cellpadding="10">
<tbody>
<tr>
<td style="color: #4c4c4c;">@(Model.LinkViewItem)</td>
</tr>
</tbody>
</table>
<table style="font-family: arial; font-size: 13px; font-weight: normal; border: none; background-color: white;" cellpadding="10">
<tbody>
<tr>
<td style="color: #4c4c4c;">Thanks.</td>
</tr>
</tbody>
</table>
<table style="width: 100%; font-family: arial; font-size: 11px; font-weight: normal; border: none; background-color: #8f0929;" cellpadding="5">
<tbody>
<tr>
<td style="color: white;">Sent by Countersoft Gemini v@(Model.Version</td>
</tr>
</tbody>
</table>','')

INSERT gemini_alerttemplates(alerttype,label,alertcontent,projects) VALUES('3','When someone is resourced',
'<table style="width: 100%; font-family: arial; font-size: 16px; font-weight: bold; border: none; background-color: #8f0929;" cellpadding="20">
<tbody>
<tr>
<td style="color: white;">GEMINI TRACKER</td>
</tr>
</tbody>
</table>
<table style="width: 100%; font-family: arial; font-size: 13px; font-weight: normal; border: none; background-color: white;" cellpadding="10">
<tbody>
<tr>
<td style="color: #4c4c4c;">Just to let you know the following <strong>@(Model.TheItem.Type)</strong>&nbsp;in <strong>@(Model.TheItem.ProjectName)</strong> has been assigned to you:</td>
</tr>
</tbody>
</table>
<table style="margin-left: 20px; font-family: arial; font-size: 13px; font-weight: normal; border: none; background-color: white; border-collapse: collapse;" cellpadding="10">
<tbody>
<tr>
<td style="color: #4c4c4c; border: 1px solid #f5f5f5; background-color: #f5f5f5; font-weight: bold; font-size: 13px;">@(Model.TheItem.IssueKey)</td>
<td style="color: #4c4c4c; border: 1px solid #f5f5f5; background-color: #f5f5f5; font-weight: bold; font-size: 13px;">@(Model.TheItem.Title)</td>
</tr>
<tr>
<td style="color: #4c4c4c; border: 1px solid #f5f5f5;"><strong>Status:</strong></td>
<td style="border: 1px solid #f5f5f5;">@(Model.TheItem.Status)</td>
</tr>
<tr>
<td style="color: #4c4c4c; border: 1px solid #f5f5f5;"><strong>Resourced:</strong></td>
<td style="color: #4c4c4c; border: 1px solid #f5f5f5;">@(Model.TheItem.ResourceNames)</td>
</tr>
</tbody>
</table>
<table style="margin-left: 20px; font-family: arial; font-size: 13px; font-weight: normal; border: none; background-color: white; border-collapse: collapse;" cellpadding="10">
<tbody>
<tr>
<td style="color: #4c4c4c;">@(Model.LinkViewItem)</td>
</tr>
</tbody>
</table>
<table style="font-family: arial; font-size: 13px; font-weight: normal; border: none; background-color: white;" cellpadding="10">
<tbody>
<tr>
<td style="color: #4c4c4c;">Thanks.</td>
</tr>
</tbody>
</table>
<table style="width: 100%; font-family: arial; font-size: 11px; font-weight: normal; border: none; background-color: #8f0929;" cellpadding="5">
<tbody>
<tr>
<td style="color: white;">Sent by Countersoft Gemini v@(Model.Version</td>
</tr>
</tbody>
</table>','')
GO

CREATE TABLE breeze_smtpserver (
	smtpserverid  INT IDENTITY(1,1) NOT NULL CONSTRAINT breeze_smtpserver_smtpserverid_pk PRIMARY KEY,
	smtpservername NVARCHAR(255) NOT NULL CONSTRAINT breeze_smtpserver_smtpservername_def  DEFAULT (''),
	server NVARCHAR(255) NOT NULL CONSTRAINT breeze_smtpserver_server_def  DEFAULT (''),
	serverport INT NOT NULL CONSTRAINT breeze_smtpserver_serverport_def  DEFAULT (0),
	username NVARCHAR(255) NOT NULL CONSTRAINT breeze_smtpserver_username_def  DEFAULT (''),
	password NVARCHAR(255) NOT NULL CONSTRAINT breeze_smtpserver_password_def  DEFAULT (''),
	encodingtype INT NOT NULL CONSTRAINT breeze_smtpserver_encodingtype_def  DEFAULT (0),
	usessl BIT NOT NULL CONSTRAINT breeze_smtpserver_usessl_def  DEFAULT (0),
	sslmode INT NOT NULL CONSTRAINT breeze_smtpserver_sslmode_def  DEFAULT (0),
	authmode INT NOT NULL CONSTRAINT breeze_smtpserver_authmode_def  DEFAULT (0),
	popbeforesmtp BIT NOT NULL CONSTRAINT breeze_smtpserver_popbeforesmtp_def  DEFAULT (0),
	created DATETIME NOT NULL CONSTRAINT breeze_smtpserver_created_def DEFAULT (GETUTCDATE()),
	tstamp TIMESTAMP NOT NULL
)
GO

CREATE TABLE breeze_queue (
	queueid INT IDENTITY(1, 1) NOT NULL CONSTRAINT breeze_queue_queueid_pk PRIMARY KEY,
	name NVARCHAR(500) NOT NULL CONSTRAINT breeze_queue_name_def DEFAULT (''), 
	queuedescription NVARCHAR(MAX) NOT NULL CONSTRAINT breeze_queue_queuedescription_def DEFAULT (''), 
	userid INT NOT NULL CONSTRAINT breeze_queue_userid_def DEFAULT(0),
	visibility VARCHAR(MAX) NOT NULL CONSTRAINT breeze_queue_visibility_def DEFAULT (''),
	colour VARCHAR(50) NOT NULL CONSTRAINT breeze_queue_color_def DEFAULT ('white'),
	sequence INT NOT NULL CONSTRAINT breeze_queue_sequence_def DEFAULT (0),
	projectid INT NOT NULL CONSTRAINT breeze_queue_projectid_def DEFAULT (0),
	active BIT NOT NULL CONSTRAINT breeze_queue_active_def DEFAULT (0),
	created DATETIME NOT NULL CONSTRAINT breeze_queue_created_def DEFAULT (GETUTCDATE()),
	tstamp TIMESTAMP NOT NULL
)
GO

CREATE TABLE breeze_mailbox (
	mailboxid INT IDENTITY(1,1) NOT NULL CONSTRAINT breeze_mailbox_mailboxid_pk PRIMARY KEY,
	queueid INT NOT NULL CONSTRAINT breeze_mailbox_queueid_fk FOREIGN KEY REFERENCES breeze_queue(queueid),
	mailboxname NVARCHAR(100) NOT NULL CONSTRAINT breeze_mailbox_mailboxname_def  DEFAULT ('') ,
	server NVARCHAR(255) NOT NULL CONSTRAINT breeze_mailbox_server_def  DEFAULT (''),
	serverport INT NOT NULL CONSTRAINT breeze_mailbox_serverport_def  DEFAULT (0),
	emailaddress NVARCHAR(255) NOT NULL CONSTRAINT breeze_mailbox_emailaddress_def  DEFAULT (''),
	username NVARCHAR(100) NOT NULL CONSTRAINT breeze_mailbox_username_def  DEFAULT (''),
	password NVARCHAR(100) NOT NULL CONSTRAINT breeze_mailbox_password_def  DEFAULT (''),
	authenticationmode INT NOT NULL CONSTRAINT breeze_mailbox_authenticationmode_def  DEFAULT (0),
	usessl BIT NOT NULL CONSTRAINT breeze_mailbox_usessl_def  DEFAULT (0),
	deletemessages BIT NOT NULL CONSTRAINT breeze_mailbox_deletemessages_def  DEFAULT (0),
	debugmode BIT NOT NULL CONSTRAINT breeze_mailbox_incomingdebugmode_def  DEFAULT (0),
	userid INT NOT NULL CONSTRAINT breeze_mailbox_userid_def  DEFAULT (0),
	usesenderassubmitter BIT NOT NULL CONSTRAINT breeze_mailbox_usesenderassubmitter_def  DEFAULT (0),
	mode INT NOT NULL CONSTRAINT breeze_mailbox_mode_def  DEFAULT (0),
	imapfolder NVARCHAR(255) NOT NULL CONSTRAINT breeze_mailbox_imapfolder_def  DEFAULT (''),
	blacklist NVARCHAR(MAX) NOT NULL CONSTRAINT breeze_mailbox_blacklist_def  DEFAULT (''),
	noreplylist NVARCHAR(MAX) NOT NULL CONSTRAINT breeze_mailbox_noreplylist_def  DEFAULT (''),
	lastmessageid NVARCHAR(255) NULL,
	lastmessagedate DATETIME NULL,
	alerttemplateid INT NULL CONSTRAINT breeze_mailbox_alerttemplateid_fk FOREIGN KEY REFERENCES gemini_alerttemplates(alerttemplateid),
	smtpserverid INT NULL CONSTRAINT breeze_mailbox_smtpserverid_fk FOREIGN KEY REFERENCES breeze_smtpserver(smtpserverid), 
	stripsignature BIT NOT NULL CONSTRAINT breeze_mailbox_stripsignature_def DEFAULT (0),
	ignoreattachments BIT NOT NULL CONSTRAINT breeze_mailbox_ignoreattachments_def DEFAULT (0),
	subjectlikeexp NVARCHAR(MAX) NOT NULL CONSTRAINT breeze_mailbox_subjectlikeexp_def DEFAULT (''),
	subjectnotlikeexp NVARCHAR(MAX) NOT NULL CONSTRAINT breeze_mailbox_subjectnotlikeexp_def DEFAULT (''),
	replaceexp NVARCHAR(MAX) NOT NULL CONSTRAINT breeze_mailbox_replaceexp_def DEFAULT (''),
	truncateexp NVARCHAR(MAX) NOT NULL CONSTRAINT breeze_mailbox_truncateexp_def DEFAULT (''),
	projectid INT NULL,
	issuetypeid INT NULL,
	created DATETIME NOT NULL CONSTRAINT breeze_mailbox_created_def DEFAULT (GETUTCDATE()),
	tstamp TIMESTAMP NOT NULL
)
GO

CREATE TABLE breeze_enquiry(
	enquiryid BIGINT IDENTITY(1, 1) NOT NULL CONSTRAINT breeze_enquiry_enquiryid_pk PRIMARY KEY,
	queueid INT NULL CONSTRAINT breeze_enquiry_queueid_fk FOREIGN KEY REFERENCES breeze_queue(queueid),
	issueid NUMERIC(10, 0) NULL CONSTRAINT breeze_enquiry_issueid_fk FOREIGN KEY REFERENCES gemini_issues(issueid),
	projectid INT NULL,
	issuekey NVARCHAR(25) NULL,
	issuecommentid NUMERIC(10, 0) NULL CONSTRAINT breeze_enquiry_issuecommentid_fk FOREIGN KEY REFERENCES gemini_issuecomments(commentid),
	alerttemplateid INT NULL CONSTRAINT breeze_enquiry_alerttemplateid_fk FOREIGN KEY REFERENCES gemini_alerttemplates(alerttemplateid), 
	enquirysource INT NOT NULL CONSTRAINT breeze_enquiry_source_def DEFAULT (0),
	enquirydata NVARCHAR(MAX) NOT NULL CONSTRAINT breeze_enquiry_enquirydata_def DEFAULT (''),
	rawenquiry NVARCHAR(MAX) NOT NULL CONSTRAINT breeze_enquiry_rawenquiry_def DEFAULT (''),
	rawid NVARCHAR(255) NOT NULL CONSTRAINT breeze_enquiry_rawid_def DEFAULT (''),
	cancontact BIT NOT NULL CONSTRAINT breeze_qnquiry_cancontact_def DEFAULT (0),
	acknowledged DATETIME  NULL CONSTRAINT breeze_qnquiry_acknowledged_def DEFAULT (GETUTCDATE()),
	received DATETIME NOT NULL CONSTRAINT breeze_enquiry_received_def DEFAULT (GETUTCDATE()),
	created DATETIME NOT NULL CONSTRAINT breeze_enquiry_created_def DEFAULT (GETUTCDATE()),
	tstamp TIMESTAMP NOT NULL
)
GO

CREATE TABLE breeze_matchexpression(
	expressionid BIGINT IDENTITY(1, 1) NOT NULL CONSTRAINT breeze_matchexpression_expressionid_pk PRIMARY KEY,
	name VARCHAR(50) NOT NULL CONSTRAINT breeze_matchexpression_name_def DEFAULT (''),
	expression NVARCHAR(MAX) NOT NULL CONSTRAINT breeze_matchexpression_expression_def DEFAULT (''),
	replacevalue NVARCHAR(MAX) NOT NULL CONSTRAINT breeze_matchexpression_replacevalue_def DEFAULT (''),
	created DATETIME NOT NULL CONSTRAINT breeze_matchexpression_created_def DEFAULT (GETUTCDATE()),
	tstamp TIMESTAMP NOT NULL
)
GO

CREATE TABLE breeze_mailboxmessage (
      messageid INT IDENTITY(1,1) NOT NULL CONSTRAINT breeze_mailboxmessage_messageid_pk PRIMARY KEY,
      mailboxid INT NOT NULL CONSTRAINT breeze_mailboxmessage_mailboxid_def DEFAULT(0),
      [uid] NVARCHAR(255) NOT NULL CONSTRAINT breeze_mailboxmesssage_uid_def DEFAULT(''),
      created DATETIME NOT NULL CONSTRAINT breeze_mailboxmessage_created_def DEFAULT (GETUTCDATE()),
      tstamp TIMESTAMP NOT NULL
)
GO

/**********************************************************************************************************************
* End
**********************************************************************************************************************/

/**********************************************************************************************************************
* Amend schema
**********************************************************************************************************************/
ALTER TABLE gemini_applicationsettings ALTER COLUMN settingvalue NVARCHAR(MAX)
GO

ALTER TABLE gemini_projects DROP CONSTRAINT gemini_projects_projectreadonly_def
ALTER TABLE gemini_projects DROP CONSTRAINT gemini_projects_projectarchived_def
EXEC sp_rename 'gemini_projects.projectreadonly', 'zprojectreadonly', 'column'
EXEC sp_rename 'gemini_projects.projectarchived', 'zprojectarchived', 'column'
GO
ALTER TABLE gemini_projects ADD projectreadonly BIT NOT NULL CONSTRAINT gemini_projects_projectreadonly_def DEFAULT(0)
ALTER TABLE gemini_projects ADD projectarchived BIT NOT NULL CONSTRAINT gemini_projects_projectarchived_def DEFAULT(0)
GO
UPDATE gemini_projects SET 
	projectreadonly = CASE WHEN zprojectreadonly COLLATE DATABASE_DEFAULT = 'Y' COLLATE DATABASE_DEFAULT THEN 1 ELSE 0 END,
	projectarchived = CASE WHEN zprojectarchived COLLATE DATABASE_DEFAULT = 'Y' COLLATE DATABASE_DEFAULT THEN 1 ELSE 0 END
GO
ALTER TABLE gemini_projects DROP COLUMN zprojectreadonly 
ALTER TABLE gemini_projects DROP COLUMN zprojectarchived 
GO

ALTER TABLE gemini_projectdefaultvalues DROP CONSTRAINT gemini_projectdefaultvalues_defaultvalue_def
GO
ALTER TABLE gemini_projectdefaultvalues ALTER COLUMN defaultvalue NVARCHAR(MAX) NOT NULL 
GO
ALTER TABLE gemini_projectdefaultvalues ADD CONSTRAINT gemini_projectdefaultvalues_defaultvalue_def DEFAULT('') FOR defaultvalue 
GO
DELETE FROM gemini_projectdefaultvalues
GO

-- Add Colors
ALTER TABLE gemini_projects ADD color varchar(20) NOT NULL CONSTRAINT gemini_projects_color_def DEFAULT('#A9A9A9')
ALTER TABLE gemini_issuetypes ADD color VARCHAR(20) NOT NULL CONSTRAINT gemini_issuetypes_color_def DEFAULT('#A9A9A9')
ALTER TABLE gemini_issuetypes ADD screen VARCHAR(MAX) NOT NULL CONSTRAINT gemini_issuetypes_screen_def DEFAULT('')
ALTER TABLE gemini_issuetypes ADD workflow VARCHAR(MAX) NOT NULL CONSTRAINT gemini_issuetypes_workflow_def DEFAULT('')
ALTER TABLE gemini_issuetypes ADD tag VARCHAR(50) NOT NULL CONSTRAINT gemini_issuetypes_tag_def DEFAULT('')
ALTER TABLE gemini_issuepriorities ADD color varchar(20) NOT NULL CONSTRAINT gemini_issuepriorities_color_def DEFAULT('#A9A9A9')
ALTER TABLE gemini_issueseverity ADD color varchar(20) NOT NULL CONSTRAINT gemini_issueseverity_color_def DEFAULT('#A9A9A9')
ALTER TABLE gemini_issuestatus ADD color varchar(20) NOT NULL CONSTRAINT gemini_issuestatus_color_def DEFAULT('#A9A9A9')
ALTER TABLE gemini_issuestatus ADD seq INT NOT NULL CONSTRAINT gemini_issuestatus_seq_def DEFAULT(0)
ALTER TABLE gemini_projects ADD options NVARCHAR(MAX) NOT NULL CONSTRAINT gemini_projects_options_def DEFAULT('')
GO

-- Versions
ALTER TABLE gemini_versions DROP CONSTRAINT gemini_versions_versionreleased_def 
EXEC sp_rename 'gemini_versions.versionreleased', 'zversionreleased', 'column'
GO
ALTER TABLE gemini_versions ADD versionreleased BIT NOT NULL CONSTRAINT gemini_versions_versionreleased_def DEFAULT (0)
GO
UPDATE gemini_versions SET versionreleased = CASE WHEN zversionreleased COLLATE DATABASE_DEFAULT = 'Y' COLLATE DATABASE_DEFAULT THEN 1 ELSE 0 END
GO
ALTER TABLE gemini_versions DROP COLUMN zversionreleased 
GO

--Components
ALTER TABLE gemini_components DROP CONSTRAINT gemini_components_componentreadonly_def
EXEC sp_rename 'gemini_components.componentreadonly', 'zcomponentreadonly', 'column'
GO
ALTER TABLE gemini_components ADD componentreadonly BIT NOT NULL CONSTRAINT gemini_components_componentreadonly_def DEFAULT(0)
GO
UPDATE gemini_components SET componentreadonly = CASE WHEN zcomponentreadonly COLLATE DATABASE_DEFAULT = 'Y' COLLATE DATABASE_DEFAULT THEN 1 ELSE 0 END
GO
ALTER TABLE gemini_components DROP COLUMN zcomponentreadonly
GO

-- Users
ALTER TABLE gemini_users ADD openid NVARCHAR(500) NOT NULL CONSTRAINT gemini_users_openid_def DEFAULT('')
ALTER TABLE gemini_users ADD timezoneid VARCHAR(100) NOT NULL CONSTRAINT gemini_users_timezoneid_def DEFAULT('UTC')
ALTER TABLE gemini_users ADD culture VARCHAR(50) NOT NULL CONSTRAINT gemini_users_culture_def DEFAULT('')
ALTER TABLE gemini_users ADD languageid VARCHAR(50) NOT NULL CONSTRAINT gemini_users_languageid_def DEFAULT('en-US')
ALTER TABLE gemini_users ADD theme VARCHAR(50) NOT NULL CONSTRAINT gemini_users_theme_def DEFAULT('')
ALTER TABLE gemini_users ADD emailme bit NOT NULL CONSTRAINT gemini_users_emailme_def DEFAULT(1)
ALTER TABLE gemini_users ADD emailmemychanges bit NOT NULL CONSTRAINT gemini_users_emailmemychanges_def DEFAULT(1)
ALTER TABLE gemini_users ADD settings NVARCHAR(MAX) NOT NULL CONSTRAINT gemini_users_settings_def DEFAULT('')
GO

-- Issues
ALTER TABLE gemini_issues ADD loggedhours INT NOT NULL CONSTRAINT gemini_issues_loggedhours_def DEFAULT (0)
ALTER TABLE gemini_issues ADD loggedminutes INT NOT NULL CONSTRAINT gemini_issues_loggedminutes_def DEFAULT (0)
ALTER TABLE gemini_issues ADD affectedversions VARCHAR(1000) NOT NULL CONSTRAINT gemini_issues_affectedversions_def DEFAULT ('')
ALTER TABLE gemini_issues ADD resources VARCHAR(1000) NOT NULL CONSTRAINT gemini_issues_resources_def DEFAULT ('')
ALTER TABLE gemini_issues ADD components VARCHAR(1000) NOT NULL CONSTRAINT gemini_issues_components_def DEFAULT ('')
ALTER TABLE gemini_issues ADD votes INT NOT NULL CONSTRAINT gemini_issues_votes_def DEFAULT (0)
ALTER TABLE gemini_issues ADD mailboxid NUMERIC(10,0) NULL 
ALTER TABLE gemini_issues ADD watchers VARCHAR(1000) NOT NULL CONSTRAINT gemini_issues_watchers_def DEFAULT ('')
ALTER TABLE gemini_issues ADD points INT NOT NULL CONSTRAINT gemini_issues_points_def DEFAULT (0)
GO

ALTER TABLE gemini_testing_history ALTER COLUMN history NVARCHAR(MAX) NOT NULL
GO
ALTER TABLE gemini_testing_run_cases ADD caseorder INT NULL
GO	

SP_RENAME 'gemini_issuetimetype.timetypeID','timetypeid','COLUMN'
GO

-- Attach Project Templates to tables
ALTER TABLE gemini_projects ADD templateid INT NOT NULL CONSTRAINT gemini_projects_templateid_def DEFAULT(1) CONSTRAINT gemini_projects_templateid_fk FOREIGN KEY REFERENCES gemini_projecttemplate(templateid)
ALTER TABLE gemini_issuetypes ADD templateid INT NOT NULL CONSTRAINT gemini_issuetypes_templateid_def DEFAULT(1) CONSTRAINT gemini_issuetypes_templateid_fk FOREIGN KEY REFERENCES gemini_projecttemplate(templateid)
ALTER TABLE gemini_issuepriorities ADD templateid INT NOT NULL CONSTRAINT gemini_issuepriorities_templateid_def DEFAULT(1) CONSTRAINT gemini_issuepriorities_templateid_fk FOREIGN KEY REFERENCES gemini_projecttemplate(templateid)
ALTER TABLE gemini_issueseverity ADD templateid INT NOT NULL CONSTRAINT gemini_issueseverity_templateid_def DEFAULT(1) CONSTRAINT gemini_issueseverity_templateid_fk FOREIGN KEY REFERENCES gemini_projecttemplate(templateid)
ALTER TABLE gemini_issuestatus ADD templateid INT NOT NULL CONSTRAINT gemini_issuestatus_templateid_def DEFAULT(1) CONSTRAINT gemini_issuestatus_templateid_fk FOREIGN KEY REFERENCES gemini_projecttemplate(templateid)
ALTER TABLE gemini_customfielddefinitions ADD templateid INT NOT NULL CONSTRAINT gemini_customfielddefinitions_templateid_def DEFAULT(1) CONSTRAINT gemini_customfielddefinitions_templateid_fk FOREIGN KEY REFERENCES gemini_projecttemplate(templateid)
ALTER TABLE gemini_issueresolutions ADD templateid INT NOT NULL CONSTRAINT gemini_issuereolutions_templateid_def DEFAULT(1) CONSTRAINT gemini_issueresolutions_templateid_fk FOREIGN KEY REFERENCES gemini_projecttemplate(templateid)
ALTER TABLE gemini_issuelinktypes ADD templateid INT NOT NULL CONSTRAINT gemini_issuelinktypes_templateid_def DEFAULT(1) CONSTRAINT gemini_issuelinktypes_templateid_fk FOREIGN KEY REFERENCES gemini_projecttemplate(templateid)
ALTER TABLE gemini_issuetimetype ADD templateid INT NOT NULL CONSTRAINT gemini_issuetimetype_templateid_def DEFAULT(1) CONSTRAINT gemini_issuetimetype_templateid_fk FOREIGN KEY REFERENCES gemini_projecttemplate(templateid)
GO

ALTER TABLE gemini_projectgroups ADD interactiongroups VARCHAR(MAX) NULL
GO

ALTER TABLE gemini_userissuesviews ADD appid VARCHAR(38) NULL
ALTER TABLE gemini_userissuesviews ADD controlid VARCHAR(38) NULL
ALTER TABLE gemini_userissuesviews ALTER COLUMN issueattribute INT NULL
GO

ALTER TABLE gemini_projectdefaultvalues ADD typeid NUMERIC(10,0) NULL CONSTRAINT gemini_projectdefaultvalues_typeid_fk FOREIGN KEY REFERENCES gemini_issuetypes(typeid)
GO
ALTER TABLE gemini_projectdefaultvalues ADD defaultvalues NVARCHAR(MAX) NOT NULL CONSTRAINT gemini_projectdefaultvalues_defaultvalues_def DEFAULT('')
GO
ALTER TABLE gemini_projectdefaultvalues DROP CONSTRAINT gemini_projectdefaultvalues_fieldid_def
GO

ALTER TABLE gemini_customfielddata ADD revised DATETIME NOT NULL CONSTRAINT gemini_customfielddata_revised_def DEFAULT GETUTCDATE()
GO

ALTER TABLE gemini_customfielddefinitions ADD canfilter BIT NOT NULL CONSTRAINT gemini_customfielddefinitions_canfilter_def DEFAULT 0
GO
ALTER TABLE gemini_customfielddefinitions ADD usestaticdata BIT NOT NULL CONSTRAINT gemini_customfielddefinitions_usestaticdata_def DEFAULT 0
GO
/**********************************************************************************************************************
* End
**********************************************************************************************************************/

/**********************************************************************************************************************
* INSERT PROJECT TEMPLATE DATA
**********************************************************************************************************************/
SET IDENTITY_INSERT gemini_projecttemplate ON
INSERT gemini_projecttemplate(templateid,templatekey,title,descrip,author,published,versionnumber,template) VALUES('10','AGILE','Agile Development Process','Standard Agile and Scrum project template','Countersoft',convert(datetime,'2012-07-03 11:54:07.000',121),'1.0.0','{"Name":"Agile Development Process","Description":"Standard Agile and Scrum project template","Publisher":"Countersoft","Published":"2012-07-03T11:54:07.9423439+01:00","Version":"1.0.0","HomePage":2,"TemplateKey":"AGILE","Menu":[{"PageType":2,"SeperateAfter":false,"ProjectGroups":[1],"Sequence":1},{"PageType":5,"SeperateAfter":false,"ProjectGroups":[1],"Sequence":2},{"PageType":1,"SeperateAfter":true,"ProjectGroups":[1],"Sequence":3},{"PageType":3,"SeperateAfter":false,"ProjectGroups":[1],"Sequence":4},{"PageType":4,"SeperateAfter":true,"ProjectGroups":[1],"Sequence":5},{"PageType":20,"SeperateAfter":false,"ProjectGroups":[1],"Sequence":6},{"PageType":8,"SeperateAfter":false,"ProjectGroups":[1],"Sequence":7}],"CustomField":[{"Label":"Customer","ScreenLabel":"Customer","ScreenDescription":"Our Client name","ScreenTooltip":"Select the customer from the list","MaxLen":0,"CanMultiSelect":true,"RegularExpression":"","Type":"C","ProjectIdFilter":true,"ShowInAttributes":true,"LookupStaticData":"Customer A\r\nCustomer B\r\nCustomer C\r\n\r\n\r\n\r\n                        \r\n                    ","MaxValue":0.0,"MinValue":0.0,"ListLimiter":""},{"Label":"Customer Backlog ID","ScreenLabel":"Customer Backlog ID","ScreenDescription":"","ScreenTooltip":"Customer Backlog ID","MaxLen":12,"CanMultiSelect":false,"RegularExpression":"","Type":"C","ProjectIdFilter":false,"ShowInAttributes":true,"LookupStaticData":"BACKLOG A\r\nBACKLOG B\r\nBACKLOG C\r\nBACKLOG D\r\nBACKLOG E","MaxValue":999.0,"MinValue":0.0,"ListLimiter":""}],"Priority":[{"Label":"Low","Image":"priority-low.png","Color":"#0000ff"},{"Label":"Medium","Image":"priority-medium.png","Color":"#ffffff"},{"Label":"High","Image":"priority-high.png","Color":"#eb0c1f"}],"Severity":[{"Label":"Trivial","Image":"severity-trivial.png","Color":""},{"Label":"Minor","Image":"severity-minor.png","Color":"#0000ff"},{"Label":"Major","Image":"severity-major.png","Color":"#8c054d"}],"Status":[{"Final":false,"Label":"Approved","Image":"status-approved.png","Color":"#36a820"},{"Final":false,"Label":"In Backlog","Image":"test-blackbox.png","Color":"#3d213d"},{"Final":false,"Label":"In Sprint","Image":"test-user.png","Color":"#337a0f"},{"Final":false,"Label":"Unassigned","Image":"status-unassigned.png","Color":"#0000ff"},{"Final":false,"Label":"Assigned","Image":"status-assigned.png","Color":"#47d119"},{"Final":false,"Label":"In Progress","Image":"status-inprogress.png","Color":"#a130a1"},{"Final":true,"Label":"Closed","Image":"status-closed.png","Color":"#ffffff"},{"Final":false,"Label":"Reopened","Image":"status-reopened.png","Color":"#0a1c91"}],"Resolution":[{"Final":true,"Label":"Parked - No Further Action"},{"Final":true,"Label":"Complete"}],"Link":[{"Label":"Duplicated"},{"Label":"Related"}],"Time":[{"Label":"Billable"},{"Label":"Non-Billable"},{"Label":"Internal"},{"Label":"Scrum Master PM"},{"Label":"Testing"}],"Process":[{"TheScreen":{"UseFromProcessRef":"","Item":[{"InternalField":8,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":3,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":2,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":0,"IsExternalCustomField":true,"IsExternalWidgetField":false,"ExternalFieldRef":"23","CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":0,"IsExternalCustomField":true,"IsExternalWidgetField":false,"ExternalFieldRef":"25","CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":9,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":15,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":16,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":17,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":1,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":42,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":14,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":38,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":23,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":35,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":10,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"Status","CreateScreen":false,"EditScreen":true,"ViewScreen":true},{"InternalField":36,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedComments","CreateScreen":false,"EditScreen":true,"ViewScreen":true},{"InternalField":37,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedHistory","CreateScreen":false,"EditScreen":true,"ViewScreen":true},{"InternalField":30,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"CalculatedTimeLogged","CreateScreen":false,"EditScreen":true,"ViewScreen":true},{"InternalField":5,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"Visibility","CreateScreen":false,"EditScreen":true,"ViewScreen":false},{"InternalField":18,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"DateCreated","CreateScreen":false,"EditScreen":false,"ViewScreen":true},{"InternalField":33,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedTime","CreateScreen":false,"EditScreen":false,"ViewScreen":true},{"InternalField":40,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedHierarchy","CreateScreen":false,"EditScreen":false,"ViewScreen":true},{"InternalField":39,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedLinks","CreateScreen":false,"EditScreen":false,"ViewScreen":true},{"InternalField":34,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedSourceControl","CreateScreen":false,"EditScreen":false,"ViewScreen":true}]},"TheWorkflow":{"UseFromProcessRef":"","Item":[{"StatusFieldRef":"Approved","Top":"10","Left":"10","TransitionStatusFieldRef":[]},{"StatusFieldRef":"In Backlog","Top":"10","Left":"160","TransitionStatusFieldRef":["In Sprint"]},{"StatusFieldRef":"In Sprint","Top":"375","Left":"45","TransitionStatusFieldRef":["Unassigned","Assigned","In Progress","In Backlog"]},{"StatusFieldRef":"Unassigned","Top":"334","Left":"308","TransitionStatusFieldRef":["Assigned","In Progress"]},{"StatusFieldRef":"Assigned","Top":"564","Left":"423","TransitionStatusFieldRef":["In Progress","Unassigned"]},{"StatusFieldRef":"In Progress","Top":"10","Left":"760","TransitionStatusFieldRef":["Closed"]},{"StatusFieldRef":"Closed","Top":"523","Left":"726","TransitionStatusFieldRef":["Reopened"]},{"StatusFieldRef":"Reopened","Top":"206","Left":"10","TransitionStatusFieldRef":["In Backlog"]}]},"Label":"Requirement","Image":"type-enhancement.png","Color":"#e612e6","Tag":""},{"TheScreen":{"UseFromProcessRef":"Requirement","Item":[{"InternalField":14,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":0,"IsExternalCustomField":true,"IsExternalWidgetField":false,"ExternalFieldRef":"6","CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":23,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":21,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":0,"IsExternalCustomField":true,"IsExternalWidgetField":false,"ExternalFieldRef":"4","CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":13,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":18,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":0,"IsExternalCustomField":false,"IsExternalWidgetField":true,"ExternalFieldRef":"0B2B3E39-F37B-489F-AFA8-0B6C3FA63CFB","CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":17,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":15,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":12,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":1,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":42,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":9,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":25,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":6,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":11,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":20,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":7,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":19,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":4,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":24,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":16,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":10,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":32,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":30,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":31,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":3,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":0,"IsExternalCustomField":false,"IsExternalWidgetField":true,"ExternalFieldRef":"878C76CF-E9AE-46B4-BE05-7CC2EA5FB574","CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":8,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":5,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":22,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":2,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":36,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":0,"IsExternalCustomField":false,"IsExternalWidgetField":true,"ExternalFieldRef":"598B4999-4C31-427D-9BBC-C84AD954E55A","CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":38,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":0,"IsExternalCustomField":false,"IsExternalWidgetField":true,"ExternalFieldRef":"C8F0693D-129C-4D19-9865-9CED964F06AB","CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":0,"IsExternalCustomField":true,"IsExternalWidgetField":false,"ExternalFieldRef":"11","CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":40,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":37,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":0,"IsExternalCustomField":false,"IsExternalWidgetField":true,"ExternalFieldRef":"74A39750-6C8F-4023-82D2-1F1F10FF4E70","CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":0,"IsExternalCustomField":false,"IsExternalWidgetField":true,"ExternalFieldRef":"28FE1D91-7730-4067-8A10-F452FCB9D090","CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":0,"IsExternalCustomField":true,"IsExternalWidgetField":false,"ExternalFieldRef":"18","CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":0,"IsExternalCustomField":false,"IsExternalWidgetField":true,"ExternalFieldRef":"617F1435-C0AA-48E6-B6A9-2C1F756E02A6","CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":39,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":0,"IsExternalCustomField":false,"IsExternalWidgetField":true,"ExternalFieldRef":"AC93C4F9-B512-4A3D-8B37-BC3D5F83429D","CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":34,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":0,"IsExternalCustomField":false,"IsExternalWidgetField":true,"ExternalFieldRef":"ACE2F8B3-C724-444F-9B72-B8B2137DDA16","CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":0,"IsExternalCustomField":false,"IsExternalWidgetField":true,"ExternalFieldRef":"EFEE5D50-7DF6-49C3-9D69-E7EFC1AFEF72","CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":33,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":35,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true}]},"TheWorkflow":{"UseFromProcessRef":"Requirement","Item":[]},"Label":"Story","Image":"type-newfeature.png","Color":"#0a5e85","Tag":""},{"TheScreen":{"UseFromProcessRef":"","Item":[{"InternalField":8,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":false,"ViewScreen":false},{"InternalField":3,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":2,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":10,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":16,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":17,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":30,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":38,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":36,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedComments","CreateScreen":false,"EditScreen":true,"ViewScreen":true},{"InternalField":33,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedTime","CreateScreen":false,"EditScreen":false,"ViewScreen":true}]},"TheWorkflow":{"UseFromProcessRef":"","Item":[{"StatusFieldRef":"In Backlog","Top":"10","Left":"10","TransitionStatusFieldRef":[]},{"StatusFieldRef":"In Sprint","Top":"10","Left":"160","TransitionStatusFieldRef":[]},{"StatusFieldRef":"Unassigned","Top":"0","Left":"0","TransitionStatusFieldRef":["Assigned"]},{"StatusFieldRef":"Assigned","Top":"10","Left":"160","TransitionStatusFieldRef":["In Progress"]},{"StatusFieldRef":"In Progress","Top":"10","Left":"310","TransitionStatusFieldRef":["Closed"]},{"StatusFieldRef":"Closed","Top":"10","Left":"460","TransitionStatusFieldRef":[]},{"StatusFieldRef":"Reopened","Top":"80","Left":"10","TransitionStatusFieldRef":[]}]},"Label":"Task","Image":"type-task.png","Color":"#e68012","Tag":""},{"TheScreen":{"UseFromProcessRef":"","Item":[{"InternalField":8,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":false,"ViewScreen":true},{"InternalField":17,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":15,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":9,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":5,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":false,"ViewScreen":false},{"InternalField":22,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":2,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":38,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":0,"IsExternalCustomField":true,"IsExternalWidgetField":false,"ExternalFieldRef":"23","CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":0,"IsExternalCustomField":true,"IsExternalWidgetField":false,"ExternalFieldRef":"25","CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":36,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedComments","CreateScreen":false,"EditScreen":true,"ViewScreen":true},{"InternalField":40,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedHierarchy","CreateScreen":false,"EditScreen":true,"ViewScreen":true},{"InternalField":37,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedHistory","CreateScreen":false,"EditScreen":true,"ViewScreen":true},{"InternalField":39,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedLinks","CreateScreen":false,"EditScreen":false,"ViewScreen":true},{"InternalField":35,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedWatchers","CreateScreen":false,"EditScreen":false,"ViewScreen":true}]},"TheWorkflow":{"UseFromProcessRef":"","Item":[{"StatusFieldRef":"Approved","Top":"0","Left":"0","TransitionStatusFieldRef":["In Backlog"]},{"StatusFieldRef":"In Backlog","Top":"0","Left":"0","TransitionStatusFieldRef":["In Sprint"]},{"StatusFieldRef":"In Sprint","Top":"0","Left":"0","TransitionStatusFieldRef":["Unassigned","Assigned","In Progress"]},{"StatusFieldRef":"Unassigned","Top":"10","Left":"460","TransitionStatusFieldRef":["Assigned"]},{"StatusFieldRef":"Assigned","Top":"10","Left":"610","TransitionStatusFieldRef":["In Progress"]},{"StatusFieldRef":"In Progress","Top":"10","Left":"760","TransitionStatusFieldRef":["Closed"]},{"StatusFieldRef":"Closed","Top":"0","Left":"0","TransitionStatusFieldRef":["Reopened"]},{"StatusFieldRef":"Reopened","Top":"0","Left":"0","TransitionStatusFieldRef":["In Backlog"]}]},"Label":"Change Request","Image":"type-changerequest.png","Color":"#171517","Tag":""},{"TheScreen":{"UseFromProcessRef":"","Item":[{"InternalField":8,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":3,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":2,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":16,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":17,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":9,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":4,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":10,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":13,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":23,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":21,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":15,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":12,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":1,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":42,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":6,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":11,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":7,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":0,"IsExternalCustomField":true,"IsExternalWidgetField":false,"ExternalFieldRef":"23","CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":0,"IsExternalCustomField":true,"IsExternalWidgetField":false,"ExternalFieldRef":"25","CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":24,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":30,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":5,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":false},{"InternalField":38,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":40,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":39,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":34,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":35,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":14,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"PercentComplete","CreateScreen":false,"EditScreen":true,"ViewScreen":true},{"InternalField":36,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedComments","CreateScreen":false,"EditScreen":true,"ViewScreen":true},{"InternalField":33,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedTime","CreateScreen":false,"EditScreen":true,"ViewScreen":true},{"InternalField":18,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"DateCreated","CreateScreen":false,"EditScreen":false,"ViewScreen":true},{"InternalField":19,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"DateRevised","CreateScreen":false,"EditScreen":false,"ViewScreen":true},{"InternalField":37,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedHistory","CreateScreen":false,"EditScreen":false,"ViewScreen":true}]},"TheWorkflow":{"UseFromProcessRef":"Requirement","Item":[]},"Label":"Bug","Image":"type-bug.png","Color":"#ff0000","Tag":""}],"IsNew":true,"IsExisting":false,"Active":true,"Archived":false,"Deleted":false,"Created":"2012-07-03T10:54:06.1482413Z","Revised":"2012-07-03T10:54:06.1482413Z","Errors":[],"HasErrors":false,"Id":0}')
INSERT gemini_projecttemplate(templateid,templatekey,title,descrip,author,published,versionnumber,template) VALUES('11','ENQUIRY','Customer support and enquiry management','Standard Customer Support and Enquiry management project template','Countersoft',convert(datetime,'2012-07-03 11:54:07.000',121),'1.0.0','{"Name":"Customer support and enquiry management","Description":"Standard Customer Support and Enquiry management project template","Publisher":"Countersoft","Published":"2012-07-03T11:54:07.9423439+01:00","Version":"1.0.0","HomePage":2,"TemplateKey":"ENQUIRY","Menu":[{"PageType":1,"SeperateAfter":false,"ProjectGroups":[1],"Sequence":1},{"PageType":2,"SeperateAfter":true,"ProjectGroups":[1],"Sequence":2},{"PageType":9,"SeperateAfter":false,"ProjectGroups":[1],"Sequence":3},{"PageType":20,"SeperateAfter":false,"ProjectGroups":[1],"Sequence":4},{"PageType":19,"SeperateAfter":true,"ProjectGroups":[1],"Sequence":5},{"PageType":8,"SeperateAfter":false,"ProjectGroups":[1],"Sequence":6},{"PageType":7,"SeperateAfter":false,"ProjectGroups":[1],"Sequence":7}],"CustomField":[{"Label":"Customer","ScreenLabel":"Customer","ScreenDescription":"","ScreenTooltip":"Select the customer from the list","MaxLen":0,"CanMultiSelect":true,"RegularExpression":"","Type":"L","ProjectIdFilter":true,"ShowInAttributes":true,"LookupStaticData":"Customer A\r\nCustomer B\r\nCustomer C\r\n\r\n\r\n\r\n                        \r\n                    ","MaxValue":0.0,"MinValue":0.0,"ListLimiter":""},{"Label":"Product Platform","ScreenLabel":"Product Platform (OS)","ScreenDescription":"","ScreenTooltip":"Product Platform (OS)","MaxLen":0,"CanMultiSelect":false,"RegularExpression":"","Type":"C","ProjectIdFilter":false,"ShowInAttributes":true,"LookupStaticData":"Windows 7\r\nWindows 8\r\nLinux\r\nUbuntu\r\nMac OS\r\niPad\r\n\r\n\r\n\r\n                        \r\n                    ","MaxValue":0.0,"MinValue":0.0,"ListLimiter":""},{"Label":"Needs Approval","ScreenLabel":"Needs Approval","ScreenDescription":"","ScreenTooltip":"Needs Approval","MaxLen":0,"CanMultiSelect":false,"RegularExpression":"","Type":"Y","ProjectIdFilter":false,"ShowInAttributes":true,"LookupStaticData":"                        \r\n                    ","MaxValue":0.0,"MinValue":0.0,"ListLimiter":""}],"Priority":[{"Label":"Low","Image":"priority-low.png","Color":"#0000ff"},{"Label":"Medium","Image":"priority-medium.png","Color":"#ffffff"},{"Label":"High","Image":"priority-high.png","Color":"#eb0c1f"}],"Severity":[{"Label":"Trivial","Image":"severity-trivial.png","Color":"#1a6145"},{"Label":"Minor","Image":"severity-minor.png","Color":"#0000ff"},{"Label":"Major","Image":"severity-major.png","Color":"#8c054d"},{"Label":"Critical","Image":"severity-showstopper.png","Color":"#fa0021"}],"Status":[{"Final":false,"Label":"Escalation-Level1","Image":"test-requirements.png","Color":"#bf6a1f"},{"Final":false,"Label":"Escalation-Level2","Image":"test-stress.png","Color":"#f50a0a"},{"Final":false,"Label":"Raised","Image":"type-newfeature.png","Color":"#a0c218"},{"Final":false,"Label":"User Rejected","Image":"status-rejected.png","Color":"#754b0d"},{"Final":false,"Label":"Moved to Delivery","Image":"test-blackbox.png","Color":"#7a12a3"},{"Final":false,"Label":"Assigned","Image":"status-assigned.png","Color":"#47d119"},{"Final":true,"Label":"Closed","Image":"status-closed.png","Color":"#0f0907"},{"Final":false,"Label":"Case Reopened","Image":"status-reopened.png","Color":"#0a1c91"},{"Final":false,"Label":"User Accepted","Image":"test-acceptance.png","Color":"#541454"},{"Final":false,"Label":"Pending Approval","Image":"status-ready-for-approval.png","Color":"#787167"},{"Final":false,"Label":"Approved","Image":"status-approved.png","Color":"#099e38"}],"Resolution":[{"Final":true,"Label":"Back to Client"},{"Final":true,"Label":"Complete"}],"Link":[{"Label":"Duplicated"},{"Label":"Related"}],"Time":[{"Label":"Billable"},{"Label":"Non-Billable"},{"Label":"Internal"}],"Process":[{"TheScreen":{"UseFromProcessRef":"","Item":[{"InternalField":8,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":3,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":2,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":38,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":9,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":10,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":5,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":false},{"InternalField":22,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":13,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":1,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":36,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedComments","CreateScreen":false,"EditScreen":true,"ViewScreen":true},{"InternalField":37,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedHistory","CreateScreen":false,"EditScreen":false,"ViewScreen":true}]},"TheWorkflow":{"UseFromProcessRef":"","Item":[{"StatusFieldRef":"Escalation-Level1","Top":"122","Left":"284","TransitionStatusFieldRef":["Escalation-Level2"]},{"StatusFieldRef":"Escalation-Level2","Top":"33","Left":"163","TransitionStatusFieldRef":[]},{"StatusFieldRef":"Moved to Delivery","Top":"10","Left":"10","TransitionStatusFieldRef":[]},{"StatusFieldRef":"Raised","Top":"299","Left":"58","TransitionStatusFieldRef":["Assigned","Approved"]},{"StatusFieldRef":"User Rejected","Top":"82","Left":"494","TransitionStatusFieldRef":[]},{"StatusFieldRef":"Assigned","Top":"308","Left":"287","TransitionStatusFieldRef":["Closed","Escalation-Level1"]},{"StatusFieldRef":"Closed","Top":"226","Left":"465","TransitionStatusFieldRef":["Case Reopened"]},{"StatusFieldRef":"Case Reopened","Top":"431","Left":"443","TransitionStatusFieldRef":["Assigned"]},{"StatusFieldRef":"User Accepted","Top":"10","Left":"610","TransitionStatusFieldRef":[]},{"StatusFieldRef":"Pending Approval","Top":"10","Left":"760","TransitionStatusFieldRef":[]},{"StatusFieldRef":"Approved","Top":"454","Left":"184","TransitionStatusFieldRef":["Assigned"]}]},"Label":"Enquiry","Image":"type-investigation.png","Color":"#ffd000","Tag":""},{"TheScreen":{"UseFromProcessRef":"","Item":[{"InternalField":8,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":3,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":2,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":4,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":9,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":10,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":16,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":17,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":21,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":15,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":12,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":1,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":42,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":11,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":7,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":24,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":30,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":5,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":false},{"InternalField":38,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":40,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":39,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":34,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":35,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":14,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"PercentComplete","CreateScreen":false,"EditScreen":true,"ViewScreen":true},{"InternalField":36,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedComments","CreateScreen":false,"EditScreen":true,"ViewScreen":true},{"InternalField":33,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedTime","CreateScreen":false,"EditScreen":true,"ViewScreen":true},{"InternalField":18,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"DateCreated","CreateScreen":false,"EditScreen":false,"ViewScreen":true},{"InternalField":19,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"DateRevised","CreateScreen":false,"EditScreen":false,"ViewScreen":true},{"InternalField":37,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedHistory","CreateScreen":false,"EditScreen":false,"ViewScreen":true}]},"TheWorkflow":{"UseFromProcessRef":"","Item":[{"StatusFieldRef":"Escalation-Level1","Top":"40","Left":"376","TransitionStatusFieldRef":[]},{"StatusFieldRef":"Escalation-Level2","Top":"41","Left":"196","TransitionStatusFieldRef":[]},{"StatusFieldRef":"Moved to Delivery","Top":"10","Left":"10","TransitionStatusFieldRef":[]},{"StatusFieldRef":"Raised","Top":"185","Left":"62","TransitionStatusFieldRef":["Assigned","Pending Approval"]},{"StatusFieldRef":"User Rejected","Top":"63","Left":"631","TransitionStatusFieldRef":["Assigned"]},{"StatusFieldRef":"Assigned","Top":"186","Left":"397","TransitionStatusFieldRef":["Closed","User Accepted"]},{"StatusFieldRef":"Closed","Top":"341","Left":"674","TransitionStatusFieldRef":["Case Reopened"]},{"StatusFieldRef":"Case Reopened","Top":"508","Left":"296","TransitionStatusFieldRef":["Pending Approval"]},{"StatusFieldRef":"User Accepted","Top":"188","Left":"687","TransitionStatusFieldRef":["Closed","User Rejected"]},{"StatusFieldRef":"Pending Approval","Top":"301","Left":"218","TransitionStatusFieldRef":["Approved"]},{"StatusFieldRef":"Approved","Top":"344","Left":"437","TransitionStatusFieldRef":["Assigned"]}]},"Label":"Open Issue","Image":"file-text.png","Color":"#3300ff","Tag":""},{"TheScreen":{"UseFromProcessRef":"","Item":[{"InternalField":8,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":3,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":2,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":9,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":0,"IsExternalCustomField":true,"IsExternalWidgetField":false,"ExternalFieldRef":"39","CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":0,"IsExternalCustomField":true,"IsExternalWidgetField":false,"ExternalFieldRef":"40","CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":36,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":15,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":16,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":17,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":38,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":35,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":5,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":false},{"InternalField":40,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":10,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"Status","CreateScreen":false,"EditScreen":true,"ViewScreen":true},{"InternalField":1,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"IssueKey","CreateScreen":false,"EditScreen":false,"ViewScreen":true},{"InternalField":37,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedHistory","CreateScreen":false,"EditScreen":false,"ViewScreen":true},{"InternalField":39,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedLinks","CreateScreen":false,"EditScreen":false,"ViewScreen":true},{"InternalField":34,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedSourceControl","CreateScreen":false,"EditScreen":false,"ViewScreen":true}]},"TheWorkflow":{"UseFromProcessRef":"","Item":[{"StatusFieldRef":"Escalation-Level1","Top":"10","Left":"10","TransitionStatusFieldRef":[]},{"StatusFieldRef":"Escalation-Level2","Top":"10","Left":"160","TransitionStatusFieldRef":[]},{"StatusFieldRef":"Moved to Delivery","Top":"437","Left":"709","TransitionStatusFieldRef":["Closed"]},{"StatusFieldRef":"Raised","Top":"194","Left":"62","TransitionStatusFieldRef":["Pending Approval"]},{"StatusFieldRef":"User Rejected","Top":"152","Left":"441","TransitionStatusFieldRef":[]},{"StatusFieldRef":"Assigned","Top":"281","Left":"620","TransitionStatusFieldRef":["Closed","Moved to Delivery"]},{"StatusFieldRef":"Closed","Top":"431","Left":"439","TransitionStatusFieldRef":[]},{"StatusFieldRef":"Case Reopened","Top":"86","Left":"229","TransitionStatusFieldRef":[]},{"StatusFieldRef":"User Accepted","Top":"94","Left":"30","TransitionStatusFieldRef":[]},{"StatusFieldRef":"Pending Approval","Top":"316","Left":"143","TransitionStatusFieldRef":["Approved"]},{"StatusFieldRef":"Approved","Top":"265","Left":"391","TransitionStatusFieldRef":["Assigned"]}]},"Label":"New Requirement","Image":"type-enhancement.png","Color":"#e612e6","Tag":""},{"TheScreen":{"UseFromProcessRef":"","Item":[{"InternalField":8,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":false,"ViewScreen":false},{"InternalField":3,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":2,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":16,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":10,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":17,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":30,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":38,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":36,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":33,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedTime","CreateScreen":false,"EditScreen":false,"ViewScreen":true}]},"TheWorkflow":{"UseFromProcessRef":"","Item":[{"StatusFieldRef":"Escalation-Level1","Top":"24","Left":"37","TransitionStatusFieldRef":[]},{"StatusFieldRef":"Escalation-Level2","Top":"10","Left":"245","TransitionStatusFieldRef":[]},{"StatusFieldRef":"Moved to Delivery","Top":"198","Left":"478","TransitionStatusFieldRef":["User Accepted","User Rejected"]},{"StatusFieldRef":"Raised","Top":"402","Left":"88","TransitionStatusFieldRef":["Assigned","Approved","Pending Approval"]},{"StatusFieldRef":"User Rejected","Top":"142","Left":"275","TransitionStatusFieldRef":["Assigned"]},{"StatusFieldRef":"Assigned","Top":"298","Left":"378","TransitionStatusFieldRef":["Closed","Moved to Delivery"]},{"StatusFieldRef":"Closed","Top":"287","Left":"589","TransitionStatusFieldRef":[]},{"StatusFieldRef":"Case Reopened","Top":"80","Left":"160","TransitionStatusFieldRef":[]},{"StatusFieldRef":"User Accepted","Top":"139","Left":"652","TransitionStatusFieldRef":["Closed"]},{"StatusFieldRef":"Pending Approval","Top":"281","Left":"20","TransitionStatusFieldRef":["Approved"]},{"StatusFieldRef":"Approved","Top":"208","Left":"132","TransitionStatusFieldRef":["Assigned"]}]},"Label":"Task","Image":"type-task.png","Color":"#e68012","Tag":""}],"IsNew":true,"IsExisting":false,"Active":true,"Archived":false,"Deleted":false,"Created":"2012-07-03T10:54:06.1482413Z","Revised":"2012-07-03T10:54:06.1482413Z","Errors":[],"HasErrors":false,"Id":0}')
INSERT gemini_projecttemplate(templateid,templatekey,title,descrip,author,published,versionnumber,template) VALUES('12','HELPDESK','Help Desk Ticketing','Standard IT help desk ticketing project template','Countersoft',convert(datetime,'2012-07-03 11:54:07.000',121),'1.0.0','{"Name":"Help Desk Ticketing","Description":"Standard IT help desk ticketing project template","Publisher":"Countersoft","Published":"2012-07-03T11:54:07.9423439+01:00","Version":"1.0.0","HomePage":2,"TemplateKey":"HELPDESK","Menu":[{"PageType":1,"SeperateAfter":true,"ProjectGroups":[1],"Sequence":1},{"PageType":20,"SeperateAfter":false,"ProjectGroups":[1],"Sequence":2},{"PageType":19,"SeperateAfter":false,"ProjectGroups":[1],"Sequence":3},{"PageType":9,"SeperateAfter":false,"ProjectGroups":[1],"Sequence":4},{"PageType":8,"SeperateAfter":false,"ProjectGroups":[1],"Sequence":5},{"PageType":7,"SeperateAfter":false,"ProjectGroups":[1],"Sequence":6}],"CustomField":[{"Label":"Customer","ScreenLabel":"Customer","ScreenDescription":"Our Client name","ScreenTooltip":"Select the customer from the list","MaxLen":0,"CanMultiSelect":true,"RegularExpression":"","Type":"L","ProjectIdFilter":true,"ShowInAttributes":true,"LookupStaticData":"Customer A\r\nCustomer B\r\nCustomer C\r\n                        \r\n                    ","MaxValue":0.0,"MinValue":0.0,"ListLimiter":""},{"Label":"User Application","ScreenLabel":"User Application","ScreenDescription":"","ScreenTooltip":"User Application","MaxLen":0,"CanMultiSelect":false,"RegularExpression":"","Type":"C","ProjectIdFilter":false,"ShowInAttributes":true,"LookupStaticData":"Peoplesoft Financials\r\nPeoplesoft HR\r\nDynamics CRM\r\nOURSYS\r\nInventory Management\r\nOther","MaxValue":0.0,"MinValue":0.0,"ListLimiter":""},{"Label":"Product Platform","ScreenLabel":"Product Platform (OS)","ScreenDescription":"","ScreenTooltip":"Product Platform (OS)","MaxLen":0,"CanMultiSelect":false,"RegularExpression":"","Type":"C","ProjectIdFilter":false,"ShowInAttributes":true,"LookupStaticData":"Windows 7\r\nWindows 8\r\nLinux\r\nUbuntu\r\nMac OS\r\niPad\r\n\r\n                        \r\n                    ","MaxValue":0.0,"MinValue":0.0,"ListLimiter":""}],"Priority":[{"Label":"Low","Image":"priority-low.png","Color":"#0000ff"},{"Label":"Medium","Image":"priority-medium.png","Color":"#ffffff"},{"Label":"High","Image":"priority-high.png","Color":"#eb0c1f"}],"Severity":[{"Label":"Trivial","Image":"severity-trivial.png","Color":""},{"Label":"Minor","Image":"severity-minor.png","Color":"#0000ff"},{"Label":"Major","Image":"severity-major.png","Color":"#8c054d"}],"Status":[{"Final":false,"Label":"In Backlog","Image":"test-blackbox.png","Color":"#3d213d"},{"Final":false,"Label":"In Sprint","Image":"test-user.png","Color":"#337a0f"},{"Final":false,"Label":"Unassigned","Image":"status-unassigned.png","Color":"#0000ff"},{"Final":false,"Label":"Assigned","Image":"status-assigned.png","Color":"#47d119"},{"Final":false,"Label":"In Progress","Image":"status-inprogress.png","Color":"#a130a1"},{"Final":true,"Label":"Closed","Image":"status-closed.png","Color":"#ffffff"},{"Final":false,"Label":"Reopened","Image":"status-reopened.png","Color":"#0a1c91"},{"Final":false,"Label":"QA Approved","Image":"status-approved.png","Color":"#1b8f1b"},{"Final":false,"Label":"user Accepted","Image":"test-acceptance.png","Color":"#541454"},{"Final":false,"Label":"Approved","Image":"status-approved.png","Color":"#099e38"}],"Resolution":[{"Final":true,"Label":"Parked - No Further Action"},{"Final":true,"Label":"Complete"}],"Link":[{"Label":"Duplicated"},{"Label":"Related"}],"Time":[{"Label":"Billable"},{"Label":"Non-Billable"},{"Label":"Internal"},{"Label":"Scrum Master PM"},{"Label":"Testing"}],"Process":[{"TheScreen":{"UseFromProcessRef":"","Item":[{"InternalField":8,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":3,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":2,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":38,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":13,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":9,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":10,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":5,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":false},{"InternalField":22,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":36,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedComments","CreateScreen":false,"EditScreen":true,"ViewScreen":true}]},"TheWorkflow":{"UseFromProcessRef":"","Item":[{"StatusFieldRef":"QA Approved","Top":"10","Left":"10","TransitionStatusFieldRef":[]},{"StatusFieldRef":"Assigned","Top":"0","Left":"0","TransitionStatusFieldRef":["Unassigned","In Progress"]},{"StatusFieldRef":"Closed","Top":"10","Left":"310","TransitionStatusFieldRef":[]},{"StatusFieldRef":"In Backlog","Top":"10","Left":"460","TransitionStatusFieldRef":[]},{"StatusFieldRef":"In Progress","Top":"217","Left":"383","TransitionStatusFieldRef":["Closed"]},{"StatusFieldRef":"In Sprint","Top":"10","Left":"760","TransitionStatusFieldRef":[]},{"StatusFieldRef":"Reopened","Top":"80","Left":"10","TransitionStatusFieldRef":[]},{"StatusFieldRef":"Unassigned","Top":"0","Left":"0","TransitionStatusFieldRef":["Assigned"]}]},"Label":"Investigation","Image":"type-investigation.png","Color":"#ffd000","Tag":""},{"TheScreen":{"UseFromProcessRef":"","Item":[{"InternalField":8,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":3,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":2,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":0,"IsExternalCustomField":true,"IsExternalWidgetField":false,"ExternalFieldRef":"30","CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":4,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":16,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":17,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":9,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":10,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":13,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":23,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":21,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":15,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":12,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":1,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":42,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":11,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":7,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":24,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":30,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":5,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":false},{"InternalField":38,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":40,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":39,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":34,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":35,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":14,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"PercentComplete","CreateScreen":false,"EditScreen":true,"ViewScreen":true},{"InternalField":36,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedComments","CreateScreen":false,"EditScreen":true,"ViewScreen":true},{"InternalField":33,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedTime","CreateScreen":false,"EditScreen":true,"ViewScreen":true},{"InternalField":18,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"DateCreated","CreateScreen":false,"EditScreen":false,"ViewScreen":true},{"InternalField":19,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"DateRevised","CreateScreen":false,"EditScreen":false,"ViewScreen":true},{"InternalField":37,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedHistory","CreateScreen":false,"EditScreen":false,"ViewScreen":true}]},"TheWorkflow":{"UseFromProcessRef":"","Item":[{"StatusFieldRef":"Assigned","Top":"10","Left":"10","TransitionStatusFieldRef":["Unassigned","In Backlog","In Sprint"]},{"StatusFieldRef":"Closed","Top":"10","Left":"310","TransitionStatusFieldRef":[]},{"StatusFieldRef":"In Backlog","Top":"0","Left":"239","TransitionStatusFieldRef":["Assigned","In Sprint"]},{"StatusFieldRef":"In Progress","Top":"10","Left":"610","TransitionStatusFieldRef":["Closed","QA Approved"]},{"StatusFieldRef":"In Sprint","Top":"0","Left":"0","TransitionStatusFieldRef":["In Progress"]},{"StatusFieldRef":"QA Approved","Top":"111","Left":"655","TransitionStatusFieldRef":["user Accepted"]},{"StatusFieldRef":"Reopened","Top":"0","Left":"9","TransitionStatusFieldRef":[]},{"StatusFieldRef":"Unassigned","Top":"0","Left":"0","TransitionStatusFieldRef":["Assigned","In Backlog"]},{"StatusFieldRef":"user Accepted","Top":"203","Left":"0","TransitionStatusFieldRef":["Closed"]}]},"Label":"Bug","Image":"type-bug.png","Color":"#ff0000","Tag":""},{"TheScreen":{"UseFromProcessRef":"","Item":[{"InternalField":8,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":3,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":2,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":9,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":15,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":16,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":17,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":1,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":42,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":14,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":38,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":23,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":35,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":10,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"Status","CreateScreen":false,"EditScreen":true,"ViewScreen":true},{"InternalField":36,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedComments","CreateScreen":false,"EditScreen":true,"ViewScreen":true},{"InternalField":37,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedHistory","CreateScreen":false,"EditScreen":true,"ViewScreen":true},{"InternalField":30,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"CalculatedTimeLogged","CreateScreen":false,"EditScreen":true,"ViewScreen":true},{"InternalField":5,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"Visibility","CreateScreen":false,"EditScreen":true,"ViewScreen":false},{"InternalField":18,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"DateCreated","CreateScreen":false,"EditScreen":false,"ViewScreen":true},{"InternalField":33,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedTime","CreateScreen":false,"EditScreen":false,"ViewScreen":true},{"InternalField":40,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedHierarchy","CreateScreen":false,"EditScreen":false,"ViewScreen":true},{"InternalField":39,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedLinks","CreateScreen":false,"EditScreen":false,"ViewScreen":true},{"InternalField":34,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedSourceControl","CreateScreen":false,"EditScreen":false,"ViewScreen":true}]},"TheWorkflow":{"UseFromProcessRef":"","Item":[{"StatusFieldRef":"Approved","Top":"0","Left":"9","TransitionStatusFieldRef":["Assigned","In Progress"]},{"StatusFieldRef":"Assigned","Top":"0","Left":"0","TransitionStatusFieldRef":["Unassigned","In Progress"]},{"StatusFieldRef":"Closed","Top":"0","Left":"0","TransitionStatusFieldRef":[]},{"StatusFieldRef":"In Backlog","Top":"10","Left":"460","TransitionStatusFieldRef":[]},{"StatusFieldRef":"In Progress","Top":"0","Left":"0","TransitionStatusFieldRef":["QA Approved"]},{"StatusFieldRef":"In Sprint","Top":"10","Left":"760","TransitionStatusFieldRef":[]},{"StatusFieldRef":"QA Approved","Top":"0","Left":"0","TransitionStatusFieldRef":["user Accepted","In Progress"]},{"StatusFieldRef":"Reopened","Top":"80","Left":"160","TransitionStatusFieldRef":[]},{"StatusFieldRef":"Unassigned","Top":"0","Left":"0","TransitionStatusFieldRef":["Assigned"]},{"StatusFieldRef":"user Accepted","Top":"0","Left":"0","TransitionStatusFieldRef":["QA Approved","Closed"]}]},"Label":"New Requirement","Image":"type-enhancement.png","Color":"#e612e6","Tag":""},{"TheScreen":{"UseFromProcessRef":"","Item":[{"InternalField":8,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":false,"ViewScreen":true},{"InternalField":2,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":17,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":15,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":42,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":9,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":5,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":false,"ViewScreen":false},{"InternalField":22,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":38,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":36,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedComments","CreateScreen":false,"EditScreen":true,"ViewScreen":true},{"InternalField":40,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedHierarchy","CreateScreen":false,"EditScreen":true,"ViewScreen":true},{"InternalField":37,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedHistory","CreateScreen":false,"EditScreen":true,"ViewScreen":true},{"InternalField":39,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedLinks","CreateScreen":false,"EditScreen":false,"ViewScreen":true},{"InternalField":35,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedWatchers","CreateScreen":false,"EditScreen":false,"ViewScreen":true}]},"TheWorkflow":{"UseFromProcessRef":"New Requirement","Item":[]},"Label":"Change Request","Image":"type-changerequest.png","Color":"#171517","Tag":""},{"TheScreen":{"UseFromProcessRef":"","Item":[{"InternalField":8,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":false,"ViewScreen":false},{"InternalField":3,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":2,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":16,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":10,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":17,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":30,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":38,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":36,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedComments","CreateScreen":false,"EditScreen":true,"ViewScreen":true},{"InternalField":33,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedTime","CreateScreen":false,"EditScreen":false,"ViewScreen":true}]},"TheWorkflow":{"UseFromProcessRef":"","Item":[{"StatusFieldRef":"Approved","Top":"0","Left":"0","TransitionStatusFieldRef":["Assigned"]},{"StatusFieldRef":"Assigned","Top":"0","Left":"0","TransitionStatusFieldRef":["In Progress"]},{"StatusFieldRef":"Closed","Top":"0","Left":"0","TransitionStatusFieldRef":[]},{"StatusFieldRef":"In Backlog","Top":"10","Left":"460","TransitionStatusFieldRef":[]},{"StatusFieldRef":"In Progress","Top":"0","Left":"0","TransitionStatusFieldRef":["Closed"]},{"StatusFieldRef":"In Sprint","Top":"10","Left":"760","TransitionStatusFieldRef":[]},{"StatusFieldRef":"QA Approved","Top":"80","Left":"10","TransitionStatusFieldRef":[]},{"StatusFieldRef":"Reopened","Top":"80","Left":"160","TransitionStatusFieldRef":[]},{"StatusFieldRef":"Unassigned","Top":"0","Left":"0","TransitionStatusFieldRef":[]},{"StatusFieldRef":"user Accepted","Top":"80","Left":"460","TransitionStatusFieldRef":[]}]},"Label":"Task","Image":"type-task.png","Color":"#e68012","Tag":""}],"IsNew":true,"IsExisting":false,"Active":true,"Archived":false,"Deleted":false,"Created":"2012-07-03T10:54:06.1482413Z","Revised":"2012-07-03T10:54:06.1482413Z","Errors":[],"HasErrors":false,"Id":0}')
INSERT gemini_projecttemplate(templateid,templatekey,title,descrip,author,published,versionnumber,template) VALUES('13','ISSUES','Bug and Issue Tracking','Standard Issue & Bug Tracking project template','Countersoft',convert(datetime,'2012-07-03 11:54:07.000',121),'1.0.0','{"Name":"Bug and Issue Tracking","Description":"Standard Issue & Bug Tracking project template","Publisher":"Countersoft","Published":"2012-07-03T11:54:07.9423439+01:00","Version":"1.0.0","HomePage":2,"TemplateKey":"ISSUES","Menu":[{"PageType":1,"SeperateAfter":false,"ProjectGroups":[1],"Sequence":1},{"PageType":2,"SeperateAfter":true,"ProjectGroups":[1],"Sequence":2},{"PageType":9,"SeperateAfter":false,"ProjectGroups":[1],"Sequence":3},{"PageType":19,"SeperateAfter":false,"ProjectGroups":[1],"Sequence":4},{"PageType":20,"SeperateAfter":true,"ProjectGroups":[1],"Sequence":5},{"PageType":3,"SeperateAfter":false,"ProjectGroups":[1],"Sequence":6},{"PageType":5,"SeperateAfter":false,"ProjectGroups":[1],"Sequence":7},{"PageType":4,"SeperateAfter":true,"ProjectGroups":[1],"Sequence":8},{"PageType":7,"SeperateAfter":false,"ProjectGroups":[1],"Sequence":9},{"PageType":8,"SeperateAfter":false,"ProjectGroups":[1],"Sequence":10}],"CustomField":[{"Label":"Customer","ScreenLabel":"Customer","ScreenDescription":"Our Client name","ScreenTooltip":"Select the customer from the list","MaxLen":0,"CanMultiSelect":true,"RegularExpression":"","Type":"L","ProjectIdFilter":true,"ShowInAttributes":true,"LookupStaticData":"Customer A\r\nCustomer B\r\nCustomer C\r\n                        \r\n                    ","MaxValue":0.0,"MinValue":0.0,"ListLimiter":""},{"Label":"User Application","ScreenLabel":"User Application","ScreenDescription":"","ScreenTooltip":"User Application","MaxLen":0,"CanMultiSelect":false,"RegularExpression":"","Type":"C","ProjectIdFilter":false,"ShowInAttributes":true,"LookupStaticData":"Peoplesoft Financials\r\nPeoplesoft HR\r\nDynamics CRM\r\nOURSYS\r\nInventory Management\r\nOther","MaxValue":0.0,"MinValue":0.0,"ListLimiter":""},{"Label":"Product Platform","ScreenLabel":"Product Platform (OS)","ScreenDescription":"","ScreenTooltip":"Product Platform (OS)","MaxLen":0,"CanMultiSelect":false,"RegularExpression":"","Type":"C","ProjectIdFilter":false,"ShowInAttributes":true,"LookupStaticData":"Windows 7\r\nWindows 8\r\nLinux\r\nUbuntu\r\nMac OS\r\niPad\r\n\r\n                        \r\n                    ","MaxValue":0.0,"MinValue":0.0,"ListLimiter":""}],"Priority":[{"Label":"Low","Image":"priority-low.png","Color":"#0000ff"},{"Label":"Medium","Image":"priority-medium.png","Color":"#ffffff"},{"Label":"High","Image":"priority-high.png","Color":"#eb0c1f"}],"Severity":[{"Label":"Trivial","Image":"severity-trivial.png","Color":""},{"Label":"Minor","Image":"severity-minor.png","Color":"#0000ff"},{"Label":"Major","Image":"severity-major.png","Color":"#8c054d"}],"Status":[{"Final":false,"Label":"In Backlog","Image":"test-blackbox.png","Color":"#3d213d"},{"Final":false,"Label":"In Sprint","Image":"test-user.png","Color":"#337a0f"},{"Final":false,"Label":"Unassigned","Image":"status-unassigned.png","Color":"#0000ff"},{"Final":false,"Label":"Assigned","Image":"status-assigned.png","Color":"#47d119"},{"Final":false,"Label":"In Progress","Image":"status-inprogress.png","Color":"#a130a1"},{"Final":true,"Label":"Closed","Image":"status-closed.png","Color":"#ffffff"},{"Final":false,"Label":"Reopened","Image":"status-reopened.png","Color":"#0a1c91"},{"Final":false,"Label":"Approved","Image":"status-approved.png","Color":""}],"Resolution":[{"Final":true,"Label":"Parked - No Further Action"},{"Final":true,"Label":"Complete"}],"Link":[{"Label":"Duplicated"},{"Label":"Related"}],"Time":[{"Label":"Billable"},{"Label":"Non-Billable"},{"Label":"Internal"},{"Label":"Scrum Master PM"},{"Label":"Testing"}],"Process":[{"TheScreen":{"UseFromProcessRef":"","Item":[{"InternalField":8,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":3,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":2,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":0,"IsExternalCustomField":true,"IsExternalWidgetField":false,"ExternalFieldRef":"26","CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":0,"IsExternalCustomField":true,"IsExternalWidgetField":false,"ExternalFieldRef":"28","CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":38,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":13,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":9,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":10,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":5,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":false},{"InternalField":22,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":36,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedComments","CreateScreen":false,"EditScreen":true,"ViewScreen":true}]},"TheWorkflow":{"UseFromProcessRef":"","Item":[{"StatusFieldRef":"Assigned","Top":"0","Left":"9","TransitionStatusFieldRef":["In Progress","Closed"]},{"StatusFieldRef":"Closed","Top":"407","Left":"0","TransitionStatusFieldRef":["Reopened"]},{"StatusFieldRef":"In Backlog","Top":"10","Left":"310","TransitionStatusFieldRef":[]},{"StatusFieldRef":"In Progress","Top":"0","Left":"577","TransitionStatusFieldRef":["Closed"]},{"StatusFieldRef":"In Sprint","Top":"10","Left":"610","TransitionStatusFieldRef":[]},{"StatusFieldRef":"Reopened","Top":"41","Left":"151","TransitionStatusFieldRef":["Assigned","Unassigned"]},{"StatusFieldRef":"Unassigned","Top":"0","Left":"0","TransitionStatusFieldRef":["Assigned"]}]},"Label":"Investigation","Image":"type-investigation.png","Color":"#ffd000","Tag":""},{"TheScreen":{"UseFromProcessRef":"","Item":[{"InternalField":8,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":3,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":2,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":0,"IsExternalCustomField":true,"IsExternalWidgetField":false,"ExternalFieldRef":"30","CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":4,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":16,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":17,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":9,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":10,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":13,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":23,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":21,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":15,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":12,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":1,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":42,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":11,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":7,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":24,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":30,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":5,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":false},{"InternalField":38,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":40,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":39,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":34,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":35,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":14,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"PercentComplete","CreateScreen":false,"EditScreen":true,"ViewScreen":true},{"InternalField":36,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedComments","CreateScreen":false,"EditScreen":true,"ViewScreen":true},{"InternalField":33,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedTime","CreateScreen":false,"EditScreen":true,"ViewScreen":true},{"InternalField":18,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"DateCreated","CreateScreen":false,"EditScreen":false,"ViewScreen":true},{"InternalField":19,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"DateRevised","CreateScreen":false,"EditScreen":false,"ViewScreen":true},{"InternalField":37,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedHistory","CreateScreen":false,"EditScreen":false,"ViewScreen":true}]},"TheWorkflow":{"UseFromProcessRef":"Investigation","Item":[]},"Label":"Bug","Image":"type-bug.png","Color":"#ff0000","Tag":""},{"TheScreen":{"UseFromProcessRef":"","Item":[{"InternalField":8,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":3,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":2,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":9,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":15,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":16,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":17,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":1,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":42,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":14,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":38,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":23,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":35,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":10,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"Status","CreateScreen":false,"EditScreen":true,"ViewScreen":true},{"InternalField":36,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedComments","CreateScreen":false,"EditScreen":true,"ViewScreen":true},{"InternalField":37,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedHistory","CreateScreen":false,"EditScreen":true,"ViewScreen":true},{"InternalField":30,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"CalculatedTimeLogged","CreateScreen":false,"EditScreen":true,"ViewScreen":true},{"InternalField":5,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"Visibility","CreateScreen":false,"EditScreen":true,"ViewScreen":false},{"InternalField":18,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"DateCreated","CreateScreen":false,"EditScreen":false,"ViewScreen":true},{"InternalField":33,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedTime","CreateScreen":false,"EditScreen":false,"ViewScreen":true},{"InternalField":40,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedHierarchy","CreateScreen":false,"EditScreen":false,"ViewScreen":true},{"InternalField":39,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedLinks","CreateScreen":false,"EditScreen":false,"ViewScreen":true},{"InternalField":34,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedSourceControl","CreateScreen":false,"EditScreen":false,"ViewScreen":true}]},"TheWorkflow":{"UseFromProcessRef":"","Item":[{"StatusFieldRef":"Approved","Top":"10","Left":"10","TransitionStatusFieldRef":["Assigned","Unassigned"]},{"StatusFieldRef":"Assigned","Top":"10","Left":"160","TransitionStatusFieldRef":[]},{"StatusFieldRef":"Closed","Top":"10","Left":"310","TransitionStatusFieldRef":[]},{"StatusFieldRef":"In Backlog","Top":"10","Left":"460","TransitionStatusFieldRef":[]},{"StatusFieldRef":"In Progress","Top":"0","Left":"0","TransitionStatusFieldRef":["Closed"]},{"StatusFieldRef":"In Sprint","Top":"10","Left":"760","TransitionStatusFieldRef":[]},{"StatusFieldRef":"Reopened","Top":"80","Left":"10","TransitionStatusFieldRef":[]},{"StatusFieldRef":"Unassigned","Top":"80","Left":"160","TransitionStatusFieldRef":["In Progress","Assigned"]}]},"Label":"New Requirement","Image":"type-enhancement.png","Color":"#e612e6","Tag":""},{"TheScreen":{"UseFromProcessRef":"","Item":[{"InternalField":8,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":false,"ViewScreen":true},{"InternalField":2,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":17,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":15,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":42,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":9,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":5,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":false,"ViewScreen":false},{"InternalField":22,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":38,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":36,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedComments","CreateScreen":false,"EditScreen":true,"ViewScreen":true},{"InternalField":40,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedHierarchy","CreateScreen":false,"EditScreen":true,"ViewScreen":true},{"InternalField":37,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedHistory","CreateScreen":false,"EditScreen":true,"ViewScreen":true},{"InternalField":39,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedLinks","CreateScreen":false,"EditScreen":false,"ViewScreen":true},{"InternalField":35,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedWatchers","CreateScreen":false,"EditScreen":false,"ViewScreen":true}]},"TheWorkflow":{"UseFromProcessRef":"New Requirement","Item":[]},"Label":"Change Request","Image":"type-changerequest.png","Color":"#171517","Tag":""},{"TheScreen":{"UseFromProcessRef":"","Item":[{"InternalField":8,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":false,"ViewScreen":false},{"InternalField":3,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":2,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":16,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":10,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":17,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":30,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":38,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":36,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedComments","CreateScreen":false,"EditScreen":true,"ViewScreen":true},{"InternalField":33,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedTime","CreateScreen":false,"EditScreen":false,"ViewScreen":true}]},"TheWorkflow":{"UseFromProcessRef":"","Item":[{"StatusFieldRef":"Approved","Top":"10","Left":"10","TransitionStatusFieldRef":[]},{"StatusFieldRef":"Assigned","Top":"0","Left":"0","TransitionStatusFieldRef":["Closed","Unassigned","In Progress"]},{"StatusFieldRef":"Closed","Top":"10","Left":"310","TransitionStatusFieldRef":[]},{"StatusFieldRef":"In Backlog","Top":"10","Left":"460","TransitionStatusFieldRef":[]},{"StatusFieldRef":"In Progress","Top":"0","Left":"0","TransitionStatusFieldRef":["Closed"]},{"StatusFieldRef":"In Sprint","Top":"10","Left":"760","TransitionStatusFieldRef":[]},{"StatusFieldRef":"Reopened","Top":"9","Left":"0","TransitionStatusFieldRef":[]},{"StatusFieldRef":"Unassigned","Top":"80","Left":"160","TransitionStatusFieldRef":["Assigned"]}]},"Label":"Task","Image":"type-task.png","Color":"#e68012","Tag":""}],"IsNew":true,"IsExisting":false,"Active":true,"Archived":false,"Deleted":false,"Created":"2012-07-03T10:54:06.1482413Z","Revised":"2012-07-03T10:54:06.1482413Z","Errors":[],"HasErrors":false,"Id":0}')
INSERT gemini_projecttemplate(templateid,templatekey,title,descrip,author,published,versionnumber,template) VALUES('14','MARKETING','Marketing team management','Standard Marketing project template','Countersoft',convert(datetime,'2012-07-03 11:54:07.000',121),'1.0.0','{"Name":"Marketing team management","Description":"Standard Marketing project template","Publisher":"Countersoft","Published":"2012-07-03T11:54:07.9423439+01:00","Version":"1.0.0","HomePage":1,"TemplateKey":"MARKETING","Menu":[{"PageType":2,"SeperateAfter":false,"ProjectGroups":[2],"Sequence":1},{"PageType":3,"SeperateAfter":true,"ProjectGroups":[2],"Sequence":2},{"PageType":1,"SeperateAfter":true,"ProjectGroups":[2],"Sequence":3},{"PageType":20,"SeperateAfter":false,"ProjectGroups":[2],"Sequence":4},{"PageType":8,"SeperateAfter":false,"ProjectGroups":[1],"Sequence":5}],"CustomField":[{"Label":"Owner","ScreenLabel":"Owner","ScreenDescription":"Who ownes it?","ScreenTooltip":"Select the owner from the list","MaxLen":0,"CanMultiSelect":false,"RegularExpression":"","Type":"C","ProjectIdFilter":false,"ShowInAttributes":false,"LookupStaticData":"N/A\r\nMe\r\nYou\r\nSome Else\r\n\r\n\r\n\r\n                        \r\n                    ","MaxValue":0.0,"MinValue":0.0,"ListLimiter":""},{"Label":"Agencies","ScreenLabel":"Agencies","ScreenDescription":"","ScreenTooltip":"Agencies Involved","MaxLen":0,"CanMultiSelect":false,"RegularExpression":"","Type":"C","ProjectIdFilter":false,"ShowInAttributes":true,"LookupStaticData":"N/A\r\nMRM Worldwide\r\nMcann\r\nOgilvy and Mather\r\nSaatchi and Saatchi\r\nJelabi\r\nJellbean Creative\r\nOther","MaxValue":0.0,"MinValue":0.0,"ListLimiter":""},{"Label":"Budget Approved","ScreenLabel":"Budget Approved","ScreenDescription":"","ScreenTooltip":"Budget Approver","MaxLen":0,"CanMultiSelect":false,"RegularExpression":"","Type":"Y","ProjectIdFilter":false,"ShowInAttributes":true,"LookupStaticData":"                        \r\n                    ","MaxValue":0.0,"MinValue":0.0,"ListLimiter":""}],"Priority":[{"Label":"High","Image":"priority-high.png","Color":"#a30d21"},{"Label":"Low","Image":"priority-low.png","Color":"#0000ff"},{"Label":"Medium","Image":"priority-medium.png","Color":"#161670"}],"Severity":[{"Label":"Minor","Image":"severity-minor.png","Color":"#0000ff"},{"Label":"Major","Image":"severity-major.png","Color":"#a80a31"}],"Status":[{"Final":true,"Label":"Planned","Image":"status-assigned.png","Color":"#16b821"},{"Final":false,"Label":"In Progress","Image":"status-inprogress.png","Color":"#941894"},{"Final":false,"Label":"Completed","Image":"status-closed.png","Color":"#c7bf2a"},{"Final":false,"Label":"Parked","Image":"status-postponed.png","Color":"#5e105e"}],"Resolution":[{"Final":false,"Label":"Unresolved"},{"Final":true,"Label":"Complete"}],"Link":[{"Label":"Duplicated"},{"Label":"Related"}],"Time":[{"Label":"Billable"},{"Label":"non-billable"},{"Label":"Internal"}],"Process":[{"TheScreen":{"UseFromProcessRef":"","Item":[{"InternalField":8,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":3,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":2,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":0,"IsExternalCustomField":true,"IsExternalWidgetField":false,"ExternalFieldRef":"44","CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":0,"IsExternalCustomField":true,"IsExternalWidgetField":false,"ExternalFieldRef":"14","CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":16,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":17,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":14,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":36,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":40,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":15,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":9,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":10,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":30,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":35,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":0,"IsExternalCustomField":true,"IsExternalWidgetField":false,"ExternalFieldRef":"43","CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":38,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true}]},"TheWorkflow":{"UseFromProcessRef":"","Item":[{"StatusFieldRef":"Completed","Top":"219","Left":"538","TransitionStatusFieldRef":[]},{"StatusFieldRef":"In Progress","Top":"241","Left":"275","TransitionStatusFieldRef":["Completed","Parked","Planned"]},{"StatusFieldRef":"Parked","Top":"10","Left":"310","TransitionStatusFieldRef":["Planned"]},{"StatusFieldRef":"Planned","Top":"229","Left":"80","TransitionStatusFieldRef":["In Progress"]}]},"Label":"Campaign","Image":"type-enhancement.png","Color":"#b31bb3","Tag":""},{"TheScreen":{"UseFromProcessRef":"","Item":[{"InternalField":8,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":3,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":2,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":38,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":36,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":10,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":30,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true}]},"TheWorkflow":{"UseFromProcessRef":"Campaign","Item":[]},"Label":"Task","Image":"type-task.png","Color":"#522c52","Tag":""}],"IsNew":true,"IsExisting":false,"Active":true,"Archived":false,"Deleted":false,"Created":"2012-07-03T10:54:06.1482413Z","Revised":"2012-07-03T10:54:06.1482413Z","Errors":[],"HasErrors":false,"Id":0}')
INSERT gemini_projecttemplate(templateid,templatekey,title,descrip,author,published,versionnumber,template) VALUES('15','REQUIREMENTS','Requirements capture and management','Standard business requirements capture and management project template','Countersoft',convert(datetime,'2012-07-03 11:54:07.000',121),'1.0.0','{"Name":"Requirements capture and management","Description":"Standard business requirements capture and management project template","Publisher":"Countersoft","Published":"2012-07-03T11:54:07.9423439+01:00","Version":"1.0.0","HomePage":2,"TemplateKey":"REQUIREMENTS","Menu":[{"PageType":1,"SeperateAfter":false,"ProjectGroups":[1],"Sequence":1},{"PageType":2,"SeperateAfter":true,"ProjectGroups":[1],"Sequence":2},{"PageType":3,"SeperateAfter":false,"ProjectGroups":[1],"Sequence":3},{"PageType":4,"SeperateAfter":false,"ProjectGroups":[1],"Sequence":4},{"PageType":19,"SeperateAfter":false,"ProjectGroups":[1],"Sequence":5}],"CustomField":[{"Label":"Customer","ScreenLabel":"Customer","ScreenDescription":"Our Client name","ScreenTooltip":"Select the customer from the list","MaxLen":0,"CanMultiSelect":true,"RegularExpression":"","Type":"C","ProjectIdFilter":true,"ShowInAttributes":true,"LookupStaticData":"Customer A\r\nCustomer B\r\nCustomer C\r\n\r\n\r\n\r\n                        \r\n                    ","MaxValue":0.0,"MinValue":0.0,"ListLimiter":""},{"Label":"Customer Backlog ID","ScreenLabel":"Customer Backlog ID","ScreenDescription":"","ScreenTooltip":"Customer Backlog ID","MaxLen":12,"CanMultiSelect":false,"RegularExpression":"","Type":"C","ProjectIdFilter":false,"ShowInAttributes":true,"LookupStaticData":"BACKLOG A\r\nBACKLOG B\r\nBACKLOG C\r\nBACKLOG D\r\nBACKLOG E","MaxValue":999.0,"MinValue":0.0,"ListLimiter":""}],"Priority":[{"Label":"Low","Image":"priority-low.png","Color":"#0000ff"},{"Label":"Medium","Image":"priority-medium.png","Color":"#ffffff"},{"Label":"High","Image":"priority-high.png","Color":"#eb0c1f"}],"Severity":[{"Label":"Trivial","Image":"severity-trivial.png","Color":""},{"Label":"Minor","Image":"severity-minor.png","Color":"#0000ff"},{"Label":"Major","Image":"severity-major.png","Color":"#8c054d"}],"Status":[{"Final":false,"Label":"Approved","Image":"status-approved.png","Color":"#36a820"},{"Final":false,"Label":"In Backlog","Image":"test-blackbox.png","Color":"#3d213d"},{"Final":false,"Label":"In Sprint","Image":"test-user.png","Color":"#337a0f"},{"Final":false,"Label":"Requirement","Image":"status-unassigned.png","Color":"#0000ff"},{"Final":false,"Label":"Walkthrough","Image":"status-assigned.png","Color":"#47d119"},{"Final":true,"Label":"Closed","Image":"status-closed.png","Color":"#ffffff"},{"Final":false,"Label":"Reopened","Image":"status-reopened.png","Color":"#0a1c91"},{"Final":false,"Label":"In Progress","Image":"status-inprogress.png","Color":"#8c3d8c"}],"Resolution":[{"Final":true,"Label":"Parked - No Further Action"},{"Final":true,"Label":"Complete"}],"Link":[{"Label":"Duplicated"},{"Label":"Related"}],"Time":[{"Label":"Billable"},{"Label":"Non-Billable"},{"Label":"Internal"},{"Label":"Scrum Master PM"},{"Label":"Testing"}],"Process":[{"TheScreen":{"UseFromProcessRef":"","Item":[{"InternalField":8,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":3,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":2,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":38,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":9,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":10,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":15,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":42,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":16,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":17,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":1,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":14,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":23,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":35,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":36,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":0,"IsExternalCustomField":true,"IsExternalWidgetField":false,"ExternalFieldRef":"45","CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":0,"IsExternalCustomField":true,"IsExternalWidgetField":false,"ExternalFieldRef":"46","CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":37,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedHistory","CreateScreen":false,"EditScreen":true,"ViewScreen":true},{"InternalField":30,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"CalculatedTimeLogged","CreateScreen":false,"EditScreen":true,"ViewScreen":true},{"InternalField":5,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"Visibility","CreateScreen":false,"EditScreen":true,"ViewScreen":false},{"InternalField":18,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"DateCreated","CreateScreen":false,"EditScreen":false,"ViewScreen":true},{"InternalField":40,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedHierarchy","CreateScreen":false,"EditScreen":false,"ViewScreen":true},{"InternalField":33,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedTime","CreateScreen":false,"EditScreen":false,"ViewScreen":true},{"InternalField":39,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedLinks","CreateScreen":false,"EditScreen":false,"ViewScreen":true},{"InternalField":0,"IsExternalCustomField":false,"IsExternalWidgetField":true,"ExternalFieldRef":"28FE1D91-7730-4067-8A10-F452FCB9D090","CreateScreen":false,"EditScreen":false,"ViewScreen":true}]},"TheWorkflow":{"UseFromProcessRef":"","Item":[{"StatusFieldRef":"Approved","Top":"212","Left":"535","TransitionStatusFieldRef":["In Backlog","Requirement"]},{"StatusFieldRef":"Closed","Top":"191","Left":"62","TransitionStatusFieldRef":["Reopened"]},{"StatusFieldRef":"In Backlog","Top":"410","Left":"351","TransitionStatusFieldRef":["In Sprint"]},{"StatusFieldRef":"In Sprint","Top":"375","Left":"45","TransitionStatusFieldRef":["Closed"]},{"StatusFieldRef":"Reopened","Top":"229","Left":"298","TransitionStatusFieldRef":["Requirement"]},{"StatusFieldRef":"Requirement","Top":"25","Left":"98","TransitionStatusFieldRef":["Closed","Walkthrough"]},{"StatusFieldRef":"Walkthrough","Top":"35","Left":"349","TransitionStatusFieldRef":["Approved"]}]},"Label":"Requirement","Image":"type-enhancement.png","Color":"#e612e6","Tag":""},{"TheScreen":{"UseFromProcessRef":"Requirement","Item":[{"InternalField":8,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":3,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":2,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":38,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":9,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":10,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":15,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":42,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":16,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":17,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":1,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":14,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":23,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":35,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":36,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":0,"IsExternalCustomField":true,"IsExternalWidgetField":false,"ExternalFieldRef":"45","CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":0,"IsExternalCustomField":true,"IsExternalWidgetField":false,"ExternalFieldRef":"46","CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":37,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedHistory","CreateScreen":false,"EditScreen":true,"ViewScreen":true},{"InternalField":30,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"CalculatedTimeLogged","CreateScreen":false,"EditScreen":true,"ViewScreen":true},{"InternalField":5,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"Visibility","CreateScreen":false,"EditScreen":true,"ViewScreen":false},{"InternalField":18,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"DateCreated","CreateScreen":false,"EditScreen":false,"ViewScreen":true},{"InternalField":40,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedHierarchy","CreateScreen":false,"EditScreen":false,"ViewScreen":true},{"InternalField":33,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedTime","CreateScreen":false,"EditScreen":false,"ViewScreen":true},{"InternalField":39,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedLinks","CreateScreen":false,"EditScreen":false,"ViewScreen":true},{"InternalField":0,"IsExternalCustomField":false,"IsExternalWidgetField":true,"ExternalFieldRef":"28FE1D91-7730-4067-8A10-F452FCB9D090","CreateScreen":false,"EditScreen":false,"ViewScreen":true}]},"TheWorkflow":{"UseFromProcessRef":"Requirement","Item":[]},"Label":"Story","Image":"type-newfeature.png","Color":"#0a5e85","Tag":""},{"TheScreen":{"UseFromProcessRef":"","Item":[{"InternalField":8,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":false,"ViewScreen":false},{"InternalField":3,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":2,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":10,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":16,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":17,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":30,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":38,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":36,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedComments","CreateScreen":false,"EditScreen":true,"ViewScreen":true},{"InternalField":33,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedTime","CreateScreen":false,"EditScreen":false,"ViewScreen":true}]},"TheWorkflow":{"UseFromProcessRef":"","Item":[{"StatusFieldRef":"Approved","Top":"189","Left":"202","TransitionStatusFieldRef":["In Progress"]},{"StatusFieldRef":"Closed","Top":"211","Left":"553","TransitionStatusFieldRef":["Reopened"]},{"StatusFieldRef":"In Backlog","Top":"264","Left":"10","TransitionStatusFieldRef":[]},{"StatusFieldRef":"In Progress","Top":"385","Left":"152","TransitionStatusFieldRef":["Closed"]},{"StatusFieldRef":"In Sprint","Top":"22","Left":"176","TransitionStatusFieldRef":[]},{"StatusFieldRef":"Reopened","Top":"92","Left":"628","TransitionStatusFieldRef":["Requirement"]},{"StatusFieldRef":"Requirement","Top":"84","Left":"328","TransitionStatusFieldRef":["Approved"]},{"StatusFieldRef":"Walkthrough","Top":"438","Left":"560","TransitionStatusFieldRef":[]}]},"Label":"Task","Image":"type-task.png","Color":"#e68012","Tag":""}],"IsNew":true,"IsExisting":false,"Active":true,"Archived":false,"Deleted":false,"Created":"2012-07-03T10:54:06.1482413Z","Revised":"2012-07-03T10:54:06.1482413Z","Errors":[],"HasErrors":false,"Id":0}')
INSERT gemini_projecttemplate(templateid,templatekey,title,descrip,author,published,versionnumber,template) VALUES('16','SENTRY','Testing and QA management','Sentry for Gemini Testing (Quality Assurance) template','Countersoft',convert(datetime,'2012-07-03 11:54:07.000',121),'1.0.0','{"Name":"Testing and QA management","Description":"Sentry for Gemini Testing (Quality Assurance) template","Publisher":"Countersoft","Published":"2012-07-03T11:54:07.9423439+01:00","Version":"1.0.0","HomePage":1,"TemplateKey":"SENTRY","Menu":[{"PageType":1,"SeperateAfter":false,"ProjectGroups":[1],"Sequence":1},{"PageType":2,"SeperateAfter":true,"ProjectGroups":[1],"Sequence":2},{"PageType":9,"SeperateAfter":false,"ProjectGroups":[1],"Sequence":3},{"PageType":19,"SeperateAfter":false,"ProjectGroups":[1],"Sequence":4},{"PageType":20,"SeperateAfter":true,"ProjectGroups":[1],"Sequence":5},{"PageType":8,"SeperateAfter":false,"ProjectGroups":[1],"Sequence":6},{"PageType":7,"SeperateAfter":false,"ProjectGroups":[1],"Sequence":7}],"CustomField":[{"Label":"Client/Project","ScreenLabel":"Client/Project","ScreenDescription":"","ScreenTooltip":"Client/Project","MaxLen":0,"CanMultiSelect":false,"RegularExpression":"","Type":"C","ProjectIdFilter":false,"ShowInAttributes":true,"LookupStaticData":"N/A\r\nClient A - Project A\r\nClient A - Project B\r\nClient A - Project C\r\nClient B - Project 1\r\nClient B - Project 2\r\nClient C\r\nClient D\r\n","MaxValue":0.0,"MinValue":0.0,"ListLimiter":""},{"Label":"Affected User(s)","ScreenLabel":"Affected User(s)","ScreenDescription":"","ScreenTooltip":"Affected User(s)","MaxLen":0,"CanMultiSelect":true,"RegularExpression":"","Type":"U","ProjectIdFilter":false,"ShowInAttributes":true,"LookupStaticData":"                        \r\n                    ","MaxValue":0.0,"MinValue":0.0,"ListLimiter":"4|"},{"Label":"Test Type","ScreenLabel":"Test Type","ScreenDescription":"","ScreenTooltip":"Test Type","MaxLen":0,"CanMultiSelect":false,"RegularExpression":"","Type":"C","ProjectIdFilter":false,"ShowInAttributes":true,"LookupStaticData":"                        \r\n                    ","MaxValue":0.0,"MinValue":0.0,"ListLimiter":""},{"Label":"Preconditions","ScreenLabel":"Pre-Conditions","ScreenDescription":"","ScreenTooltip":"Pre-Conditions","MaxLen":250,"CanMultiSelect":false,"RegularExpression":"","Type":"R","ProjectIdFilter":false,"ShowInAttributes":false,"LookupStaticData":"                        \r\n                    ","MaxValue":0.0,"MinValue":0.0,"ListLimiter":""},{"Label":"Expected Results","ScreenLabel":"Expected Results","ScreenDescription":"","ScreenTooltip":"Expected Results","MaxLen":250,"CanMultiSelect":false,"RegularExpression":"","Type":"R","ProjectIdFilter":false,"ShowInAttributes":false,"LookupStaticData":"                        \r\n                    ","MaxValue":0.0,"MinValue":0.0,"ListLimiter":""}],"Priority":[{"Label":"High","Image":"priority-high.png","Color":"#e81717"},{"Label":"Low","Image":"priority-low.png","Color":"#b8b8e6"},{"Label":"Medium","Image":"priority-medium.png","Color":"#c7732a"}],"Severity":[{"Label":"Minor","Image":"severity-minor.png","Color":"#9595d6"},{"Label":"Major","Image":"severity-major.png","Color":"#bd20bd"},{"Label":"Critical","Image":"severity-showstopper.png","Color":"#e80523"}],"Status":[{"Final":true,"Label":"Reported","Image":"test-smoke.png","Color":"#177d4b"},{"Final":false,"Label":"Planned","Image":"test-installation.png","Color":"#2a0aa8"},{"Final":false,"Label":"Production Ready","Image":"type-enhancement.png","Color":"#2e292e"},{"Final":false,"Label":"Assessing","Image":"status-check.png","Color":"#e316e3"},{"Final":false,"Label":"Accepted","Image":"test-acceptance.png","Color":"#126b0d"},{"Final":false,"Label":"Rejected","Image":"status-rejected.png","Color":"#e60330"},{"Final":false,"Label":"In Progress","Image":"status-inprogress.png","Color":"#c95d36"},{"Final":false,"Label":"Ready For Deploy","Image":"test-whitebox.png","Color":"#2e292e"},{"Final":false,"Label":"Deployed","Image":"test-blackbox.png","Color":"#3636d1"},{"Final":false,"Label":"Closed","Image":"status-closed.png","Color":"#818f17"}],"Resolution":[{"Final":false,"Label":"Unresolved"},{"Final":true,"Label":"Rejected"},{"Final":true,"Label":"Resolved"}],"Link":[{"Label":"Duplicated"},{"Label":"Related"}],"Time":[{"Label":"Billable"},{"Label":"Non-billable"},{"Label":"Internal"}],"Process":[{"TheScreen":{"UseFromProcessRef":"","Item":[{"InternalField":8,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":3,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":2,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":false,"ViewScreen":true},{"InternalField":9,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":4,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":6,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":14,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":36,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":16,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":17,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":38,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":24,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"Source","CreateScreen":false,"EditScreen":true,"ViewScreen":true},{"InternalField":10,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"Status","CreateScreen":false,"EditScreen":true,"ViewScreen":true},{"InternalField":11,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"Resolution","CreateScreen":false,"EditScreen":true,"ViewScreen":true},{"InternalField":7,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssignedTo","CreateScreen":false,"EditScreen":true,"ViewScreen":true},{"InternalField":18,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"DateCreated","CreateScreen":false,"EditScreen":false,"ViewScreen":true},{"InternalField":21,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"ClosedDate","CreateScreen":false,"EditScreen":false,"ViewScreen":true},{"InternalField":20,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"ResolvedDate","CreateScreen":false,"EditScreen":false,"ViewScreen":true},{"InternalField":30,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"CalculatedTimeLogged","CreateScreen":false,"EditScreen":false,"ViewScreen":true},{"InternalField":1,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"IssueKey","CreateScreen":false,"EditScreen":false,"ViewScreen":true},{"InternalField":0,"IsExternalCustomField":false,"IsExternalWidgetField":true,"ExternalFieldRef":"D8DECEB7-BA6D-4A3F-A4A7-C79A5A5585BA","CreateScreen":false,"EditScreen":false,"ViewScreen":true},{"InternalField":0,"IsExternalCustomField":false,"IsExternalWidgetField":true,"ExternalFieldRef":"598B4999-4C31-427D-9BBC-C84AD954E55A","CreateScreen":false,"EditScreen":false,"ViewScreen":true},{"InternalField":0,"IsExternalCustomField":false,"IsExternalWidgetField":true,"ExternalFieldRef":"EFEE5D50-7DF6-49C3-9D69-E7EFC1AFEF72","CreateScreen":false,"EditScreen":false,"ViewScreen":true},{"InternalField":0,"IsExternalCustomField":false,"IsExternalWidgetField":true,"ExternalFieldRef":"AC93C4F9-B512-4A3D-8B37-BC3D5F83429D","CreateScreen":false,"EditScreen":false,"ViewScreen":true},{"InternalField":37,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedHistory","CreateScreen":false,"EditScreen":false,"ViewScreen":true},{"InternalField":0,"IsExternalCustomField":false,"IsExternalWidgetField":true,"ExternalFieldRef":"15627A0F-5AD2-4D35-BF24-60D678F59354","CreateScreen":false,"EditScreen":false,"ViewScreen":true},{"InternalField":39,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedLinks","CreateScreen":false,"EditScreen":false,"ViewScreen":true}]},"TheWorkflow":{"UseFromProcessRef":"","Item":[{"StatusFieldRef":"Assessing","Top":"94","Left":"32","TransitionStatusFieldRef":["Rejected","Accepted"]},{"StatusFieldRef":"Accepted","Top":"253","Left":"124","TransitionStatusFieldRef":["In Progress"]},{"StatusFieldRef":"Rejected","Top":"17","Left":"224","TransitionStatusFieldRef":[]},{"StatusFieldRef":"In Progress","Top":"247","Left":"458","TransitionStatusFieldRef":["Ready For Deploy","Rejected"]},{"StatusFieldRef":"Ready For Deploy","Top":"517","Left":"284","TransitionStatusFieldRef":["Deployed"]},{"StatusFieldRef":"Deployed","Top":"570","Left":"608","TransitionStatusFieldRef":["Closed"]},{"StatusFieldRef":"Closed","Top":"452","Left":"687","TransitionStatusFieldRef":[]}]},"Label":"Test Plan","Image":"type-investigation.png","Color":"#374fe8","Tag":"PLAN"},{"TheScreen":{"UseFromProcessRef":"","Item":[{"InternalField":8,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":0,"IsExternalCustomField":true,"IsExternalWidgetField":false,"ExternalFieldRef":"51","CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":3,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":2,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":0,"IsExternalCustomField":true,"IsExternalWidgetField":false,"ExternalFieldRef":"52","CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":0,"IsExternalCustomField":true,"IsExternalWidgetField":false,"ExternalFieldRef":"53","CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":10,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":13,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":7,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":16,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":17,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":14,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":23,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":0,"IsExternalCustomField":true,"IsExternalWidgetField":false,"ExternalFieldRef":"48","CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":38,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":false},{"InternalField":0,"IsExternalCustomField":true,"IsExternalWidgetField":false,"ExternalFieldRef":"47","CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":36,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":15,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":9,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":11,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":4,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":24,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":33,"IsExternalCustomField":false,"IsExternalWidgetField":false,"CreateScreen":true,"EditScreen":true,"ViewScreen":true},{"InternalField":20,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"ResolvedDate","CreateScreen":false,"EditScreen":false,"ViewScreen":true},{"InternalField":21,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"ClosedDate","CreateScreen":false,"EditScreen":false,"ViewScreen":true},{"InternalField":18,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"DateCreated","CreateScreen":false,"EditScreen":false,"ViewScreen":true},{"InternalField":19,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"DateRevised","CreateScreen":false,"EditScreen":false,"ViewScreen":true},{"InternalField":1,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"IssueKey","CreateScreen":false,"EditScreen":false,"ViewScreen":true},{"InternalField":30,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"CalculatedTimeLogged","CreateScreen":false,"EditScreen":false,"ViewScreen":true},{"InternalField":0,"IsExternalCustomField":false,"IsExternalWidgetField":true,"ExternalFieldRef":"617F1435-C0AA-48E6-B6A9-2C1F756E02A6","CreateScreen":false,"EditScreen":false,"ViewScreen":true},{"InternalField":0,"IsExternalCustomField":false,"IsExternalWidgetField":true,"ExternalFieldRef":"ACE2F8B3-C724-444F-9B72-B8B2137DDA16","CreateScreen":false,"EditScreen":false,"ViewScreen":true},{"InternalField":0,"IsExternalCustomField":false,"IsExternalWidgetField":true,"ExternalFieldRef":"15627A0F-5AD2-4D35-BF24-60D678F59354","CreateScreen":false,"EditScreen":false,"ViewScreen":true},{"InternalField":39,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedLinks","CreateScreen":false,"EditScreen":false,"ViewScreen":true},{"InternalField":37,"IsExternalCustomField":false,"IsExternalWidgetField":false,"ExternalFieldRef":"AssociatedHistory","CreateScreen":false,"EditScreen":false,"ViewScreen":true}]},"TheWorkflow":{"UseFromProcessRef":"Test Plan","Item":[]},"Label":"Test Case","Image":"type-task.png","Color":"#20700c","Tag":"CASE"}],"IsNew":true,"IsExisting":false,"Active":true,"Archived":false,"Deleted":false,"Created":"2012-07-03T10:54:06.1482413Z","Revised":"2012-07-03T10:54:06.1482413Z","Errors":[],"HasErrors":false,"Id":0}')
SET IDENTITY_INSERT gemini_projecttemplate OFF
GO

INSERT gemini_issuetypes(seq,typedesc,imagepath,color,screen,workflow,tag,templateid) VALUES('3','Task','assets/images/meta/AGILE/type-task.png','#e68012','{"ReferenceId":*STORY*,"Create":[{"ProjectGroups":[1],"Required":false,"Sequence":0,"Label":"","Id":"Type","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"Title","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":2,"Label":"","Id":"Description","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":3,"Label":"","Id":"Status","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":4,"Label":"","Id":"StartDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":5,"Label":"","Id":"DueDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":6,"Label":"","Id":"CalculatedTimeLogged","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":7,"Label":"","Id":"AssociatedAttachments","Type":0}],"Edit":[{"ProjectGroups":[1],"Required":false,"Sequence":0,"Label":"","Id":"Title","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"Description","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":2,"Label":"","Id":"Status","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":3,"Label":"","Id":"StartDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":4,"Label":"","Id":"DueDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":5,"Label":"","Id":"CalculatedTimeLogged","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":6,"Label":"","Id":"AssociatedAttachments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":7,"Label":"","Id":"AssociatedComments","Type":0}],"View":[{"ProjectGroups":[1],"Required":false,"Sequence":0,"Label":"","Id":"Title","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"Status","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":2,"Label":"","Id":"StartDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":3,"Label":"","Id":"DueDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":4,"Label":"","Id":"CalculatedTimeLogged","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":5,"Label":"","Id":"Description","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":6,"Label":"","Id":"AssociatedAttachments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":7,"Label":"","Id":"AssociatedComments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":8,"Label":"","Id":"AssociatedTime","Type":0}],"IsNew":true,"IsExisting":false,"Active":true,"Archived":false,"Deleted":false,"Created":"2012-08-01T18:40:49.4754629Z","Revised":"2012-08-01T18:40:49.4754629Z","Errors":[],"HasErrors":false,"Id":0}','{"ReferenceId":*STORY*,"Status":[{"StatusId":*INBACKLOG*,"Top":10,"Left":10,"Transitions":[]},{"StatusId":*INSPRINT*,"Top":10,"Left":160,"Transitions":[]},{"StatusId":*TESTED*,"Top":10,"Left":160,"Transitions":[{"StatusId":*INPROGRESS*,"Groups":[1]}]},{"StatusId":*INPROGRESS*,"Top":10,"Left":310,"Transitions":[{"StatusId":*CLOSED*,"Groups":[1]}]},{"StatusId":*CLOSED*,"Top":10,"Left":460,"Transitions":[]}]}','','10')
INSERT gemini_issuetypes(seq,typedesc,imagepath,color,screen,workflow,tag,templateid) VALUES('2','Bug','assets/images/meta/ISSUES/type-bug.png','#ff0000','{"ReferenceId":*NEWFEATURE*,"Create":[{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"Type","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":2,"Label":"","Id":"Title","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":3,"Label":"","Id":"Description","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":4,"Label":"","Id":"30","Type":1},{"ProjectGroups":[1],"Required":false,"Sequence":5,"Label":"","Id":"Severity","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":6,"Label":"","Id":"StartDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":7,"Label":"","Id":"DueDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":8,"Label":"","Id":"Priority","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":9,"Label":"","Id":"Status","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":10,"Label":"","Id":"Component","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":11,"Label":"","Id":"AffectedVersions","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":12,"Label":"","Id":"ClosedDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":13,"Label":"","Id":"EstimatedEffort","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":14,"Label":"","Id":"FixedInVersion","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":15,"Label":"","Id":"IssueKey","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":16,"Label":"","Id":"Points","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":17,"Label":"","Id":"Resolution","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":18,"Label":"","Id":"AssignedTo","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":19,"Label":"","Id":"Source","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":20,"Label":"","Id":"CalculatedTimeLogged","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":21,"Label":"","Id":"Visibility","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":22,"Label":"","Id":"AssociatedAttachments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":23,"Label":"","Id":"AssociatedHierarchy","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":24,"Label":"","Id":"AssociatedLinks","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":25,"Label":"","Id":"AssociatedSourceControl","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":26,"Label":"","Id":"AssociatedWatchers","Type":0}],"Edit":[{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"Type","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":2,"Label":"","Id":"Title","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":3,"Label":"","Id":"Description","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":4,"Label":"","Id":"30","Type":1},{"ProjectGroups":[1],"Required":false,"Sequence":5,"Label":"","Id":"Severity","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":6,"Label":"","Id":"StartDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":7,"Label":"","Id":"DueDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":8,"Label":"","Id":"Priority","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":9,"Label":"","Id":"Status","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":10,"Label":"","Id":"Component","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":11,"Label":"","Id":"AffectedVersions","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":12,"Label":"","Id":"ClosedDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":13,"Label":"","Id":"EstimatedEffort","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":14,"Label":"","Id":"FixedInVersion","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":15,"Label":"","Id":"IssueKey","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":16,"Label":"","Id":"Points","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":17,"Label":"","Id":"Resolution","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":18,"Label":"","Id":"AssignedTo","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":19,"Label":"","Id":"Source","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":20,"Label":"","Id":"CalculatedTimeLogged","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":21,"Label":"","Id":"Visibility","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":22,"Label":"","Id":"AssociatedAttachments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":23,"Label":"","Id":"AssociatedHierarchy","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":24,"Label":"","Id":"AssociatedLinks","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":25,"Label":"","Id":"AssociatedSourceControl","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":26,"Label":"","Id":"AssociatedWatchers","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":27,"Label":"","Id":"PercentComplete","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":28,"Label":"","Id":"AssociatedComments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":29,"Label":"","Id":"AssociatedTime","Type":0}],"View":[{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"Type","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":2,"Label":"","Id":"Title","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":3,"Label":"","Id":"Description","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":4,"Label":"","Id":"30","Type":1},{"ProjectGroups":[1],"Required":false,"Sequence":5,"Label":"","Id":"Severity","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":6,"Label":"","Id":"StartDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":7,"Label":"","Id":"DueDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":8,"Label":"","Id":"Priority","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":9,"Label":"","Id":"Status","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":10,"Label":"","Id":"Component","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":11,"Label":"","Id":"AffectedVersions","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":12,"Label":"","Id":"ClosedDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":13,"Label":"","Id":"EstimatedEffort","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":14,"Label":"","Id":"FixedInVersion","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":15,"Label":"","Id":"IssueKey","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":16,"Label":"","Id":"Points","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":17,"Label":"","Id":"Resolution","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":18,"Label":"","Id":"AssignedTo","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":19,"Label":"","Id":"Source","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":20,"Label":"","Id":"CalculatedTimeLogged","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":21,"Label":"","Id":"AssociatedAttachments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":22,"Label":"","Id":"AssociatedHierarchy","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":23,"Label":"","Id":"AssociatedLinks","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":24,"Label":"","Id":"AssociatedSourceControl","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":25,"Label":"","Id":"AssociatedWatchers","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":26,"Label":"","Id":"PercentComplete","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":27,"Label":"","Id":"AssociatedComments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":28,"Label":"","Id":"AssociatedTime","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":29,"Label":"","Id":"DateCreated","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":30,"Label":"","Id":"DateRevised","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":31,"Label":"","Id":"AssociatedHistory","Type":0}],"IsNew":true,"IsExisting":false,"Active":true,"Archived":false,"Deleted":false,"Created":"2012-08-01T08:48:33.6919324Z","Revised":"2012-08-01T08:48:33.6919324Z","Errors":[],"HasErrors":false,"Id":0}','{"ReferenceId":*NEWFEATURE*,"Status":[{"StatusId":*ASSIGNED*,"Top":0,"Left":9,"Transitions":[{"StatusId":*INPROGRESS*,"Groups":[1]},{"StatusId":*CLOSED*,"Groups":[1]}]},{"StatusId":*CLOSED*,"Top":407,"Left":0,"Transitions":[{"StatusId":*REOPENED*,"Groups":[1]}]},{"StatusId":*INPROGRESS*,"Top":0,"Left":577,"Transitions":[{"StatusId":*CLOSED*,"Groups":[1]}]},{"StatusId":*REOPENED*,"Top":41,"Left":151,"Transitions":[{"StatusId":*ASSIGNED*,"Groups":[1]},{"StatusId":*UNASSIGNED*,"Groups":[1]}]},{"StatusId":*UNASSIGNED*,"Top":0,"Left":0,"Transitions":[{"StatusId":*ASSIGNED*,"Groups":[1]}]}]}','','13')
INSERT gemini_issuetypes(seq,typedesc,imagepath,color,screen,workflow,tag,templateid) VALUES('2','Enquiry','assets/images/meta/ENQUIRY/status-reopened.png','#e63777','{"ReferenceId":*OPENISSUE*,"Create":[],"Edit":[],"View":[],"IsNew":true,"IsExisting":false,"Active":true,"Archived":false,"Deleted":false,"Created":"2012-08-01T09:00:19.0392759Z","Revised":"2012-08-01T09:00:19.0392759Z","Errors":[],"HasErrors":false,"Id":0}','{"ReferenceId":*OPENISSUE*,"Status":[]}','','11')
INSERT gemini_issuetypes(seq,typedesc,imagepath,color,screen,workflow,tag,templateid) VALUES('1','Open Issue','assets/images/meta/test-validation.png','#3300ff','{"ReferenceId":0,"Create":[{"ProjectGroups":[1],"Required":false,"Sequence":0,"Label":"","Id":"Type","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"Title","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":2,"Label":"","Id":"Description","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":3,"Label":"","Id":"24","Type":1},{"ProjectGroups":[1],"Required":false,"Sequence":4,"Label":"","Id":"26","Type":1},{"ProjectGroups":[1],"Required":false,"Sequence":5,"Label":"","Id":"25","Type":1},{"ProjectGroups":[1],"Required":false,"Sequence":6,"Label":"","Id":"Priority","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":7,"Label":"","Id":"DueDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":8,"Label":"","Id":"AssignedTo","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":9,"Label":"","Id":"Source","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":10,"Label":"","Id":"AssociatedComments","Type":0}],"Edit":[{"ProjectGroups":[1],"Required":false,"Sequence":0,"Label":"","Id":"Type","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"Title","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":2,"Label":"","Id":"Description","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":3,"Label":"","Id":"Priority","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":4,"Label":"","Id":"Status","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":5,"Label":"","Id":"StartDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":6,"Label":"","Id":"DueDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":7,"Label":"","Id":"Resolution","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":8,"Label":"","Id":"AssignedTo","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":9,"Label":"","Id":"Source","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":10,"Label":"","Id":"Visibility","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":11,"Label":"","Id":"24","Type":1},{"ProjectGroups":[1],"Required":false,"Sequence":12,"Label":"","Id":"26","Type":1},{"ProjectGroups":[1],"Required":false,"Sequence":13,"Label":"","Id":"25","Type":1}],"View":[{"ProjectGroups":[1],"Required":false,"Sequence":0,"Label":"","Id":"Type","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"Title","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":2,"Label":"","Id":"26","Type":1},{"ProjectGroups":[1],"Required":false,"Sequence":3,"Label":"","Id":"25","Type":1},{"ProjectGroups":[1],"Required":false,"Sequence":4,"Label":"","Id":"Status","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":5,"Label":"","Id":"Resolution","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":6,"Label":"","Id":"AssignedTo","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":7,"Label":"","Id":"StartDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":8,"Label":"","Id":"DueDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":9,"Label":"","Id":"Severity","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":10,"Label":"","Id":"ClosedDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":11,"Label":"","Id":"Priority","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":12,"Label":"","Id":"EstimatedEffort","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":13,"Label":"","Id":"FixedInVersion","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":14,"Label":"","Id":"IssueKey","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":15,"Label":"","Id":"Source","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":16,"Label":"","Id":"CalculatedTimeLogged","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":17,"Label":"","Id":"DateCreated","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":18,"Label":"","Id":"DateRevised","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":19,"Label":"","Id":"24","Type":1},{"ProjectGroups":[1],"Required":false,"Sequence":20,"Label":"","Id":"ReportedBy","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":21,"Label":"","Id":"Visibility","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":22,"Label":"","Id":"AssociatedAttachments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":23,"Label":"","Id":"Description","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":24,"Label":"","Id":"AssociatedComments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":25,"Label":"","Id":"AssociatedHierarchy","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":26,"Label":"","Id":"AssociatedLinks","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":27,"Label":"","Id":"AssociatedWatchers","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":28,"Label":"","Id":"AssociatedTime","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":29,"Label":"","Id":"AssociatedHistory","Type":0}],"IsNew":true,"IsExisting":false,"Active":true,"Archived":false,"Deleted":false,"Created":"2012-08-02T10:19:09.1783991Z","Revised":"2012-08-02T10:19:09.1783991Z","Errors":[],"HasErrors":false,"Id":0}','{"ReferenceId":0,"Status":[{"StatusId":*ESCALATION-LEVEL1*,"Top":40,"Left":376,"Transitions":[]},{"StatusId":*ESCALATION-LEVEL2*,"Top":41,"Left":196,"Transitions":[]},{"StatusId":*MOVEDTODELIVERY*,"Top":10,"Left":10,"Transitions":[]},{"StatusId":*RAISED*,"Top":185,"Left":62,"Transitions":[{"StatusId":*ASSIGNED*,"Groups":[1]},{"StatusId":*PENDINGAPPROVAL*,"Groups":[1]}]},{"StatusId":*USERREJECTED*,"Top":63,"Left":631,"Transitions":[{"StatusId":*ASSIGNED*,"Groups":[1]}]},{"StatusId":*ASSIGNED*,"Top":186,"Left":397,"Transitions":[{"StatusId":*CLOSED*,"Groups":[1]},{"StatusId":*USERACCEPTED*,"Groups":[1]}]},{"StatusId":*CLOSED*,"Top":341,"Left":674,"Transitions":[{"StatusId":*CASEREOPENED*,"Groups":[1]}]},{"StatusId":*CASEREOPENED*,"Top":508,"Left":296,"Transitions":[{"StatusId":*PENDINGAPPROVAL*,"Groups":[1]}]},{"StatusId":*USERACCEPTED*,"Top":188,"Left":687,"Transitions":[{"StatusId":*CLOSED*,"Groups":[1]},{"StatusId":*USERREJECTED*,"Groups":[1]}]},{"StatusId":*PENDINGAPPROVAL*,"Top":301,"Left":218,"Transitions":[{"StatusId":*APPROVED*,"Groups":[1]}]},{"StatusId":*APPROVED*,"Top":344,"Left":437,"Transitions":[{"StatusId":*ASSIGNED*,"Groups":[1]}]}]}','','11')
INSERT gemini_issuetypes(seq,typedesc,imagepath,color,screen,workflow,tag,templateid) VALUES('5','Bug','assets/images/meta/AGILE/type-bug.png','#ff0000','{"ReferenceId":*STORY*,"Create":[{"ProjectGroups":[1],"Required":false,"Sequence":0,"Label":"","Id":"Type","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"Title","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":2,"Label":"","Id":"Description","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":3,"Label":"","Id":"FixedInVersion","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":4,"Label":"","Id":"23","Type":1},{"ProjectGroups":[1],"Required":false,"Sequence":5,"Label":"","Id":"Component","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":6,"Label":"","Id":"Priority","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":7,"Label":"","Id":"Points","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":8,"Label":"","Id":"StartDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":9,"Label":"","Id":"DueDate","Type":0}],"Edit":[{"ProjectGroups":[1],"Required":false,"Sequence":0,"Label":"","Id":"Type","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"FixedInVersion","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":2,"Label":"","Id":"23","Type":1},{"ProjectGroups":[1],"Required":false,"Sequence":3,"Label":"","Id":"Component","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":4,"Label":"","Id":"Priority","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":5,"Label":"","Id":"Points","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":6,"Label":"","Id":"PercentComplete","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":7,"Label":"","Id":"StartDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":8,"Label":"","Id":"DueDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":9,"Label":"","Id":"Status","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":10,"Label":"","Id":"Visibility","Type":0}],"View":[{"ProjectGroups":[1],"Required":false,"Sequence":0,"Label":"","Id":"IssueKey","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"Type","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":2,"Label":"","Id":"Title","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":3,"Label":"","Id":"FixedInVersion","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":4,"Label":"","Id":"23","Type":1},{"ProjectGroups":[1],"Required":false,"Sequence":5,"Label":"","Id":"Component","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":6,"Label":"","Id":"Priority","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":7,"Label":"","Id":"Points","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":8,"Label":"","Id":"PercentComplete","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":9,"Label":"","Id":"StartDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":10,"Label":"","Id":"DueDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":11,"Label":"","Id":"Status","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":12,"Label":"","Id":"Resolution","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":13,"Label":"","Id":"CalculatedTimeLogged","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":14,"Label":"","Id":"Visibility","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":15,"Label":"","Id":"AssignedTo","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":16,"Label":"","Id":"DateCreated","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":17,"Label":"","Id":"DateRevised","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":18,"Label":"","Id":"Description","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":19,"Label":"","Id":"AssociatedComments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":20,"Label":"","Id":"AssociatedHierarchy","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":21,"Label":"","Id":"AssociatedLinks","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":22,"Label":"","Id":"AssociatedAttachments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":23,"Label":"","Id":"AssociatedTime","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":24,"Label":"","Id":"AssociatedSourceControl","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":25,"Label":"","Id":"AssociatedHistory","Type":0}],"IsNew":true,"IsExisting":false,"Active":true,"Archived":false,"Deleted":false,"Created":"2012-08-01T10:18:20.5330421Z","Revised":"2012-08-01T10:18:20.5330421Z","Errors":[],"HasErrors":false,"Id":0}','{"ReferenceId":*STORY*,"Status":[{"StatusId":*TESTED*,"Top":564,"Left":423,"Transitions":[{"StatusId":*INPROGRESS*,"Groups":[1]}]},{"StatusId":*CLOSED*,"Top":523,"Left":726,"Transitions":[]},{"StatusId":*INBACKLOG*,"Top":67,"Left":345,"Transitions":[]},{"StatusId":*INPROGRESS*,"Top":10,"Left":760,"Transitions":[{"StatusId":*CLOSED*,"Groups":[1]}]},{"StatusId":*INSPRINT*,"Top":498,"Left":91,"Transitions":[{"StatusId":*TESTED*,"Groups":[1]},{"StatusId":*INBACKLOG*,"Groups":[1]}]},{"StatusId":*INPROGRESS*,"Groups":[1]}]}]}','','10')
INSERT gemini_issuetypes(seq,typedesc,imagepath,color,screen,workflow,tag,templateid) VALUES('1','Investigation','assets/images/meta/HELPDESK/type-investigation.png','#ffd000','{"ReferenceId":0,"Create":[{"ProjectGroups":[1],"Required":false,"Sequence":0,"Label":"","Id":"Type","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"Title","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":2,"Label":"","Id":"Description","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":3,"Label":"","Id":"AssociatedAttachments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":4,"Label":"","Id":"Component","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":5,"Label":"","Id":"Priority","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":6,"Label":"","Id":"Status","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":7,"Label":"","Id":"Visibility","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":8,"Label":"","Id":"AssociatedComments","Type":0}],"Edit":[{"ProjectGroups":[1],"Required":false,"Sequence":0,"Label":"","Id":"Type","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"Title","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":2,"Label":"","Id":"Description","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":3,"Label":"","Id":"AssociatedAttachments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":4,"Label":"","Id":"Component","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":5,"Label":"","Id":"Priority","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":6,"Label":"","Id":"Status","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":7,"Label":"","Id":"Visibility","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":8,"Label":"","Id":"Votes","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":9,"Label":"","Id":"AssociatedComments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":10,"Label":"","Id":"27","Type":1},{"ProjectGroups":[1],"Required":false,"Sequence":11,"Label":"","Id":"DueDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":12,"Label":"","Id":"29","Type":1},{"ProjectGroups":[1],"Required":false,"Sequence":13,"Label":"","Id":"Resolution","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":14,"Label":"","Id":"AssignedTo","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":15,"Label":"","Id":"StartDate","Type":0}],"View":[{"ProjectGroups":[1],"Required":false,"Sequence":0,"Label":"","Id":"IssueKey","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"Type","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":2,"Label":"","Id":"Title","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":3,"Label":"","Id":"29","Type":1},{"ProjectGroups":[1],"Required":false,"Sequence":4,"Label":"","Id":"27","Type":1},{"ProjectGroups":[1],"Required":false,"Sequence":5,"Label":"","Id":"Component","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":6,"Label":"","Id":"Priority","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":7,"Label":"","Id":"Status","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":8,"Label":"","Id":"ClosedDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":9,"Label":"","Id":"DateCreated","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":10,"Label":"","Id":"StartDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":11,"Label":"","Id":"DueDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":12,"Label":"","Id":"ReportedBy","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":13,"Label":"","Id":"Resolution","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":14,"Label":"","Id":"ResolvedDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":15,"Label":"","Id":"AssignedTo","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":16,"Label":"","Id":"Description","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":17,"Label":"","Id":"AssociatedAttachments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":18,"Label":"","Id":"AssociatedComments","Type":0}],"IsNew":true,"IsExisting":false,"Active":true,"Archived":false,"Deleted":false,"Created":"2012-08-02T10:20:10.2578927Z","Revised":"2012-08-02T10:20:10.2578927Z","Errors":[],"HasErrors":false,"Id":0}','{"ReferenceId":*SUPPORTREQUEST*,"Status":[{"StatusId":*INPROGRESS*,"Groups":[1]}]},{"StatusId":*CLOSED*,"Top":10,"Left":310,"Transitions":[]},{"StatusId":*INPROGRESS*,"Top":217,"Left":383,"Transitions":[{"StatusId":*CLOSED*,"Groups":[1]}]},{"StatusId":*UNASSIGNED*,"Top":0,"Left":0,"Transitions":[]}]}','','12')
INSERT gemini_issuetypes(seq,typedesc,imagepath,color,screen,workflow,tag,templateid) VALUES('4','Task','assets/images/meta/AGILE/type-task.png','#e68012','{"ReferenceId":0,"Create":[{"ProjectGroups":[1],"Required":false,"Sequence":0,"Label":"","Id":"Type","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"Title","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":2,"Label":"","Id":"Description","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":3,"Label":"","Id":"StartDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":4,"Label":"","Id":"Status","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":5,"Label":"","Id":"DueDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":6,"Label":"","Id":"AssociatedAttachments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":7,"Label":"","Id":"AssociatedComments","Type":0}],"Edit":[{"ProjectGroups":[1],"Required":false,"Sequence":0,"Label":"","Id":"Title","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"Description","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":2,"Label":"","Id":"StartDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":3,"Label":"","Id":"Status","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":4,"Label":"","Id":"DueDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":5,"Label":"","Id":"AssociatedAttachments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":6,"Label":"","Id":"AssociatedComments","Type":0}],"View":[{"ProjectGroups":[1],"Required":false,"Sequence":0,"Label":"","Id":"Title","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"StartDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":2,"Label":"","Id":"Status","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":3,"Label":"","Id":"DueDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":4,"Label":"","Id":"CalculatedTimeLogged","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":5,"Label":"","Id":"IssueKey","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":6,"Label":"","Id":"Description","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":7,"Label":"","Id":"AssociatedAttachments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":8,"Label":"","Id":"AssociatedComments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":9,"Label":"","Id":"AssociatedTime","Type":0}],"IsNew":true,"IsExisting":false,"Active":true,"Archived":false,"Deleted":false,"Created":"2012-08-02T10:19:28.7685196Z","Revised":"2012-08-02T10:19:28.7685196Z","Errors":[],"HasErrors":false,"Id":0}','{"ReferenceId":0,"Status":[{"StatusId":*ESCALATION-LEVEL1*,"Top":24,"Left":37,"Transitions":[]},{"StatusId":*ESCALATION-LEVEL2*,"Top":10,"Left":245,"Transitions":[]},{"StatusId":*MOVEDTODELIVERY*,"Top":198,"Left":478,"Transitions":[{"StatusId":*USERACCEPTED*,"Groups":[1]},{"StatusId":*USERREJECTED*,"Groups":[1]}]},{"StatusId":*RAISED*,"Top":402,"Left":88,"Transitions":[{"StatusId":*ASSIGNED*,"Groups":[1]},{"StatusId":*APPROVED*,"Groups":[1]},{"StatusId":*PENDINGAPPROVAL*,"Groups":[1]}]},{"StatusId":*USERREJECTED*,"Top":142,"Left":275,"Transitions":[{"StatusId":*ASSIGNED*,"Groups":[1]}]},{"StatusId":*ASSIGNED*,"Top":298,"Left":378,"Transitions":[{"StatusId":*CLOSED*,"Groups":[1]},{"StatusId":*MOVEDTODELIVERY*,"Groups":[1]}]},{"StatusId":*CLOSED*,"Top":287,"Left":589,"Transitions":[]},{"StatusId":*CASEREOPENED*,"Top":80,"Left":160,"Transitions":[]},{"StatusId":*USERACCEPTED*,"Top":139,"Left":652,"Transitions":[{"StatusId":*CLOSED*,"Groups":[1]}]},{"StatusId":*PENDINGAPPROVAL*,"Top":281,"Left":20,"Transitions":[{"StatusId":*APPROVED*,"Groups":[1]}]},{"StatusId":*APPROVED*,"Top":208,"Left":132,"Transitions":[{"StatusId":*ASSIGNED*,"Groups":[1]}]}]}','','11')
INSERT gemini_issuetypes(seq,typedesc,imagepath,color,screen,workflow,tag,templateid) VALUES('4','Change Request','assets/images/meta/HELPDESK/type-changerequest.png','#171517','{"ReferenceId":*INVESTIGATION*,"Create":[{"ProjectGroups":[1],"Required":false,"Sequence":0,"Label":"","Id":"Type","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"Description","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":2,"Label":"","Id":"DueDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":3,"Label":"","Id":"EstimatedEffort","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":4,"Label":"","Id":"Points","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":5,"Label":"","Id":"Priority","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":6,"Label":"","Id":"Visibility","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":7,"Label":"","Id":"AssociatedAttachments","Type":0}],"Edit":[{"ProjectGroups":[1],"Required":false,"Sequence":0,"Label":"","Id":"Description","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"DueDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":2,"Label":"","Id":"EstimatedEffort","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":3,"Label":"","Id":"Points","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":4,"Label":"","Id":"Priority","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":5,"Label":"","Id":"Votes","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":6,"Label":"","Id":"AssociatedAttachments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":7,"Label":"","Id":"AssociatedComments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":8,"Label":"","Id":"AssociatedHierarchy","Type":0}],"View":[{"ProjectGroups":[1],"Required":false,"Sequence":0,"Label":"","Id":"Type","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"DueDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":2,"Label":"","Id":"EstimatedEffort","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":3,"Label":"","Id":"Points","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":4,"Label":"","Id":"Priority","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":5,"Label":"","Id":"Votes","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":6,"Label":"","Id":"IssueKey","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":7,"Label":"","Id":"Description","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":8,"Label":"","Id":"AssociatedAttachments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":9,"Label":"","Id":"AssociatedComments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":10,"Label":"","Id":"AssociatedHierarchy","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":11,"Label":"","Id":"AssociatedHistory","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":12,"Label":"","Id":"AssociatedLinks","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":13,"Label":"","Id":"AssociatedWatchers","Type":0}],"IsNew":true,"IsExisting":false,"Active":true,"Archived":false,"Deleted":false,"Created":"2012-08-01T19:03:41.8119561Z","Revised":"2012-08-01T19:03:41.8119561Z","Errors":[],"HasErrors":false,"Id":0}','{"ReferenceId":*SUPPORTREQUEST*,"Status":[{"StatusId":*APPROVED*,"Top":0,"Left":9,"Transitions":[{"StatusId":*INPROGRESS*,"Groups":[1]}]},{"StatusId":*INPROGRESS*,"Groups":[1]}]},{"StatusId":*CLOSED*,"Top":0,"Left":0,"Transitions":[]},{"StatusId":*INPROGRESS*,"Top":0,"Left":0,"Transitions":[]},{"StatusId":*UNASSIGNED*,"Top":0,"Left":0,"Transitions":[]}]}','','12')
INSERT gemini_issuetypes(seq,typedesc,imagepath,color,screen,workflow,tag,templateid) VALUES('1','Test Plan','assets/images/meta/SENTRY/type-investigation.png','#374fe8','{"ReferenceId":0,"Create":[{"ProjectGroups":[1],"Required":false,"Sequence":0,"Label":"","Id":"Type","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"Title","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":2,"Label":"","Id":"Description","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":3,"Label":"","Id":"Priority","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":4,"Label":"","Id":"Severity","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":5,"Label":"","Id":"ReportedBy","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":6,"Label":"","Id":"PercentComplete","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":7,"Label":"","Id":"AssociatedComments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":8,"Label":"","Id":"StartDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":9,"Label":"","Id":"DueDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":10,"Label":"","Id":"AssociatedAttachments","Type":0}],"Edit":[{"ProjectGroups":[1],"Required":false,"Sequence":0,"Label":"","Id":"Type","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"Title","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":2,"Label":"","Id":"Priority","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":3,"Label":"","Id":"Severity","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":4,"Label":"","Id":"ReportedBy","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":5,"Label":"","Id":"PercentComplete","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":6,"Label":"","Id":"AssociatedComments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":7,"Label":"","Id":"StartDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":8,"Label":"","Id":"DueDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":9,"Label":"","Id":"AssociatedAttachments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":10,"Label":"","Id":"Source","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":11,"Label":"","Id":"Status","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":12,"Label":"","Id":"Resolution","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":13,"Label":"","Id":"AssignedTo","Type":0}],"View":[{"ProjectGroups":[1],"Required":false,"Sequence":0,"Label":"","Id":"Type","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"Title","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":2,"Label":"","Id":"Priority","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":3,"Label":"","Id":"Severity","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":4,"Label":"","Id":"ReportedBy","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":5,"Label":"","Id":"PercentComplete","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":6,"Label":"","Id":"StartDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":7,"Label":"","Id":"DueDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":8,"Label":"","Id":"Source","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":9,"Label":"","Id":"Status","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":10,"Label":"","Id":"Resolution","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":11,"Label":"","Id":"AssignedTo","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":12,"Label":"","Id":"DateCreated","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":13,"Label":"","Id":"ClosedDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":14,"Label":"","Id":"ResolvedDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":15,"Label":"","Id":"CalculatedTimeLogged","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":16,"Label":"","Id":"IssueKey","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":17,"Label":"","Id":"Description","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":18,"Label":"","Id":"AssociatedAttachments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":19,"Label":"","Id":"AssociatedComments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":20,"Label":"","Id":"AssociatedLinks","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":21,"Label":"","Id":"598B4999-4C31-427D-9BBC-C84AD954E55A","Type":2},{"ProjectGroups":[1],"Required":false,"Sequence":22,"Label":"","Id":"EFEE5D50-7DF6-49C3-9D69-E7EFC1AFEF72","Type":2},{"ProjectGroups":[1],"Required":false,"Sequence":23,"Label":"","Id":"AC93C4F9-B512-4A3D-8B37-BC3D5F83429D","Type":2},{"ProjectGroups":[1],"Required":false,"Sequence":24,"Label":"","Id":"D8DECEB7-BA6D-4A3F-A4A7-C79A5A5585BA","Type":2}],"IsNew":true,"IsExisting":false,"Active":true,"Archived":false,"Deleted":false,"Created":"2012-08-13T11:50:23.7683507Z","Revised":"2012-08-13T11:50:23.7683507Z","Errors":[],"HasErrors":false,"Id":0}','{"ReferenceId":0,"Status":[{"StatusId":*REPORTED*,"Top":10,"Left":10,"Transitions":[{"StatusId":*PLANNED*,"Groups":[1]},{"StatusId":*ASSESSING*,"Groups":[1]}]},{"StatusId":*PLANNED*,"Top":10,"Left":160,"Transitions":[{"StatusId":*REPORTED*,"Groups":[1]}]},{"StatusId":*PRODUCTIONREADY*,"Top":10,"Left":310,"Transitions":[]},{"StatusId":*ASSESSING*,"Top":94,"Left":32,"Transitions":[{"StatusId":*REJECTED*,"Groups":[1]},{"StatusId":*ACCEPTED*,"Groups":[1]},{"StatusId":*REPORTED*,"Groups":[1]}]},{"StatusId":*ACCEPTED*,"Top":253,"Left":124,"Transitions":[{"StatusId":*INPROGRESS*,"Groups":[1]},{"StatusId":*ASSESSING*,"Groups":[1]}]},{"StatusId":*REJECTED*,"Top":169,"Left":274,"Transitions":[{"StatusId":*ACCEPTED*,"Groups":[1]}]},{"StatusId":*INPROGRESS*,"Top":247,"Left":458,"Transitions":[{"StatusId":*READYFORDEPLOY*,"Groups":[1]},{"StatusId":*REJECTED*,"Groups":[1]},{"StatusId":*ACCEPTED*,"Groups":[1]}]},{"StatusId":*READYFORDEPLOY*,"Top":517,"Left":284,"Transitions":[{"StatusId":*DEPLOYED*,"Groups":[1]},{"StatusId":*INPROGRESS*,"Groups":[1]}]},{"StatusId":*DEPLOYED*,"Top":570,"Left":608,"Transitions":[{"StatusId":*CLOSED*,"Groups":[1]},{"StatusId":*READYFORDEPLOY*,"Groups":[1]}]},{"StatusId":*CLOSED*,"Top":452,"Left":687,"Transitions":[]}]}','PLAN','16')
INSERT gemini_issuetypes(seq,typedesc,imagepath,color,screen,workflow,tag,templateid) VALUES('4','Change Request','assets/images/meta/AGILE/type-changerequest.png','#171517','{"ReferenceId":*STORY*,"Create":[{"ProjectGroups":[1],"Required":false,"Sequence":0,"Label":"","Id":"Type","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"Title","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":2,"Label":"","Id":"Description","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":3,"Label":"","Id":"FixedInVersion","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":4,"Label":"","Id":"23","Type":1},{"ProjectGroups":[1],"Required":false,"Sequence":5,"Label":"","Id":"Component","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":6,"Label":"","Id":"Priority","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":7,"Label":"","Id":"Points","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":8,"Label":"","Id":"StartDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":9,"Label":"","Id":"DueDate","Type":0}],"Edit":[{"ProjectGroups":[1],"Required":false,"Sequence":0,"Label":"","Id":"Type","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"FixedInVersion","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":2,"Label":"","Id":"23","Type":1},{"ProjectGroups":[1],"Required":false,"Sequence":3,"Label":"","Id":"Component","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":4,"Label":"","Id":"Priority","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":5,"Label":"","Id":"Points","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":6,"Label":"","Id":"PercentComplete","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":7,"Label":"","Id":"StartDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":8,"Label":"","Id":"DueDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":9,"Label":"","Id":"Status","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":10,"Label":"","Id":"Visibility","Type":0}],"View":[{"ProjectGroups":[1],"Required":false,"Sequence":0,"Label":"","Id":"IssueKey","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"Type","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":2,"Label":"","Id":"Title","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":3,"Label":"","Id":"FixedInVersion","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":4,"Label":"","Id":"23","Type":1},{"ProjectGroups":[1],"Required":false,"Sequence":5,"Label":"","Id":"Component","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":6,"Label":"","Id":"Priority","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":7,"Label":"","Id":"Points","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":8,"Label":"","Id":"PercentComplete","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":9,"Label":"","Id":"StartDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":10,"Label":"","Id":"DueDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":11,"Label":"","Id":"Status","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":12,"Label":"","Id":"Resolution","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":13,"Label":"","Id":"CalculatedTimeLogged","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":14,"Label":"","Id":"Visibility","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":15,"Label":"","Id":"AssignedTo","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":16,"Label":"","Id":"DateCreated","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":17,"Label":"","Id":"DateRevised","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":18,"Label":"","Id":"Description","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":19,"Label":"","Id":"AssociatedComments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":20,"Label":"","Id":"AssociatedHierarchy","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":21,"Label":"","Id":"AssociatedLinks","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":22,"Label":"","Id":"AssociatedAttachments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":23,"Label":"","Id":"AssociatedTime","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":24,"Label":"","Id":"AssociatedSourceControl","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":25,"Label":"","Id":"AssociatedHistory","Type":0}],"IsNew":true,"IsExisting":false,"Active":true,"Archived":false,"Deleted":false,"Created":"2012-08-01T10:18:16.5718155Z","Revised":"2012-08-01T10:18:16.5718155Z","Errors":[],"HasErrors":false,"Id":0}','{"ReferenceId":*STORY*,"Status":[{"StatusId":*INBACKLOG*,"Top":0,"Left":0,"Transitions":[{"StatusId":*INSPRINT*,"Groups":[1]}]},{"StatusId":*INSPRINT*,"Top":0,"Left":0,"Transitions":[{"StatusId":*TESTED*,"Groups":[1]},{"StatusId":*INPROGRESS*,"Groups":[1]}]},{"StatusId":*TESTED*,"Top":10,"Left":610,"Transitions":[{"StatusId":*INPROGRESS*,"Groups":[1]}]},{"StatusId":*INPROGRESS*,"Top":10,"Left":760,"Transitions":[{"StatusId":*CLOSED*,"Groups":[1]}]},{"StatusId":*CLOSED*,"Top":0,"Left":0,"Transitions":[]}]}','','10')
INSERT gemini_issuetypes(seq,typedesc,imagepath,color,screen,workflow,tag,templateid) VALUES('3','New Requirement','assets/images/meta/ENQUIRY/status-ready-for-approval.png','#18ad59','{"ReferenceId":*OPENISSUE*,"Create":[{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"Type","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":2,"Label":"","Id":"Title","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":3,"Label":"","Id":"Description","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":4,"Label":"","Id":"Priority","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":5,"Label":"","Id":"39","Type":1},{"ProjectGroups":[1],"Required":false,"Sequence":6,"Label":"","Id":"40","Type":1},{"ProjectGroups":[1],"Required":false,"Sequence":7,"Label":"","Id":"AssociatedComments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":8,"Label":"","Id":"EstimatedEffort","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":9,"Label":"","Id":"StartDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":10,"Label":"","Id":"DueDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":11,"Label":"","Id":"AssociatedAttachments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":12,"Label":"","Id":"AssociatedWatchers","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":13,"Label":"","Id":"Visibility","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":14,"Label":"","Id":"AssociatedHierarchy","Type":0}],"Edit":[{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"Type","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":2,"Label":"","Id":"Title","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":3,"Label":"","Id":"Description","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":4,"Label":"","Id":"Priority","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":5,"Label":"","Id":"39","Type":1},{"ProjectGroups":[1],"Required":false,"Sequence":6,"Label":"","Id":"40","Type":1},{"ProjectGroups":[1],"Required":false,"Sequence":7,"Label":"","Id":"AssociatedComments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":8,"Label":"","Id":"EstimatedEffort","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":9,"Label":"","Id":"StartDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":10,"Label":"","Id":"DueDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":11,"Label":"","Id":"AssociatedAttachments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":12,"Label":"","Id":"AssociatedWatchers","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":13,"Label":"","Id":"Visibility","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":14,"Label":"","Id":"AssociatedHierarchy","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":15,"Label":"","Id":"Status","Type":0}],"View":[{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"Type","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":2,"Label":"","Id":"Title","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":3,"Label":"","Id":"Description","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":4,"Label":"","Id":"Priority","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":5,"Label":"","Id":"39","Type":1},{"ProjectGroups":[1],"Required":false,"Sequence":6,"Label":"","Id":"40","Type":1},{"ProjectGroups":[1],"Required":false,"Sequence":7,"Label":"","Id":"AssociatedComments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":8,"Label":"","Id":"EstimatedEffort","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":9,"Label":"","Id":"StartDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":10,"Label":"","Id":"DueDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":11,"Label":"","Id":"AssociatedAttachments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":12,"Label":"","Id":"AssociatedWatchers","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":13,"Label":"","Id":"AssociatedHierarchy","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":14,"Label":"","Id":"Status","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":15,"Label":"","Id":"IssueKey","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":16,"Label":"","Id":"AssociatedHistory","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":17,"Label":"","Id":"AssociatedLinks","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":18,"Label":"","Id":"AssociatedSourceControl","Type":0}],"IsNew":true,"IsExisting":false,"Active":true,"Archived":false,"Deleted":false,"Created":"2012-08-01T09:00:22.5454765Z","Revised":"2012-08-01T09:00:22.5454765Z","Errors":[],"HasErrors":false,"Id":0}','{"ReferenceId":0,"Status":[{"StatusId":*ESCALATION-LEVEL1*,"Top":10,"Left":10,"Transitions":[]},{"StatusId":*ESCALATION-LEVEL2*,"Top":10,"Left":160,"Transitions":[]},{"StatusId":*MOVEDTODELIVERY*,"Top":437,"Left":709,"Transitions":[{"StatusId":*CLOSED*,"Groups":[1]}]},{"StatusId":*RAISED*,"Top":194,"Left":62,"Transitions":[{"StatusId":*PENDINGAPPROVAL*,"Groups":[1]}]},{"StatusId":*USERREJECTED*,"Top":152,"Left":441,"Transitions":[]},{"StatusId":*ASSIGNED*,"Top":281,"Left":620,"Transitions":[{"StatusId":*CLOSED*,"Groups":[1]},{"StatusId":*MOVEDTODELIVERY*,"Groups":[1]}]},{"StatusId":*CLOSED*,"Top":431,"Left":439,"Transitions":[]},{"StatusId":*CASEREOPENED*,"Top":86,"Left":229,"Transitions":[]},{"StatusId":*USERACCEPTED*,"Top":94,"Left":30,"Transitions":[]},{"StatusId":*PENDINGAPPROVAL*,"Top":316,"Left":143,"Transitions":[{"StatusId":*APPROVED*,"Groups":[1]}]},{"StatusId":*APPROVED*,"Top":265,"Left":391,"Transitions":[{"StatusId":*ASSIGNED*,"Groups":[1]}]}]}','','11')
INSERT gemini_issuetypes(seq,typedesc,imagepath,color,screen,workflow,tag,templateid) VALUES('5','Task','assets/images/meta/HELPDESK/type-task.png','#e68012','{"ReferenceId":*INVESTIGATION*,"Create":[{"ProjectGroups":[1],"Required":false,"Sequence":0,"Label":"","Id":"Type","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"Title","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":2,"Label":"","Id":"Description","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":3,"Label":"","Id":"StartDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":4,"Label":"","Id":"Status","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":5,"Label":"","Id":"DueDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":6,"Label":"","Id":"AssociatedAttachments","Type":0}],"Edit":[{"ProjectGroups":[1],"Required":false,"Sequence":0,"Label":"","Id":"Title","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"Description","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":2,"Label":"","Id":"StartDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":3,"Label":"","Id":"Status","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":4,"Label":"","Id":"DueDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":5,"Label":"","Id":"AssociatedAttachments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":6,"Label":"","Id":"AssociatedComments","Type":0}],"View":[{"ProjectGroups":[1],"Required":false,"Sequence":0,"Label":"","Id":"Title","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"StartDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":2,"Label":"","Id":"Status","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":3,"Label":"","Id":"DueDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":4,"Label":"","Id":"CalculatedTimeLogged","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":5,"Label":"","Id":"IssueKey","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":6,"Label":"","Id":"Description","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":7,"Label":"","Id":"AssociatedAttachments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":8,"Label":"","Id":"AssociatedComments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":9,"Label":"","Id":"AssociatedTime","Type":0}],"IsNew":true,"IsExisting":false,"Active":true,"Archived":false,"Deleted":false,"Created":"2012-08-02T10:20:19.2254056Z","Revised":"2012-08-02T10:20:19.2254056Z","Errors":[],"HasErrors":false,"Id":0}','{"ReferenceId":*SUPPORTREQUEST*,"Status":[{"StatusId":*APPROVED*,"Top":0,"Left":0,"Transitions":[]},{"StatusId":*CLOSED*,"Top":0,"Left":0,"Transitions":[]},{"StatusId":*INPROGRESS*,"Top":0,"Left":0,"Transitions":[{"StatusId":*CLOSED*,"Groups":[1]}]},{"StatusId":*UNASSIGNED*,"Top":0,"Left":0,"Transitions":[]}]}','','12')
INSERT gemini_issuetypes(seq,typedesc,imagepath,color,screen,workflow,tag,templateid) VALUES('4','Change Request','assets/images/meta/ISSUES/type-changerequest.png','#171517','{"ReferenceId":0,"Create":[{"ProjectGroups":[1],"Required":false,"Sequence":0,"Label":"","Id":"Type","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"Description","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":2,"Label":"","Id":"DueDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":3,"Label":"","Id":"EstimatedEffort","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":4,"Label":"","Id":"Points","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":5,"Label":"","Id":"Priority","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":6,"Label":"","Id":"Visibility","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":7,"Label":"","Id":"AssociatedAttachments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":8,"Label":"","Id":"AssociatedComments","Type":0}],"Edit":[{"ProjectGroups":[1],"Required":false,"Sequence":0,"Label":"","Id":"Description","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"DueDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":2,"Label":"","Id":"EstimatedEffort","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":3,"Label":"","Id":"Points","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":4,"Label":"","Id":"Priority","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":5,"Label":"","Id":"Votes","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":6,"Label":"","Id":"AssociatedAttachments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":7,"Label":"","Id":"AssociatedComments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":8,"Label":"","Id":"AssociatedHierarchy","Type":0}],"View":[{"ProjectGroups":[1],"Required":false,"Sequence":0,"Label":"","Id":"Type","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"DueDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":2,"Label":"","Id":"EstimatedEffort","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":3,"Label":"","Id":"Points","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":4,"Label":"","Id":"Priority","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":5,"Label":"","Id":"Votes","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":6,"Label":"","Id":"IssueKey","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":7,"Label":"","Id":"Description","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":8,"Label":"","Id":"AssociatedAttachments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":9,"Label":"","Id":"AssociatedComments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":10,"Label":"","Id":"AssociatedHierarchy","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":11,"Label":"","Id":"AssociatedHistory","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":12,"Label":"","Id":"AssociatedLinks","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":13,"Label":"","Id":"AssociatedWatchers","Type":0}],"IsNew":true,"IsExisting":false,"Active":true,"Archived":false,"Deleted":false,"Created":"2012-08-02T10:19:47.988619Z","Revised":"2012-08-02T10:19:47.988619Z","Errors":[],"HasErrors":false,"Id":0}','{"ReferenceId":*NEWFEATURE*,"Status":[{"StatusId":*ASSIGNED*,"Top":10,"Left":160,"Transitions":[]},{"StatusId":*CLOSED*,"Top":10,"Left":310,"Transitions":[]},{"StatusId":*INPROGRESS*,"Top":0,"Left":0,"Transitions":[{"StatusId":*CLOSED*,"Groups":[1]}]},{"StatusId":*REOPENED*,"Top":80,"Left":10,"Transitions":[]},{"StatusId":*UNASSIGNED*,"Top":80,"Left":160,"Transitions":[{"StatusId":*INPROGRESS*,"Groups":[1]},{"StatusId":*ASSIGNED*,"Groups":[1]}]}]}','','13')
INSERT gemini_issuetypes(seq,typedesc,imagepath,color,screen,workflow,tag,templateid) VALUES('3','Support Request','assets/images/meta/HELPDESK/type-enhancement.png','#e612e6','{"ReferenceId":*INVESTIGATION*,"Create":[{"ProjectGroups":[1],"Required":false,"Sequence":0,"Label":"","Id":"Type","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"Title","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":2,"Label":"","Id":"Description","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":3,"Label":"","Id":"Priority","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":4,"Label":"","Id":"EstimatedEffort","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":5,"Label":"","Id":"StartDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":6,"Label":"","Id":"DueDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":7,"Label":"","Id":"Points","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":8,"Label":"","Id":"PercentComplete","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":9,"Label":"","Id":"AssociatedAttachments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":10,"Label":"","Id":"AffectedVersions","Type":0}],"Edit":[{"ProjectGroups":[1],"Required":false,"Sequence":0,"Label":"","Id":"Type","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"Title","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":2,"Label":"","Id":"Description","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":3,"Label":"","Id":"Priority","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":4,"Label":"","Id":"EstimatedEffort","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":5,"Label":"","Id":"StartDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":6,"Label":"","Id":"DueDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":7,"Label":"","Id":"Points","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":8,"Label":"","Id":"PercentComplete","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":9,"Label":"","Id":"AssociatedAttachments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":10,"Label":"","Id":"AffectedVersions","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":11,"Label":"","Id":"AssociatedWatchers","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":12,"Label":"","Id":"Status","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":13,"Label":"","Id":"AssociatedComments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":14,"Label":"","Id":"Visibility","Type":0}],"View":[{"ProjectGroups":[1],"Required":false,"Sequence":0,"Label":"","Id":"Type","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"Title","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":2,"Label":"","Id":"Priority","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":3,"Label":"","Id":"EstimatedEffort","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":4,"Label":"","Id":"StartDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":5,"Label":"","Id":"DueDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":6,"Label":"","Id":"IssueKey","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":7,"Label":"","Id":"Points","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":8,"Label":"","Id":"PercentComplete","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":9,"Label":"","Id":"AffectedVersions","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":10,"Label":"","Id":"Status","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":11,"Label":"","Id":"CalculatedTimeLogged","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":12,"Label":"","Id":"DateCreated","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":13,"Label":"","Id":"Description","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":14,"Label":"","Id":"AssociatedAttachments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":15,"Label":"","Id":"AssociatedWatchers","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":16,"Label":"","Id":"AssociatedComments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":17,"Label":"","Id":"AssociatedHistory","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":18,"Label":"","Id":"AssociatedTime","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":19,"Label":"","Id":"AssociatedHierarchy","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":20,"Label":"","Id":"AssociatedLinks","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":21,"Label":"","Id":"AssociatedSourceControl","Type":0}],"IsNew":true,"IsExisting":false,"Active":true,"Archived":false,"Deleted":false,"Created":"2012-08-01T19:03:38.1437463Z","Revised":"2012-08-01T19:03:38.1437463Z","Errors":[],"HasErrors":false,"Id":0}','{"ReferenceId":0,"Status":[{"StatusId":*UNASSIGNED*,"Top":112,"Left":156,"Transitions":[{"StatusId":*APPROVED*,"Groups":[1]},{"StatusId":*REJECTED*,"Groups":[1]}]},{"StatusId":*APPROVED*,"Top":275,"Left":28,"Transitions":[{"StatusId":*INPROGRESS*,"Groups":[1]}]},{"StatusId":*REJECTED*,"Top":103,"Left":392,"Transitions":[{"StatusId":*INPROGRESS*,"Groups":[1]}]},{"StatusId":*INPROGRESS*,"Top":390,"Left":359,"Transitions":[{"StatusId":*CLOSED*,"Groups":[1]}]},{"StatusId":*CLOSED*,"Top":254,"Left":557,"Transitions":[]}]}','','12')
INSERT gemini_issuetypes(seq,typedesc,imagepath,color,screen,workflow,tag,templateid) VALUES('2','Story','assets/images/meta/AGILE/type-enhancement.png','#e612e6','{"ReferenceId":0,"Create":[{"ProjectGroups":[1],"Required":false,"Sequence":0,"Label":"","Id":"Type","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"Title","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":2,"Label":"","Id":"Description","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":3,"Label":"","Id":"FixedInVersion","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":4,"Label":"","Id":"Component","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":5,"Label":"","Id":"Priority","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":6,"Label":"","Id":"Points","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":7,"Label":"","Id":"StartDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":8,"Label":"","Id":"DueDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":9,"Label":"","Id":"AssociatedComments","Type":0}],"Edit":[{"ProjectGroups":[1],"Required":false,"Sequence":0,"Label":"","Id":"Type","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"FixedInVersion","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":2,"Label":"","Id":"Component","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":3,"Label":"","Id":"Priority","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":4,"Label":"","Id":"Points","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":5,"Label":"","Id":"PercentComplete","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":6,"Label":"","Id":"StartDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":7,"Label":"","Id":"DueDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":8,"Label":"","Id":"Status","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":9,"Label":"","Id":"Visibility","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":10,"Label":"","Id":"Source","Type":0}],"View":[{"ProjectGroups":[1],"Required":false,"Sequence":0,"Label":"","Id":"IssueKey","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"Type","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":2,"Label":"","Id":"Title","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":3,"Label":"","Id":"FixedInVersion","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":4,"Label":"","Id":"Component","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":5,"Label":"","Id":"Priority","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":6,"Label":"","Id":"Points","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":7,"Label":"","Id":"PercentComplete","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":8,"Label":"","Id":"StartDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":9,"Label":"","Id":"DueDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":10,"Label":"","Id":"Status","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":11,"Label":"","Id":"Resolution","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":12,"Label":"","Id":"CalculatedTimeLogged","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":13,"Label":"","Id":"Visibility","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":14,"Label":"","Id":"AssignedTo","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":15,"Label":"","Id":"DateCreated","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":16,"Label":"","Id":"DateRevised","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":17,"Label":"","Id":"Source","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":18,"Label":"","Id":"Description","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":19,"Label":"","Id":"AssociatedComments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":20,"Label":"","Id":"AssociatedHierarchy","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":21,"Label":"","Id":"AssociatedLinks","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":22,"Label":"","Id":"AssociatedAttachments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":23,"Label":"","Id":"AssociatedTime","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":24,"Label":"","Id":"AssociatedSourceControl","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":25,"Label":"","Id":"AssociatedHistory","Type":0}],"IsNew":true,"IsExisting":false,"Active":true,"Archived":false,"Deleted":false,"Created":"2012-08-13T11:47:13.3984622Z","Revised":"2012-08-13T11:47:13.3984622Z","Errors":[],"HasErrors":false,"Id":0}','{"ReferenceId":0,"Status":[{"StatusId":*INBACKLOG*,"Top":24,"Left":48,"Transitions":[]},{"StatusId":*INSPRINT*,"Top":108,"Left":48,"Transitions":[{"StatusId":*INPROGRESS*,"Groups":[1]}]},{"StatusId":*INPROGRESS*,"Top":288,"Left":219,"Transitions":[{"StatusId":*TESTED*,"Groups":[1]}]},{"StatusId":*TESTED*,"Top":459,"Left":313,"Transitions":[{"StatusId":*CLOSED*,"Groups":[1]}]},{"StatusId":*CLOSED*,"Top":587,"Left":525,"Transitions":[]}]}','','10')
INSERT gemini_issuetypes(seq,typedesc,imagepath,color,screen,workflow,tag,templateid) VALUES('0','New Feature','assets/images/meta/type-newfeature.png','#ed974c','{"ReferenceId":0,"Create":[{"ProjectGroups":[1],"Required":false,"Sequence":0,"Label":"","Id":"Type","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"Title","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":2,"Label":"","Id":"Description","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":3,"Label":"","Id":"Priority","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":4,"Label":"","Id":"Severity","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":5,"Label":"","Id":"Component","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":6,"Label":"","Id":"AffectedVersions","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":7,"Label":"","Id":"Visibility","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":8,"Label":"","Id":"AssociatedComments","Type":0}],"Edit":[{"ProjectGroups":[1],"Required":false,"Sequence":0,"Label":"","Id":"Type","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"Priority","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":2,"Label":"","Id":"Severity","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":3,"Label":"","Id":"Status","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":4,"Label":"","Id":"Resolution","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":5,"Label":"","Id":"Component","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":6,"Label":"","Id":"AffectedVersions","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":7,"Label":"","Id":"FixedInVersion","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":8,"Label":"","Id":"AssignedTo","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":9,"Label":"","Id":"PercentComplete","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":10,"Label":"","Id":"EstimatedEffort","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":11,"Label":"","Id":"StartDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":12,"Label":"","Id":"DueDate","Type":0}],"View":[{"ProjectGroups":[1],"Required":false,"Sequence":0,"Label":"","Id":"IssueKey","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"PercentComplete","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":2,"Label":"","Id":"AffectedVersions","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":3,"Label":"","Id":"ClosedDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":4,"Label":"","Id":"Component","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":5,"Label":"","Id":"DateCreated","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":6,"Label":"","Id":"DueDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":7,"Label":"","Id":"EstimatedEffort","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":8,"Label":"","Id":"FixedInVersion","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":9,"Label":"","Id":"Priority","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":10,"Label":"","Id":"ReportedBy","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":11,"Label":"","Id":"Resolution","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":12,"Label":"","Id":"ResolvedDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":13,"Label":"","Id":"AssignedTo","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":14,"Label":"","Id":"Severity","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":15,"Label":"","Id":"StartDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":16,"Label":"","Id":"Status","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":17,"Label":"","Id":"CalculatedTimeExceeded","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":18,"Label":"","Id":"CalculatedTimeLogged","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":19,"Label":"","Id":"CalculatedTimeRemaining","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":20,"Label":"","Id":"Title","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":21,"Label":"","Id":"Type","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":22,"Label":"","Id":"Visibility","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":23,"Label":"","Id":"Description","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":24,"Label":"","Id":"AssociatedComments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":25,"Label":"","Id":"AssociatedAttachments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":26,"Label":"","Id":"AssociatedHierarchy","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":27,"Label":"","Id":"AssociatedTime","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":28,"Label":"","Id":"AssociatedWatchers","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":29,"Label":"","Id":"AssociatedSourceControl","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":30,"Label":"","Id":"AssociatedHistory","Type":0}],"IsNew":true,"IsExisting":false,"Active":true,"Archived":false,"Deleted":false,"Created":"2012-08-02T10:18:59.4148407Z","Revised":"2012-08-02T10:18:59.4148407Z","Errors":[],"HasErrors":false,"Id":0}','{"ReferenceId":0,"Status":[{"StatusId":*ASSIGNED*,"Top":128,"Left":54,"Transitions":[{"StatusId":*INPROGRESS*,"Groups":[1]}]},{"StatusId":*CLOSED*,"Top":477,"Left":497,"Transitions":[{"StatusId":*REOPENED*,"Groups":[1]}]},{"StatusId":*INPROGRESS*,"Top":297,"Left":259,"Transitions":[{"StatusId":*CLOSED*,"Groups":[1]}]},{"StatusId":*REOPENED*,"Top":126,"Left":673,"Transitions":[{"StatusId":*UNASSIGNED*,"Groups":[1]},{"StatusId":*INPROGRESS*,"Groups":[1]}]},{"StatusId":*UNASSIGNED*,"Top":35,"Left":273,"Transitions":[{"StatusId":*ASSIGNED*,"Groups":[1]},{"StatusId":*INPROGRESS*,"Groups":[1]}]}]}','','13')
INSERT gemini_issuetypes(seq,typedesc,imagepath,color,screen,workflow,tag,templateid) VALUES('1','Investigation','assets/images/meta/ISSUES/type-investigation.png','#ffd000','{"ReferenceId":*NEWFEATURE*,"Create":[{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"Type","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":2,"Label":"","Id":"Title","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":3,"Label":"","Id":"Description","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":4,"Label":"","Id":"26","Type":1},{"ProjectGroups":[1],"Required":false,"Sequence":5,"Label":"","Id":"28","Type":1},{"ProjectGroups":[1],"Required":false,"Sequence":6,"Label":"","Id":"AssociatedAttachments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":7,"Label":"","Id":"Component","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":8,"Label":"","Id":"Priority","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":9,"Label":"","Id":"Status","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":10,"Label":"","Id":"Visibility","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":11,"Label":"","Id":"Votes","Type":0}],"Edit":[{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"Type","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":2,"Label":"","Id":"Title","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":3,"Label":"","Id":"Description","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":4,"Label":"","Id":"26","Type":1},{"ProjectGroups":[1],"Required":false,"Sequence":5,"Label":"","Id":"28","Type":1},{"ProjectGroups":[1],"Required":false,"Sequence":6,"Label":"","Id":"AssociatedAttachments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":7,"Label":"","Id":"Component","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":8,"Label":"","Id":"Priority","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":9,"Label":"","Id":"Status","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":10,"Label":"","Id":"Visibility","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":11,"Label":"","Id":"Votes","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":12,"Label":"","Id":"AssociatedComments","Type":0}],"View":[{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"Type","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":2,"Label":"","Id":"Title","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":3,"Label":"","Id":"Description","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":4,"Label":"","Id":"26","Type":1},{"ProjectGroups":[1],"Required":false,"Sequence":5,"Label":"","Id":"28","Type":1},{"ProjectGroups":[1],"Required":false,"Sequence":6,"Label":"","Id":"AssociatedAttachments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":7,"Label":"","Id":"Component","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":8,"Label":"","Id":"Priority","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":9,"Label":"","Id":"Status","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":10,"Label":"","Id":"Votes","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":11,"Label":"","Id":"AssociatedComments","Type":0}],"IsNew":true,"IsExisting":false,"Active":true,"Archived":false,"Deleted":false,"Created":"2012-08-01T08:48:03.368198Z","Revised":"2012-08-01T08:48:03.368198Z","Errors":[],"HasErrors":false,"Id":0}','{"ReferenceId":*NEWFEATURE*,"Status":[{"StatusId":*ASSIGNED*,"Top":0,"Left":9,"Transitions":[{"StatusId":*INPROGRESS*,"Groups":[1]},{"StatusId":*CLOSED*,"Groups":[1]}]},{"StatusId":*CLOSED*,"Top":407,"Left":0,"Transitions":[{"StatusId":*REOPENED*,"Groups":[1]}]},{"StatusId":*INPROGRESS*,"Top":0,"Left":577,"Transitions":[{"StatusId":*CLOSED*,"Groups":[1]}]},{"StatusId":*REOPENED*,"Top":41,"Left":151,"Transitions":[{"StatusId":*ASSIGNED*,"Groups":[1]},{"StatusId":*UNASSIGNED*,"Groups":[1]}]},{"StatusId":*UNASSIGNED*,"Top":0,"Left":0,"Transitions":[{"StatusId":*ASSIGNED*,"Groups":[1]}]}]}','','13')
INSERT gemini_issuetypes(seq,typedesc,imagepath,color,screen,workflow,tag,templateid) VALUES('3','Enhancement','assets/images/meta/ISSUES/type-enhancement.png','#e612e6','{"ReferenceId":*NEWFEATURE*,"Create":[{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"Type","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":2,"Label":"","Id":"Title","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":3,"Label":"","Id":"Description","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":4,"Label":"","Id":"Priority","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":5,"Label":"","Id":"EstimatedEffort","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":6,"Label":"","Id":"StartDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":7,"Label":"","Id":"DueDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":8,"Label":"","Id":"IssueKey","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":9,"Label":"","Id":"Points","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":10,"Label":"","Id":"PercentComplete","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":11,"Label":"","Id":"AssociatedAttachments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":12,"Label":"","Id":"AffectedVersions","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":13,"Label":"","Id":"AssociatedWatchers","Type":0}],"Edit":[{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"Type","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":2,"Label":"","Id":"Title","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":3,"Label":"","Id":"Description","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":4,"Label":"","Id":"Priority","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":5,"Label":"","Id":"EstimatedEffort","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":6,"Label":"","Id":"StartDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":7,"Label":"","Id":"DueDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":8,"Label":"","Id":"IssueKey","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":9,"Label":"","Id":"Points","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":10,"Label":"","Id":"PercentComplete","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":11,"Label":"","Id":"AssociatedAttachments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":12,"Label":"","Id":"AffectedVersions","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":13,"Label":"","Id":"AssociatedWatchers","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":14,"Label":"","Id":"Status","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":15,"Label":"","Id":"AssociatedComments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":16,"Label":"","Id":"AssociatedHistory","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":17,"Label":"","Id":"CalculatedTimeLogged","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":18,"Label":"","Id":"Visibility","Type":0}],"View":[{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"Type","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":2,"Label":"","Id":"Title","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":3,"Label":"","Id":"Description","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":4,"Label":"","Id":"Priority","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":5,"Label":"","Id":"EstimatedEffort","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":6,"Label":"","Id":"StartDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":7,"Label":"","Id":"DueDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":8,"Label":"","Id":"IssueKey","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":9,"Label":"","Id":"Points","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":10,"Label":"","Id":"PercentComplete","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":11,"Label":"","Id":"AssociatedAttachments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":12,"Label":"","Id":"AffectedVersions","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":13,"Label":"","Id":"AssociatedWatchers","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":14,"Label":"","Id":"Status","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":15,"Label":"","Id":"AssociatedComments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":16,"Label":"","Id":"AssociatedHistory","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":17,"Label":"","Id":"CalculatedTimeLogged","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":18,"Label":"","Id":"DateCreated","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":19,"Label":"","Id":"AssociatedTime","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":20,"Label":"","Id":"AssociatedHierarchy","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":21,"Label":"","Id":"AssociatedLinks","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":22,"Label":"","Id":"AssociatedSourceControl","Type":0}],"IsNew":true,"IsExisting":false,"Active":true,"Archived":false,"Deleted":false,"Created":"2012-08-01T08:48:37.9301748Z","Revised":"2012-08-01T08:48:37.9301748Z","Errors":[],"HasErrors":false,"Id":0}','{"ReferenceId":*NEWFEATURE*,"Status":[{"StatusId":*ASSIGNED*,"Top":10,"Left":160,"Transitions":[]},{"StatusId":*CLOSED*,"Top":10,"Left":310,"Transitions":[]},{"StatusId":*INPROGRESS*,"Top":0,"Left":0,"Transitions":[{"StatusId":*CLOSED*,"Groups":[1]}]},{"StatusId":*REOPENED*,"Top":80,"Left":10,"Transitions":[]},{"StatusId":*UNASSIGNED*,"Top":80,"Left":160,"Transitions":[{"StatusId":*INPROGRESS*,"Groups":[1]},{"StatusId":*ASSIGNED*,"Groups":[1]}]}]}','','13')
INSERT gemini_issuetypes(seq,typedesc,imagepath,color,screen,workflow,tag,templateid) VALUES('5','Task','assets/images/meta/ISSUES/type-task.png','#3c12e6','{"ReferenceId":*NEWFEATURE*,"Create":[{"ProjectGroups":[1],"Required":false,"Sequence":0,"Label":"","Id":"Type","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"Title","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":2,"Label":"","Id":"Description","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":3,"Label":"","Id":"StartDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":4,"Label":"","Id":"Status","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":5,"Label":"","Id":"DueDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":6,"Label":"","Id":"CalculatedTimeLogged","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":7,"Label":"","Id":"AssociatedAttachments","Type":0}],"Edit":[{"ProjectGroups":[1],"Required":false,"Sequence":0,"Label":"","Id":"Title","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"Description","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":2,"Label":"","Id":"StartDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":3,"Label":"","Id":"Status","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":4,"Label":"","Id":"DueDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":5,"Label":"","Id":"CalculatedTimeLogged","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":6,"Label":"","Id":"AssociatedAttachments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":7,"Label":"","Id":"AssociatedComments","Type":0}],"View":[{"ProjectGroups":[1],"Required":false,"Sequence":0,"Label":"","Id":"Title","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"StartDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":2,"Label":"","Id":"Status","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":3,"Label":"","Id":"DueDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":4,"Label":"","Id":"CalculatedTimeLogged","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":5,"Label":"","Id":"Description","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":6,"Label":"","Id":"AssociatedAttachments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":7,"Label":"","Id":"AssociatedComments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":8,"Label":"","Id":"AssociatedTime","Type":0}],"IsNew":true,"IsExisting":false,"Active":true,"Archived":false,"Deleted":false,"Created":"2012-08-01T19:35:46.3840353Z","Revised":"2012-08-01T19:35:46.3840353Z","Errors":[],"HasErrors":false,"Id":0}','{"ReferenceId":*NEWFEATURE*,"Status":[{"StatusId":*ASSIGNED*,"Top":0,"Left":0,"Transitions":[{"StatusId":*CLOSED*,"Groups":[1]},{"StatusId":*UNASSIGNED*,"Groups":[1]},{"StatusId":*INPROGRESS*,"Groups":[1]}]},{"StatusId":*CLOSED*,"Top":10,"Left":310,"Transitions":[]},{"StatusId":*INPROGRESS*,"Top":0,"Left":0,"Transitions":[{"StatusId":*CLOSED*,"Groups":[1]}]},{"StatusId":*REOPENED*,"Top":9,"Left":0,"Transitions":[]},{"StatusId":*UNASSIGNED*,"Top":80,"Left":160,"Transitions":[{"StatusId":*ASSIGNED*,"Groups":[1]}]}]}','','13')
INSERT gemini_issuetypes(seq,typedesc,imagepath,color,screen,workflow,tag,templateid) VALUES('2','Task','assets/images/meta/MARKETING/type-task.png','#522c52','{"ReferenceId":0,"Create":[{"ProjectGroups":[1],"Required":false,"Sequence":0,"Label":"","Id":"Type","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"Title","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":2,"Label":"","Id":"Description","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":3,"Label":"","Id":"FixedInVersion","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":4,"Label":"","Id":"Component","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":5,"Label":"","Id":"Status","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":6,"Label":"","Id":"34","Type":1},{"ProjectGroups":[1],"Required":false,"Sequence":7,"Label":"","Id":"35","Type":1},{"ProjectGroups":[1],"Required":false,"Sequence":8,"Label":"","Id":"33","Type":1},{"ProjectGroups":[1],"Required":false,"Sequence":9,"Label":"","Id":"AssociatedComments","Type":0}],"Edit":[{"ProjectGroups":[1],"Required":false,"Sequence":0,"Label":"","Id":"Type","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"Title","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":2,"Label":"","Id":"Description","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":3,"Label":"","Id":"FixedInVersion","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":4,"Label":"","Id":"Component","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":5,"Label":"","Id":"Status","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":6,"Label":"","Id":"Resolution","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":7,"Label":"","Id":"34","Type":1},{"ProjectGroups":[1],"Required":false,"Sequence":8,"Label":"","Id":"35","Type":1},{"ProjectGroups":[1],"Required":false,"Sequence":9,"Label":"","Id":"33","Type":1},{"ProjectGroups":[1],"Required":false,"Sequence":10,"Label":"","Id":"StartDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":11,"Label":"","Id":"DueDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":12,"Label":"","Id":"EstimatedEffort","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":13,"Label":"","Id":"AssignedTo","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":14,"Label":"","Id":"PercentComplete","Type":0}],"View":[{"ProjectGroups":[1],"Required":false,"Sequence":0,"Label":"","Id":"IssueKey","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"PercentComplete","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":2,"Label":"","Id":"Title","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":3,"Label":"","Id":"Type","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":4,"Label":"","Id":"Component","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":5,"Label":"","Id":"FixedInVersion","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":6,"Label":"","Id":"Status","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":7,"Label":"","Id":"Resolution","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":8,"Label":"","Id":"CalculatedTimeLogged","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":9,"Label":"","Id":"34","Type":1},{"ProjectGroups":[1],"Required":false,"Sequence":10,"Label":"","Id":"35","Type":1},{"ProjectGroups":[1],"Required":false,"Sequence":11,"Label":"","Id":"33","Type":1},{"ProjectGroups":[1],"Required":false,"Sequence":12,"Label":"","Id":"StartDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":13,"Label":"","Id":"DueDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":14,"Label":"","Id":"EstimatedEffort","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":15,"Label":"","Id":"AssignedTo","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":16,"Label":"","Id":"Description","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":17,"Label":"","Id":"AssociatedAttachments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":18,"Label":"","Id":"AssociatedComments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":19,"Label":"","Id":"AssociatedHierarchy","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":20,"Label":"","Id":"C8F0693D-129C-4D19-9865-9CED964F06AB","Type":2}],"IsNew":true,"IsExisting":false,"Active":true,"Archived":false,"Deleted":false,"Created":"2012-08-02T10:20:28.9279606Z","Revised":"2012-08-02T10:20:28.9279606Z","Errors":[],"HasErrors":false,"Id":0}','{"ReferenceId":0,"Status":[{"StatusId":*PARKED*,"Top":21,"Left":211,"Transitions":[]},{"StatusId":*PLANNED*,"Top":221,"Left":497,"Transitions":[{"StatusId":*COMPLETED*,"Groups":[1]}]},{"StatusId":*INPROGRESS*,"Top":124,"Left":279,"Transitions":[{"StatusId":*PLANNED*,"Groups":[1]}]},{"StatusId":*COMPLETED*,"Top":311,"Left":261,"Transitions":[{"StatusId":*INPROGRESS*,"Groups":[1]}]}]}','','14')
INSERT gemini_issuetypes(seq,typedesc,imagepath,color,screen,workflow,tag,templateid) VALUES('3','Task','assets/images/meta/REQUIREMENTS/type-task.png','#e68012','{"ReferenceId":*REQUIREMENT*,"Create":[{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"Type","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":2,"Label":"","Id":"Title","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":3,"Label":"","Id":"Description","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":4,"Label":"","Id":"Status","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":5,"Label":"","Id":"StartDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":6,"Label":"","Id":"DueDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":7,"Label":"","Id":"CalculatedTimeLogged","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":8,"Label":"","Id":"AssociatedAttachments","Type":0}],"Edit":[{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"Title","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":2,"Label":"","Id":"Description","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":3,"Label":"","Id":"Status","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":4,"Label":"","Id":"StartDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":5,"Label":"","Id":"DueDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":6,"Label":"","Id":"CalculatedTimeLogged","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":7,"Label":"","Id":"AssociatedAttachments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":8,"Label":"","Id":"AssociatedComments","Type":0}],"View":[{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"Title","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":2,"Label":"","Id":"Description","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":3,"Label":"","Id":"Status","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":4,"Label":"","Id":"StartDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":5,"Label":"","Id":"DueDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":6,"Label":"","Id":"CalculatedTimeLogged","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":7,"Label":"","Id":"AssociatedAttachments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":8,"Label":"","Id":"AssociatedComments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":9,"Label":"","Id":"AssociatedTime","Type":0}],"IsNew":true,"IsExisting":false,"Active":true,"Archived":false,"Deleted":false,"Created":"2012-08-01T11:03:58.7526593Z","Revised":"2012-08-01T11:03:58.7526593Z","Errors":[],"HasErrors":false,"Id":0}','{"ReferenceId":*REQUIREMENT*,"Status":[{"StatusId":*APPROVED*,"Top":189,"Left":202,"Transitions":[{"StatusId":*INPROGRESS*,"Groups":[1]}]},{"StatusId":*CLOSED*,"Top":211,"Left":553,"Transitions":[{"StatusId":*REOPENED*,"Groups":[1]}]},{"StatusId":*INPROGRESS*,"Top":385,"Left":152,"Transitions":[{"StatusId":*CLOSED*,"Groups":[1]}]},{"StatusId":*REOPENED*,"Top":92,"Left":628,"Transitions":[{"StatusId":*REQUIREMENT*,"Groups":[1]}]},{"StatusId":*REQUIREMENT*,"Top":84,"Left":328,"Transitions":[{"StatusId":*APPROVED*,"Groups":[1]}]},{"StatusId":*WALKTHROUGH*,"Top":438,"Left":560,"Transitions":[]}]}','','15')
INSERT gemini_issuetypes(seq,typedesc,imagepath,color,screen,workflow,tag,templateid) VALUES('1','Requirement','assets/images/meta/REQUIREMENTS/type-enhancement.png','#e612e6','{"ReferenceId":0,"Create":[{"ProjectGroups":[1],"Required":false,"Sequence":0,"Label":"","Id":"Type","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"Title","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":2,"Label":"","Id":"Description","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":3,"Label":"","Id":"Priority","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":4,"Label":"","Id":"Component","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":5,"Label":"","Id":"FixedInVersion","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":6,"Label":"","Id":"EstimatedEffort","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":7,"Label":"","Id":"StartDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":8,"Label":"","Id":"DueDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":9,"Label":"","Id":"AssociatedComments","Type":0}],"Edit":[{"ProjectGroups":[1],"Required":false,"Sequence":0,"Label":"","Id":"Type","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"Title","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":2,"Label":"","Id":"Description","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":3,"Label":"","Id":"Priority","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":4,"Label":"","Id":"Component","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":5,"Label":"","Id":"FixedInVersion","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":6,"Label":"","Id":"EstimatedEffort","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":7,"Label":"","Id":"StartDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":8,"Label":"","Id":"DueDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":9,"Label":"","Id":"AssignedTo","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":10,"Label":"","Id":"Status","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":11,"Label":"","Id":"Resolution","Type":0}],"View":[{"ProjectGroups":[1],"Required":false,"Sequence":0,"Label":"","Id":"IssueKey","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"Type","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":2,"Label":"","Id":"Title","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":3,"Label":"","Id":"Priority","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":4,"Label":"","Id":"Component","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":5,"Label":"","Id":"FixedInVersion","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":6,"Label":"","Id":"StartDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":7,"Label":"","Id":"DueDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":8,"Label":"","Id":"AssignedTo","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":9,"Label":"","Id":"Status","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":10,"Label":"","Id":"Resolution","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":11,"Label":"","Id":"DateCreated","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":12,"Label":"","Id":"ClosedDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":13,"Label":"","Id":"ResolvedDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":14,"Label":"","Id":"DateRevised","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":15,"Label":"","Id":"EstimatedEffort","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":16,"Label":"","Id":"CalculatedTimeLogged","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":17,"Label":"","Id":"CalculatedTimeExceeded","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":18,"Label":"","Id":"CalculatedTimeRemaining","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":19,"Label":"","Id":"Visibility","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":20,"Label":"","Id":"Description","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":21,"Label":"","Id":"AssociatedAttachments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":22,"Label":"","Id":"AssociatedComments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":23,"Label":"","Id":"AssociatedHierarchy","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":24,"Label":"","Id":"C8F0693D-129C-4D19-9865-9CED964F06AB","Type":2},{"ProjectGroups":[1],"Required":false,"Sequence":25,"Label":"","Id":"AssociatedWatchers","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":26,"Label":"","Id":"AssociatedHistory","Type":0}],"IsNew":true,"IsExisting":false,"Active":true,"Archived":false,"Deleted":false,"Created":"2012-08-02T10:20:36.7084056Z","Revised":"2012-08-02T10:20:36.7084056Z","Errors":[],"HasErrors":false,"Id":0}','{"ReferenceId":0,"Status":[{"StatusId":*REQUIREMENT*,"Top":25,"Left":98,"Transitions":[{"StatusId":*CLOSED*,"Groups":[1]},{"StatusId":*WALKTHROUGH*,"Groups":[1]}]},{"StatusId":*WALKTHROUGH*,"Top":40,"Left":575,"Transitions":[{"StatusId":*APPROVED*,"Groups":[1]}]},{"StatusId":*APPROVED*,"Top":329,"Left":526,"Transitions":[{"StatusId":*INPROGRESS*,"Groups":[1]}]},{"StatusId":*INPROGRESS*,"Top":558,"Left":503,"Transitions":[{"StatusId":*CLOSED*,"Groups":[1]}]},{"StatusId":*CLOSED*,"Top":378,"Left":70,"Transitions":[{"StatusId":*REOPENED*,"Groups":[1]}]},{"StatusId":*REOPENED*,"Top":222,"Left":288,"Transitions":[{"StatusId":*WALKTHROUGH*,"Groups":[1]}]}]}','','15')
INSERT gemini_issuetypes(seq,typedesc,imagepath,color,screen,workflow,tag,templateid) VALUES('2','Test Case','assets/images/meta/SENTRY/type-task.png','#20700c','{"ReferenceId":0,"Create":[{"ProjectGroups":[1],"Required":false,"Sequence":0,"Label":"","Id":"Type","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"Title","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":2,"Label":"","Id":"41","Type":1},{"ProjectGroups":[1],"Required":false,"Sequence":3,"Label":"","Id":"Description","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":4,"Label":"","Id":"42","Type":1},{"ProjectGroups":[1],"Required":false,"Sequence":5,"Label":"","Id":"43","Type":1},{"ProjectGroups":[1],"Required":false,"Sequence":6,"Label":"","Id":"Status","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":7,"Label":"","Id":"Component","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":8,"Label":"","Id":"AssignedTo","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":9,"Label":"","Id":"StartDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":10,"Label":"","Id":"DueDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":11,"Label":"","Id":"PercentComplete","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":12,"Label":"","Id":"AffectedVersions","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":13,"Label":"","Id":"AssociatedAttachments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":14,"Label":"","Id":"AssociatedComments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":15,"Label":"","Id":"EstimatedEffort","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":16,"Label":"","Id":"Priority","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":17,"Label":"","Id":"Resolution","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":18,"Label":"","Id":"Severity","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":19,"Label":"","Id":"Source","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":20,"Label":"","Id":"AssociatedTime","Type":0}],"Edit":[{"ProjectGroups":[1],"Required":false,"Sequence":0,"Label":"","Id":"Type","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"Title","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":2,"Label":"","Id":"41","Type":1},{"ProjectGroups":[1],"Required":false,"Sequence":3,"Label":"","Id":"Description","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":4,"Label":"","Id":"42","Type":1},{"ProjectGroups":[1],"Required":false,"Sequence":5,"Label":"","Id":"43","Type":1},{"ProjectGroups":[1],"Required":false,"Sequence":6,"Label":"","Id":"Status","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":7,"Label":"","Id":"Component","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":8,"Label":"","Id":"AssignedTo","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":9,"Label":"","Id":"StartDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":10,"Label":"","Id":"DueDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":11,"Label":"","Id":"PercentComplete","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":12,"Label":"","Id":"AffectedVersions","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":13,"Label":"","Id":"AssociatedAttachments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":14,"Label":"","Id":"AssociatedComments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":15,"Label":"","Id":"EstimatedEffort","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":16,"Label":"","Id":"Priority","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":17,"Label":"","Id":"Resolution","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":18,"Label":"","Id":"Severity","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":19,"Label":"","Id":"Source","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":20,"Label":"","Id":"AssociatedTime","Type":0}],"View":[{"ProjectGroups":[1],"Required":false,"Sequence":0,"Label":"","Id":"Type","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"Title","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":2,"Label":"","Id":"41","Type":1},{"ProjectGroups":[1],"Required":false,"Sequence":3,"Label":"","Id":"Status","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":4,"Label":"","Id":"Component","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":5,"Label":"","Id":"AssignedTo","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":6,"Label":"","Id":"StartDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":7,"Label":"","Id":"DueDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":8,"Label":"","Id":"PercentComplete","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":9,"Label":"","Id":"AffectedVersions","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":10,"Label":"","Id":"EstimatedEffort","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":11,"Label":"","Id":"Priority","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":12,"Label":"","Id":"Resolution","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":13,"Label":"","Id":"Severity","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":14,"Label":"","Id":"Source","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":15,"Label":"","Id":"ResolvedDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":16,"Label":"","Id":"ClosedDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":17,"Label":"","Id":"DateCreated","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":18,"Label":"","Id":"DateRevised","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":19,"Label":"","Id":"IssueKey","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":20,"Label":"","Id":"CalculatedTimeLogged","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":21,"Label":"","Id":"Description","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":22,"Label":"","Id":"42","Type":1},{"ProjectGroups":[1],"Required":false,"Sequence":23,"Label":"","Id":"43","Type":1},{"ProjectGroups":[1],"Required":false,"Sequence":24,"Label":"","Id":"AssociatedComments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":25,"Label":"","Id":"AssociatedLinks","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":26,"Label":"","Id":"AssociatedTime","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":27,"Label":"","Id":"AssociatedHistory","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":28,"Label":"","Id":"617F1435-C0AA-48E6-B6A9-2C1F756E02A6","Type":2},{"ProjectGroups":[1],"Required":false,"Sequence":29,"Label":"","Id":"15627A0F-5AD2-4D35-BF24-60D678F59354","Type":2},{"ProjectGroups":[1],"Required":false,"Sequence":30,"Label":"","Id":"ACE2F8B3-C724-444F-9B72-B8B2137DDA16","Type":2}],"IsNew":true,"IsExisting":false,"Active":true,"Archived":false,"Deleted":false,"Created":"2012-08-13T11:22:02.4410404Z","Revised":"2012-08-13T11:22:02.4410404Z","Errors":[],"HasErrors":false,"Id":0}','{"ReferenceId":*TESTPLAN*,"Status":[]}','CASE','16')

UPDATE gemini_issuetypes SET screen = REPLACE(screen, '*STORY*', (SELECT b.typeid FROM gemini_issuetypes b WHERE b.templateid = 10 AND typedesc COLLATE DATABASE_DEFAULT ='Story' COLLATE DATABASE_DEFAULT)) WHERE templateid=10 AND screen LIKE '%*STORY*%'
UPDATE gemini_issuetypes SET workflow = REPLACE(workflow, '*STORY*', (SELECT b.typeid FROM gemini_issuetypes b WHERE b.templateid = 10 AND typedesc COLLATE DATABASE_DEFAULT ='Story' COLLATE DATABASE_DEFAULT )) WHERE templateid=10 AND workflow LIKE '%*STORY*%'
UPDATE gemini_issuetypes SET screen = REPLACE(screen, '*NEWFEATURE*', (SELECT b.typeid FROM gemini_issuetypes b WHERE b.templateid = 13 AND typedesc COLLATE DATABASE_DEFAULT  = 'New Feature' COLLATE DATABASE_DEFAULT )) WHERE templateid=13 AND screen LIKE '%*NEWFEATURE*%'
UPDATE gemini_issuetypes SET workflow = REPLACE(workflow, '*NEWFEATURE*', (SELECT b.typeid FROM gemini_issuetypes b WHERE b.templateid = 13 AND typedesc COLLATE DATABASE_DEFAULT ='New Feature' COLLATE DATABASE_DEFAULT )) WHERE templateid=13 AND workflow LIKE '%*NEWFEATURE*%'
UPDATE gemini_issuetypes SET screen = REPLACE(screen, '*OPENISSUE*', (SELECT b.typeid FROM gemini_issuetypes b WHERE b.templateid = 11 AND typedesc COLLATE DATABASE_DEFAULT ='Open Issue' COLLATE DATABASE_DEFAULT )) WHERE templateid=11 AND screen LIKE '%*OPENISSUE*%'
UPDATE gemini_issuetypes SET workflow = REPLACE(workflow, '*OPENISSUE*', (SELECT b.typeid FROM gemini_issuetypes b WHERE b.templateid = 11 AND typedesc COLLATE DATABASE_DEFAULT ='Open Issue' COLLATE DATABASE_DEFAULT )) WHERE templateid=11 AND workflow LIKE '%*OPENISSUE*%'
UPDATE gemini_issuetypes SET screen = REPLACE(screen, '*INVESTIGATION*', (SELECT b.typeid FROM gemini_issuetypes b WHERE b.templateid = 12 AND typedesc COLLATE DATABASE_DEFAULT = 'Investigation' COLLATE DATABASE_DEFAULT )) WHERE templateid=12 AND screen LIKE '%*INVESTIGATION*%'
UPDATE gemini_issuetypes SET screen = REPLACE(screen, '*REQUIREMENT*', (SELECT b.typeid FROM gemini_issuetypes b WHERE b.templateid = 15 AND typedesc COLLATE DATABASE_DEFAULT ='Requirement' COLLATE DATABASE_DEFAULT )) WHERE templateid=15 AND screen LIKE '%*REQUIREMENT*%'
UPDATE gemini_issuetypes SET workflow = REPLACE(workflow, '*REQUIREMENT*', (SELECT b.typeid FROM gemini_issuetypes b WHERE b.templateid = 15 AND typedesc COLLATE DATABASE_DEFAULT ='Requirement' COLLATE DATABASE_DEFAULT )) WHERE templateid=15 AND workflow LIKE '%*REQUIREMENT*%'

UPDATE gemini_issuetypes SET workflow = REPLACE(workflow, '*SUPPORTREQUEST*', (SELECT b.typeid FROM gemini_issuetypes b WHERE b.templateid = 12 AND typedesc COLLATE DATABASE_DEFAULT ='Support Request' COLLATE DATABASE_DEFAULT )) WHERE templateid=12 AND workflow LIKE '%*SUPPORTREQUEST*%'
UPDATE gemini_issuetypes SET workflow = REPLACE(workflow, '*TESTPLAN*', (SELECT b.typeid FROM gemini_issuetypes b WHERE b.templateid = 16 AND typedesc COLLATE DATABASE_DEFAULT ='Test Plan' COLLATE DATABASE_DEFAULT )) WHERE templateid=16 AND workflow LIKE '%*TESTPLAN*%'


INSERT gemini_issuepriorities(seq,prioritydesc,imagepath,color,templateid) VALUES('1','Low','assets/images/meta/AGILE/priority-low.png','#0000ff','10')
INSERT gemini_issuepriorities(seq,prioritydesc,imagepath,color,templateid) VALUES('2','Medium','assets/images/meta/AGILE/priority-medium.png','#ffffff','10')
INSERT gemini_issuepriorities(seq,prioritydesc,imagepath,color,templateid) VALUES('3','High','assets/images/meta/AGILE/priority-high.png','#eb0c1f','10')
INSERT gemini_issuepriorities(seq,prioritydesc,imagepath,color,templateid) VALUES('1','Low','assets/images/meta/ENQUIRY/priority-low.png','#0000ff','11')
INSERT gemini_issuepriorities(seq,prioritydesc,imagepath,color,templateid) VALUES('2','Medium','assets/images/meta/ENQUIRY/priority-medium.png','#ffffff','11')
INSERT gemini_issuepriorities(seq,prioritydesc,imagepath,color,templateid) VALUES('3','High','assets/images/meta/ENQUIRY/priority-high.png','#eb0c1f','11')
INSERT gemini_issuepriorities(seq,prioritydesc,imagepath,color,templateid) VALUES('1','Low','assets/images/meta/HELPDESK/priority-low.png','#0000ff','12')
INSERT gemini_issuepriorities(seq,prioritydesc,imagepath,color,templateid) VALUES('2','Medium','assets/images/meta/HELPDESK/priority-medium.png','#ffffff','12')
INSERT gemini_issuepriorities(seq,prioritydesc,imagepath,color,templateid) VALUES('3','High','assets/images/meta/HELPDESK/priority-high.png','#eb0c1f','12')
INSERT gemini_issuepriorities(seq,prioritydesc,imagepath,color,templateid) VALUES('1','Low','assets/images/meta/ISSUES/priority-low.png','#0000ff','13')
INSERT gemini_issuepriorities(seq,prioritydesc,imagepath,color,templateid) VALUES('2','Medium','assets/images/meta/ISSUES/priority-medium.png','#ffffff','13')
INSERT gemini_issuepriorities(seq,prioritydesc,imagepath,color,templateid) VALUES('3','High','assets/images/meta/ISSUES/priority-high.png','#eb0c1f','13')
INSERT gemini_issuepriorities(seq,prioritydesc,imagepath,color,templateid) VALUES('1','High','assets/images/meta/MARKETING/priority-high.png','#a30d21','14')
INSERT gemini_issuepriorities(seq,prioritydesc,imagepath,color,templateid) VALUES('2','Low','assets/images/meta/MARKETING/priority-low.png','#0000ff','14')
INSERT gemini_issuepriorities(seq,prioritydesc,imagepath,color,templateid) VALUES('3','Medium','assets/images/meta/MARKETING/priority-medium.png','#161670','14')
INSERT gemini_issuepriorities(seq,prioritydesc,imagepath,color,templateid) VALUES('1','Low','assets/images/meta/REQUIREMENTS/priority-low.png','#0000ff','15')
INSERT gemini_issuepriorities(seq,prioritydesc,imagepath,color,templateid) VALUES('2','Medium','assets/images/meta/REQUIREMENTS/priority-medium.png','#ffffff','15')
INSERT gemini_issuepriorities(seq,prioritydesc,imagepath,color,templateid) VALUES('3','High','assets/images/meta/REQUIREMENTS/priority-high.png','#eb0c1f','15')
INSERT gemini_issuepriorities(seq,prioritydesc,imagepath,color,templateid) VALUES('1','High','assets/images/meta/SENTRY/priority-high.png','#e81717','16')
INSERT gemini_issuepriorities(seq,prioritydesc,imagepath,color,templateid) VALUES('2','Low','assets/images/meta/SENTRY/priority-low.png','#b8b8e6','16')
INSERT gemini_issuepriorities(seq,prioritydesc,imagepath,color,templateid) VALUES('3','Medium','assets/images/meta/SENTRY/priority-medium.png','#c7732a','16')

INSERT gemini_issueseverity(seq,severitydesc,imagepath,color,templateid) VALUES('1','Trivial','assets/images/meta/AGILE/severity-trivial.png','','10')
INSERT gemini_issueseverity(seq,severitydesc,imagepath,color,templateid) VALUES('2','Minor','assets/images/meta/AGILE/severity-minor.png','#0000ff','10')
INSERT gemini_issueseverity(seq,severitydesc,imagepath,color,templateid) VALUES('3','Major','assets/images/meta/AGILE/severity-major.png','#8c054d','10')
INSERT gemini_issueseverity(seq,severitydesc,imagepath,color,templateid) VALUES('1','Trivial','assets/images/meta/ENQUIRY/severity-trivial.png','#1a6145','11')
INSERT gemini_issueseverity(seq,severitydesc,imagepath,color,templateid) VALUES('2','Minor','assets/images/meta/ENQUIRY/severity-minor.png','#0000ff','11')
INSERT gemini_issueseverity(seq,severitydesc,imagepath,color,templateid) VALUES('3','Major','assets/images/meta/ENQUIRY/severity-major.png','#8c054d','11')
INSERT gemini_issueseverity(seq,severitydesc,imagepath,color,templateid) VALUES('4','Critical','assets/images/meta/ENQUIRY/severity-showstopper.png','#fa0021','11')
INSERT gemini_issueseverity(seq,severitydesc,imagepath,color,templateid) VALUES('1','Trivial','assets/images/meta/HELPDESK/severity-trivial.png','','12')
INSERT gemini_issueseverity(seq,severitydesc,imagepath,color,templateid) VALUES('2','Minor','assets/images/meta/HELPDESK/severity-minor.png','#0000ff','12')
INSERT gemini_issueseverity(seq,severitydesc,imagepath,color,templateid) VALUES('3','Major','assets/images/meta/HELPDESK/severity-major.png','#8c054d','12')
INSERT gemini_issueseverity(seq,severitydesc,imagepath,color,templateid) VALUES('1','Trivial','assets/images/meta/ISSUES/severity-trivial.png','','13')
INSERT gemini_issueseverity(seq,severitydesc,imagepath,color,templateid) VALUES('2','Minor','assets/images/meta/ISSUES/severity-minor.png','#0000ff','13')
INSERT gemini_issueseverity(seq,severitydesc,imagepath,color,templateid) VALUES('3','Major','assets/images/meta/ISSUES/severity-major.png','#8c054d','13')
INSERT gemini_issueseverity(seq,severitydesc,imagepath,color,templateid) VALUES('1','Minor','assets/images/meta/MARKETING/severity-minor.png','#0000ff','14')
INSERT gemini_issueseverity(seq,severitydesc,imagepath,color,templateid) VALUES('2','Major','assets/images/meta/MARKETING/severity-major.png','#a80a31','14')
INSERT gemini_issueseverity(seq,severitydesc,imagepath,color,templateid) VALUES('1','Trivial','assets/images/meta/REQUIREMENTS/severity-trivial.png','','15')
INSERT gemini_issueseverity(seq,severitydesc,imagepath,color,templateid) VALUES('2','Minor','assets/images/meta/REQUIREMENTS/severity-minor.png','#0000ff','15')
INSERT gemini_issueseverity(seq,severitydesc,imagepath,color,templateid) VALUES('3','Major','assets/images/meta/REQUIREMENTS/severity-major.png','#8c054d','15')
INSERT gemini_issueseverity(seq,severitydesc,imagepath,color,templateid) VALUES('1','Minor','assets/images/meta/SENTRY/severity-minor.png','#9595d6','16')
INSERT gemini_issueseverity(seq,severitydesc,imagepath,color,templateid) VALUES('2','Major','assets/images/meta/SENTRY/severity-major.png','#bd20bd','16')
INSERT gemini_issueseverity(seq,severitydesc,imagepath,color,templateid) VALUES('3','Critical','assets/images/meta/SENTRY/severity-showstopper.png','#e80523','16')

INSERT gemini_issuestatus(isfinal,statusdesc,statustype,imagepath,comment,color,seq,templateid) VALUES('0','In Backlog','0','assets/images/meta/AGILE/test-blackbox.png','','#3d213d','1','10')
INSERT gemini_issuestatus(isfinal,statusdesc,statustype,imagepath,comment,color,seq,templateid) VALUES('0','In Sprint','0','assets/images/meta/status-ready-for-approval.png','','#b4bf17','2','10')
INSERT gemini_issuestatus(isfinal,statusdesc,statustype,imagepath,comment,color,seq,templateid) VALUES('0','Tested','0','assets/images/meta/type-qualitycheck.png','','#d16817','4','10')
INSERT gemini_issuestatus(isfinal,statusdesc,statustype,imagepath,comment,color,seq,templateid) VALUES('0','In Progress','0','assets/images/meta/AGILE/status-inprogress.png','','#166b08','3','10')
INSERT gemini_issuestatus(isfinal,statusdesc,statustype,imagepath,comment,color,seq,templateid) VALUES('1','Closed','1','assets/images/meta/AGILE/status-closed.png','','#1326f5','5','10')
INSERT gemini_issuestatus(isfinal,statusdesc,statustype,imagepath,comment,color,seq,templateid) VALUES('0','Escalation-Level1','0','assets/images/meta/ENQUIRY/test-requirements.png','','#bf6a1f','1','11')
INSERT gemini_issuestatus(isfinal,statusdesc,statustype,imagepath,comment,color,seq,templateid) VALUES('0','Escalation-Level2','0','assets/images/meta/ENQUIRY/test-stress.png','','#f50a0a','2','11')
INSERT gemini_issuestatus(isfinal,statusdesc,statustype,imagepath,comment,color,seq,templateid) VALUES('0','Raised','0','assets/images/meta/ENQUIRY/type-newfeature.png','','#a0c218','3','11')
INSERT gemini_issuestatus(isfinal,statusdesc,statustype,imagepath,comment,color,seq,templateid) VALUES('0','User Rejected','0','assets/images/meta/ENQUIRY/status-rejected.png','','#754b0d','6','11')
INSERT gemini_issuestatus(isfinal,statusdesc,statustype,imagepath,comment,color,seq,templateid) VALUES('0','Moved to Delivery','0','assets/images/meta/ENQUIRY/test-blackbox.png','','#7a12a3','9','11')
INSERT gemini_issuestatus(isfinal,statusdesc,statustype,imagepath,comment,color,seq,templateid) VALUES('0','Assigned','0','assets/images/meta/ENQUIRY/status-assigned.png','','#47d119','7','11')
INSERT gemini_issuestatus(isfinal,statusdesc,statustype,imagepath,comment,color,seq,templateid) VALUES('1','Closed','1','assets/images/meta/ENQUIRY/status-closed.png','','#0f0907','10','11')
INSERT gemini_issuestatus(isfinal,statusdesc,statustype,imagepath,comment,color,seq,templateid) VALUES('0','Case Reopened','0','assets/images/meta/ENQUIRY/status-reopened.png','','#0a1c91','11','11')
INSERT gemini_issuestatus(isfinal,statusdesc,statustype,imagepath,comment,color,seq,templateid) VALUES('0','User Accepted','0','assets/images/meta/ENQUIRY/test-acceptance.png','','#541454','8','11')
INSERT gemini_issuestatus(isfinal,statusdesc,statustype,imagepath,comment,color,seq,templateid) VALUES('0','Pending Approval','0','assets/images/meta/ENQUIRY/status-ready-for-approval.png','','#787167','4','11')
INSERT gemini_issuestatus(isfinal,statusdesc,statustype,imagepath,comment,color,seq,templateid) VALUES('0','Approved','0','assets/images/meta/ENQUIRY/status-approved.png','','#099e38','5','11')
INSERT gemini_issuestatus(isfinal,statusdesc,statustype,imagepath,comment,color,seq,templateid) VALUES('0','Unassigned','0','assets/images/meta/HELPDESK/status-unassigned.png','','#0000ff','1','12')
INSERT gemini_issuestatus(isfinal,statusdesc,statustype,imagepath,comment,color,seq,templateid) VALUES('0','In Progress','0','assets/images/meta/HELPDESK/status-inprogress.png','','#a130a1','5','12')
INSERT gemini_issuestatus(isfinal,statusdesc,statustype,imagepath,comment,color,seq,templateid) VALUES('1','Closed','1','assets/images/meta/HELPDESK/status-closed.png','','#eb8023','6','12')
INSERT gemini_issuestatus(isfinal,statusdesc,statustype,imagepath,comment,color,seq,templateid) VALUES('0','Approved','0','assets/images/meta/HELPDESK/status-approved.png','','#099e38','2','12')
INSERT gemini_issuestatus(isfinal,statusdesc,statustype,imagepath,comment,color,seq,templateid) VALUES('0','Unassigned','0','assets/images/meta/ISSUES/status-unassigned.png','','#0000ff','1','13')
INSERT gemini_issuestatus(isfinal,statusdesc,statustype,imagepath,comment,color,seq,templateid) VALUES('0','Assigned','0','assets/images/meta/ISSUES/status-assigned.png','','#d9782e','2','13')
INSERT gemini_issuestatus(isfinal,statusdesc,statustype,imagepath,comment,color,seq,templateid) VALUES('0','In Progress','0','assets/images/meta/ISSUES/status-inprogress.png','','#a130a1','3','13')
INSERT gemini_issuestatus(isfinal,statusdesc,statustype,imagepath,comment,color,seq,templateid) VALUES('1','Closed','1','assets/images/meta/ISSUES/status-closed.png','','#28b845','4','13')
INSERT gemini_issuestatus(isfinal,statusdesc,statustype,imagepath,comment,color,seq,templateid) VALUES('0','Reopened','0','assets/images/meta/ISSUES/status-reopened.png','','#f52525','5','13')
INSERT gemini_issuestatus(isfinal,statusdesc,statustype,imagepath,comment,color,seq,templateid) VALUES('1','Planned','1','assets/images/meta/MARKETING/status-assigned.png','','#16b821','2','14')
INSERT gemini_issuestatus(isfinal,statusdesc,statustype,imagepath,comment,color,seq,templateid) VALUES('0','In Progress','0','assets/images/meta/MARKETING/status-inprogress.png','','#941894','3','14')
INSERT gemini_issuestatus(isfinal,statusdesc,statustype,imagepath,comment,color,seq,templateid) VALUES('0','Completed','0','assets/images/meta/MARKETING/status-closed.png','','#c7bf2a','4','14')
INSERT gemini_issuestatus(isfinal,statusdesc,statustype,imagepath,comment,color,seq,templateid) VALUES('0','Parked','0','assets/images/meta/MARKETING/status-postponed.png','','#5e105e','1','14')
INSERT gemini_issuestatus(isfinal,statusdesc,statustype,imagepath,comment,color,seq,templateid) VALUES('0','Approved','0','assets/images/meta/REQUIREMENTS/status-approved.png','','#36a820','3','15')
INSERT gemini_issuestatus(isfinal,statusdesc,statustype,imagepath,comment,color,seq,templateid) VALUES('0','Requirement','0','assets/images/meta/REQUIREMENTS/status-unassigned.png','','#0000ff','1','15')
INSERT gemini_issuestatus(isfinal,statusdesc,statustype,imagepath,comment,color,seq,templateid) VALUES('0','Walkthrough','0','assets/images/meta/REQUIREMENTS/status-assigned.png','','#47d119','2','15')
INSERT gemini_issuestatus(isfinal,statusdesc,statustype,imagepath,comment,color,seq,templateid) VALUES('1','Closed','1','assets/images/meta/REQUIREMENTS/status-closed.png','','#eb1deb','5','15')
INSERT gemini_issuestatus(isfinal,statusdesc,statustype,imagepath,comment,color,seq,templateid) VALUES('0','Reopened','0','assets/images/meta/REQUIREMENTS/status-reopened.png','','#0a1c91','6','15')
INSERT gemini_issuestatus(isfinal,statusdesc,statustype,imagepath,comment,color,seq,templateid) VALUES('0','In Progress','0','assets/images/meta/REQUIREMENTS/status-inprogress.png','','#8c3d8c','4','15')
INSERT gemini_issuestatus(isfinal,statusdesc,statustype,imagepath,comment,color,seq,templateid) VALUES('0','Rejected','0','assets/images/meta/status-rejected.png','','#b3164f','3','12')
INSERT gemini_issuestatus(isfinal,statusdesc,statustype,imagepath,comment,color,seq,templateid) VALUES('0','Reported','1','assets/images/meta/SENTRY/test-smoke.png','','#177d4b','1','16')
INSERT gemini_issuestatus(isfinal,statusdesc,statustype,imagepath,comment,color,seq,templateid) VALUES('0','Planned','0','assets/images/meta/SENTRY/test-installation.png','','#2a0aa8','2','16')
INSERT gemini_issuestatus(isfinal,statusdesc,statustype,imagepath,comment,color,seq,templateid) VALUES('0','Production Ready','0','assets/images/meta/SENTRY/type-enhancement.png','','#2e292e','3','16')
INSERT gemini_issuestatus(isfinal,statusdesc,statustype,imagepath,comment,color,seq,templateid) VALUES('0','Assessing','0','assets/images/meta/SENTRY/status-check.png','','#e316e3','4','16')
INSERT gemini_issuestatus(isfinal,statusdesc,statustype,imagepath,comment,color,seq,templateid) VALUES('0','Accepted','0','assets/images/meta/SENTRY/test-acceptance.png','','#126b0d','5','16')
INSERT gemini_issuestatus(isfinal,statusdesc,statustype,imagepath,comment,color,seq,templateid) VALUES('1','Rejected','0','assets/images/meta/SENTRY/status-rejected.png','','#e60330','6','16')
INSERT gemini_issuestatus(isfinal,statusdesc,statustype,imagepath,comment,color,seq,templateid) VALUES('0','In Progress','0','assets/images/meta/SENTRY/status-inprogress.png','','#c95d36','7','16')
INSERT gemini_issuestatus(isfinal,statusdesc,statustype,imagepath,comment,color,seq,templateid) VALUES('0','Ready For Deploy','0','assets/images/meta/SENTRY/test-whitebox.png','','#2e292e','8','16')
INSERT gemini_issuestatus(isfinal,statusdesc,statustype,imagepath,comment,color,seq,templateid) VALUES('0','Deployed','0','assets/images/meta/SENTRY/test-blackbox.png','','#3636d1','9','16')
INSERT gemini_issuestatus(isfinal,statusdesc,statustype,imagepath,comment,color,seq,templateid) VALUES('1','Closed','0','assets/images/meta/SENTRY/status-closed.png','','#818f17','10','16')

UPDATE gemini_issuetypes SET workflow = REPLACE(workflow, '*INBACKLOG*', (SELECT b.statusid FROM gemini_issuestatus b WHERE b.templateid = 10 AND statusdesc COLLATE DATABASE_DEFAULT = 'In Backlog' COLLATE DATABASE_DEFAULT )) WHERE templateid= 10 AND workflow LIKE '%*INBACKLOG*%'
UPDATE gemini_issuetypes SET workflow = REPLACE(workflow, '*INSPRINT*', (SELECT b.statusid FROM gemini_issuestatus b WHERE b.templateid = 10 AND statusdesc COLLATE DATABASE_DEFAULT = 'In Sprint' COLLATE DATABASE_DEFAULT )) WHERE templateid= 10 AND workflow LIKE '%*INSPRINT*%'
UPDATE gemini_issuetypes SET workflow = REPLACE(workflow, '*TESTED*', (SELECT b.statusid FROM gemini_issuestatus b WHERE b.templateid = 10 AND statusdesc COLLATE DATABASE_DEFAULT = 'Tested' COLLATE DATABASE_DEFAULT )) WHERE templateid= 10 AND workflow LIKE '%*TESTED*%'
UPDATE gemini_issuetypes SET workflow = REPLACE(workflow, '*INPROGRESS*', (SELECT b.statusid FROM gemini_issuestatus b WHERE b.templateid = 10 AND statusdesc COLLATE DATABASE_DEFAULT = 'In Progress' COLLATE DATABASE_DEFAULT )) WHERE templateid= 10 AND workflow LIKE '%*INPROGRESS*%'
UPDATE gemini_issuetypes SET workflow = REPLACE(workflow, '*CLOSED*', (SELECT b.statusid FROM gemini_issuestatus b WHERE b.templateid = 10 AND statusdesc COLLATE DATABASE_DEFAULT = 'Closed' COLLATE DATABASE_DEFAULT )) WHERE templateid= 10 AND workflow LIKE '%*CLOSED*%'
UPDATE gemini_issuetypes SET workflow = REPLACE(workflow, '*ESCALATION-LEVEL1*', (SELECT b.statusid FROM gemini_issuestatus b WHERE b.templateid = 11 AND statusdesc COLLATE DATABASE_DEFAULT = 'Escalation-Level1' COLLATE DATABASE_DEFAULT )) WHERE templateid= 11 AND workflow LIKE '%*ESCALATION-LEVEL1*%'
UPDATE gemini_issuetypes SET workflow = REPLACE(workflow, '*ESCALATION-LEVEL2*', (SELECT b.statusid FROM gemini_issuestatus b WHERE b.templateid = 11 AND statusdesc COLLATE DATABASE_DEFAULT = 'Escalation-Level2' COLLATE DATABASE_DEFAULT )) WHERE templateid= 11 AND workflow LIKE '%*ESCALATION-LEVEL2*%'
UPDATE gemini_issuetypes SET workflow = REPLACE(workflow, '*RAISED*', (SELECT b.statusid FROM gemini_issuestatus b WHERE b.templateid = 11 AND statusdesc COLLATE DATABASE_DEFAULT = 'Raised' COLLATE DATABASE_DEFAULT )) WHERE templateid= 11 AND workflow LIKE '%*RAISED*%'
UPDATE gemini_issuetypes SET workflow = REPLACE(workflow, '*USERREJECTED*', (SELECT b.statusid FROM gemini_issuestatus b WHERE b.templateid = 11 AND statusdesc COLLATE DATABASE_DEFAULT = 'User Rejected' COLLATE DATABASE_DEFAULT )) WHERE templateid= 11 AND workflow LIKE '%*USERREJECTED*%'
UPDATE gemini_issuetypes SET workflow = REPLACE(workflow, '*MOVEDTODELIVERY*', (SELECT b.statusid FROM gemini_issuestatus b WHERE b.templateid = 11 AND statusdesc COLLATE DATABASE_DEFAULT = 'Moved to Delivery' COLLATE DATABASE_DEFAULT )) WHERE templateid= 11 AND workflow LIKE '%*MOVEDTODELIVERY*%'
UPDATE gemini_issuetypes SET workflow = REPLACE(workflow, '*ASSIGNED*', (SELECT b.statusid FROM gemini_issuestatus b WHERE b.templateid = 11 AND statusdesc COLLATE DATABASE_DEFAULT = 'Assigned' COLLATE DATABASE_DEFAULT )) WHERE templateid= 11 AND workflow LIKE '%*ASSIGNED*%'
UPDATE gemini_issuetypes SET workflow = REPLACE(workflow, '*CLOSED*', (SELECT b.statusid FROM gemini_issuestatus b WHERE b.templateid = 11 AND statusdesc COLLATE DATABASE_DEFAULT = 'Closed' COLLATE DATABASE_DEFAULT )) WHERE templateid= 11 AND workflow LIKE '%*CLOSED*%'
UPDATE gemini_issuetypes SET workflow = REPLACE(workflow, '*CASEREOPENED*', (SELECT b.statusid FROM gemini_issuestatus b WHERE b.templateid = 11 AND statusdesc COLLATE DATABASE_DEFAULT = 'Case Reopened' COLLATE DATABASE_DEFAULT )) WHERE templateid= 11 AND workflow LIKE '%*CASEREOPENED*%'
UPDATE gemini_issuetypes SET workflow = REPLACE(workflow, '*USERACCEPTED*', (SELECT b.statusid FROM gemini_issuestatus b WHERE b.templateid = 11 AND statusdesc COLLATE DATABASE_DEFAULT = 'User Accepted' COLLATE DATABASE_DEFAULT )) WHERE templateid= 11 AND workflow LIKE '%*USERACCEPTED*%'
UPDATE gemini_issuetypes SET workflow = REPLACE(workflow, '*PENDINGAPPROVAL*', (SELECT b.statusid FROM gemini_issuestatus b WHERE b.templateid = 11 AND statusdesc COLLATE DATABASE_DEFAULT = 'Pending Approval' COLLATE DATABASE_DEFAULT )) WHERE templateid= 11 AND workflow LIKE '%*PENDINGAPPROVAL*%'
UPDATE gemini_issuetypes SET workflow = REPLACE(workflow, '*APPROVED*', (SELECT b.statusid FROM gemini_issuestatus b WHERE b.templateid = 11 AND statusdesc COLLATE DATABASE_DEFAULT = 'Approved' COLLATE DATABASE_DEFAULT )) WHERE templateid= 11 AND workflow LIKE '%*APPROVED*%'
UPDATE gemini_issuetypes SET workflow = REPLACE(workflow, '*UNASSIGNED*', (SELECT b.statusid FROM gemini_issuestatus b WHERE b.templateid = 12 AND statusdesc COLLATE DATABASE_DEFAULT = 'Unassigned' COLLATE DATABASE_DEFAULT )) WHERE templateid= 12 AND workflow LIKE '%*UNASSIGNED*%'
UPDATE gemini_issuetypes SET workflow = REPLACE(workflow, '*INPROGRESS*', (SELECT b.statusid FROM gemini_issuestatus b WHERE b.templateid = 12 AND statusdesc COLLATE DATABASE_DEFAULT = 'In Progress' COLLATE DATABASE_DEFAULT )) WHERE templateid= 12 AND workflow LIKE '%*INPROGRESS*%'
UPDATE gemini_issuetypes SET workflow = REPLACE(workflow, '*CLOSED*', (SELECT b.statusid FROM gemini_issuestatus b WHERE b.templateid = 12 AND statusdesc COLLATE DATABASE_DEFAULT = 'Closed' COLLATE DATABASE_DEFAULT )) WHERE templateid= 12 AND workflow LIKE '%*CLOSED*%'
UPDATE gemini_issuetypes SET workflow = REPLACE(workflow, '*APPROVED*', (SELECT b.statusid FROM gemini_issuestatus b WHERE b.templateid = 12 AND statusdesc COLLATE DATABASE_DEFAULT = 'Approved' COLLATE DATABASE_DEFAULT )) WHERE templateid= 12 AND workflow LIKE '%*APPROVED*%'
UPDATE gemini_issuetypes SET workflow = REPLACE(workflow, '*UNASSIGNED*', (SELECT b.statusid FROM gemini_issuestatus b WHERE b.templateid = 13 AND statusdesc COLLATE DATABASE_DEFAULT = 'Unassigned' COLLATE DATABASE_DEFAULT )) WHERE templateid= 13 AND workflow LIKE '%*UNASSIGNED*%'
UPDATE gemini_issuetypes SET workflow = REPLACE(workflow, '*ASSIGNED*', (SELECT b.statusid FROM gemini_issuestatus b WHERE b.templateid = 13 AND statusdesc COLLATE DATABASE_DEFAULT = 'Assigned' COLLATE DATABASE_DEFAULT )) WHERE templateid= 13 AND workflow LIKE '%*ASSIGNED*%'
UPDATE gemini_issuetypes SET workflow = REPLACE(workflow, '*INPROGRESS*', (SELECT b.statusid FROM gemini_issuestatus b WHERE b.templateid = 13 AND statusdesc COLLATE DATABASE_DEFAULT = 'In Progress' COLLATE DATABASE_DEFAULT )) WHERE templateid= 13 AND workflow LIKE '%*INPROGRESS*%'
UPDATE gemini_issuetypes SET workflow = REPLACE(workflow, '*CLOSED*', (SELECT b.statusid FROM gemini_issuestatus b WHERE b.templateid = 13 AND statusdesc COLLATE DATABASE_DEFAULT = 'Closed' COLLATE DATABASE_DEFAULT )) WHERE templateid= 13 AND workflow LIKE '%*CLOSED*%'
UPDATE gemini_issuetypes SET workflow = REPLACE(workflow, '*REOPENED*', (SELECT b.statusid FROM gemini_issuestatus b WHERE b.templateid = 13 AND statusdesc COLLATE DATABASE_DEFAULT = 'Reopened' COLLATE DATABASE_DEFAULT )) WHERE templateid= 13 AND workflow LIKE '%*REOPENED*%'
UPDATE gemini_issuetypes SET workflow = REPLACE(workflow, '*PLANNED*', (SELECT b.statusid FROM gemini_issuestatus b WHERE b.templateid = 14 AND statusdesc COLLATE DATABASE_DEFAULT = 'Planned' COLLATE DATABASE_DEFAULT )) WHERE templateid= 14 AND workflow LIKE '%*PLANNED*%'
UPDATE gemini_issuetypes SET workflow = REPLACE(workflow, '*INPROGRESS*', (SELECT b.statusid FROM gemini_issuestatus b WHERE b.templateid = 14 AND statusdesc COLLATE DATABASE_DEFAULT = 'In Progress' COLLATE DATABASE_DEFAULT )) WHERE templateid= 14 AND workflow LIKE '%*INPROGRESS*%'
UPDATE gemini_issuetypes SET workflow = REPLACE(workflow, '*COMPLETED*', (SELECT b.statusid FROM gemini_issuestatus b WHERE b.templateid = 14 AND statusdesc COLLATE DATABASE_DEFAULT = 'Completed' COLLATE DATABASE_DEFAULT )) WHERE templateid= 14 AND workflow LIKE '%*COMPLETED*%'
UPDATE gemini_issuetypes SET workflow = REPLACE(workflow, '*PARKED*', (SELECT b.statusid FROM gemini_issuestatus b WHERE b.templateid = 14 AND statusdesc COLLATE DATABASE_DEFAULT = 'Parked' COLLATE DATABASE_DEFAULT )) WHERE templateid= 14 AND workflow LIKE '%*PARKED*%'
UPDATE gemini_issuetypes SET workflow = REPLACE(workflow, '*APPROVED*', (SELECT b.statusid FROM gemini_issuestatus b WHERE b.templateid = 15 AND statusdesc COLLATE DATABASE_DEFAULT = 'Approved' COLLATE DATABASE_DEFAULT )) WHERE templateid= 15 AND workflow LIKE '%*APPROVED*%'
UPDATE gemini_issuetypes SET workflow = REPLACE(workflow, '*REQUIREMENT*', (SELECT b.statusid FROM gemini_issuestatus b WHERE b.templateid = 15 AND statusdesc COLLATE DATABASE_DEFAULT = 'Requirement' COLLATE DATABASE_DEFAULT )) WHERE templateid= 15 AND workflow LIKE '%*REQUIREMENT*%'
UPDATE gemini_issuetypes SET workflow = REPLACE(workflow, '*WALKTHROUGH*', (SELECT b.statusid FROM gemini_issuestatus b WHERE b.templateid = 15 AND statusdesc COLLATE DATABASE_DEFAULT = 'Walkthrough' COLLATE DATABASE_DEFAULT )) WHERE templateid= 15 AND workflow LIKE '%*WALKTHROUGH*%'
UPDATE gemini_issuetypes SET workflow = REPLACE(workflow, '*CLOSED*', (SELECT b.statusid FROM gemini_issuestatus b WHERE b.templateid = 15 AND statusdesc COLLATE DATABASE_DEFAULT = 'Closed' COLLATE DATABASE_DEFAULT )) WHERE templateid= 15 AND workflow LIKE '%*CLOSED*%'
UPDATE gemini_issuetypes SET workflow = REPLACE(workflow, '*REOPENED*', (SELECT b.statusid FROM gemini_issuestatus b WHERE b.templateid = 15 AND statusdesc COLLATE DATABASE_DEFAULT = 'Reopened' COLLATE DATABASE_DEFAULT )) WHERE templateid= 15 AND workflow LIKE '%*REOPENED*%'
UPDATE gemini_issuetypes SET workflow = REPLACE(workflow, '*INPROGRESS*', (SELECT b.statusid FROM gemini_issuestatus b WHERE b.templateid = 15 AND statusdesc COLLATE DATABASE_DEFAULT = 'In Progress' COLLATE DATABASE_DEFAULT )) WHERE templateid= 15 AND workflow LIKE '%*INPROGRESS*%'
UPDATE gemini_issuetypes SET workflow = REPLACE(workflow, '*REJECTED*', (SELECT b.statusid FROM gemini_issuestatus b WHERE b.templateid = 12 AND statusdesc COLLATE DATABASE_DEFAULT = 'Rejected' COLLATE DATABASE_DEFAULT )) WHERE templateid= 12 AND workflow LIKE '%*REJECTED*%'
UPDATE gemini_issuetypes SET workflow = REPLACE(workflow, '*REPORTED*', (SELECT b.statusid FROM gemini_issuestatus b WHERE b.templateid = 16 AND statusdesc COLLATE DATABASE_DEFAULT = 'Reported' COLLATE DATABASE_DEFAULT )) WHERE templateid= 16 AND workflow LIKE '%*REPORTED*%'
UPDATE gemini_issuetypes SET workflow = REPLACE(workflow, '*PLANNED*', (SELECT b.statusid FROM gemini_issuestatus b WHERE b.templateid = 16 AND statusdesc COLLATE DATABASE_DEFAULT = 'Planned' COLLATE DATABASE_DEFAULT )) WHERE templateid= 16 AND workflow LIKE '%*PLANNED*%'
UPDATE gemini_issuetypes SET workflow = REPLACE(workflow, '*PRODUCTIONREADY*', (SELECT b.statusid FROM gemini_issuestatus b WHERE b.templateid = 16 AND statusdesc COLLATE DATABASE_DEFAULT = 'Production Ready' COLLATE DATABASE_DEFAULT )) WHERE templateid= 16 AND workflow LIKE '%*PRODUCTIONREADY*%'
UPDATE gemini_issuetypes SET workflow = REPLACE(workflow, '*ASSESSING*', (SELECT b.statusid FROM gemini_issuestatus b WHERE b.templateid = 16 AND statusdesc COLLATE DATABASE_DEFAULT = 'Assessing' COLLATE DATABASE_DEFAULT )) WHERE templateid= 16 AND workflow LIKE '%*ASSESSING*%'
UPDATE gemini_issuetypes SET workflow = REPLACE(workflow, '*ACCEPTED*', (SELECT b.statusid FROM gemini_issuestatus b WHERE b.templateid = 16 AND statusdesc COLLATE DATABASE_DEFAULT = 'Accepted' COLLATE DATABASE_DEFAULT )) WHERE templateid= 16 AND workflow LIKE '%*ACCEPTED*%'
UPDATE gemini_issuetypes SET workflow = REPLACE(workflow, '*REJECTED*', (SELECT b.statusid FROM gemini_issuestatus b WHERE b.templateid = 16 AND statusdesc COLLATE DATABASE_DEFAULT = 'Rejected' COLLATE DATABASE_DEFAULT )) WHERE templateid= 16 AND workflow LIKE '%*REJECTED*%'
UPDATE gemini_issuetypes SET workflow = REPLACE(workflow, '*INPROGRESS*', (SELECT b.statusid FROM gemini_issuestatus b WHERE b.templateid = 16 AND statusdesc COLLATE DATABASE_DEFAULT = 'In Progress' COLLATE DATABASE_DEFAULT )) WHERE templateid= 16 AND workflow LIKE '%*INPROGRESS*%'
UPDATE gemini_issuetypes SET workflow = REPLACE(workflow, '*READYFORDEPLOY*', (SELECT b.statusid FROM gemini_issuestatus b WHERE b.templateid = 16 AND statusdesc COLLATE DATABASE_DEFAULT = 'Ready For Deploy' COLLATE DATABASE_DEFAULT )) WHERE templateid= 16 AND workflow LIKE '%*READYFORDEPLOY*%'
UPDATE gemini_issuetypes SET workflow = REPLACE(workflow, '*DEPLOYED*', (SELECT b.statusid FROM gemini_issuestatus b WHERE b.templateid = 16 AND statusdesc COLLATE DATABASE_DEFAULT = 'Deployed' COLLATE DATABASE_DEFAULT )) WHERE templateid= 16 AND workflow LIKE '%*DEPLOYED*%'
UPDATE gemini_issuetypes SET workflow = REPLACE(workflow, '*CLOSED*', (SELECT b.statusid FROM gemini_issuestatus b WHERE b.templateid = 16 AND statusdesc COLLATE DATABASE_DEFAULT = 'Closed' COLLATE DATABASE_DEFAULT )) WHERE templateid= 16 AND workflow LIKE '%*CLOSED*%'

INSERT gemini_issueresolutions(seq,isfinal,resdesc,templateid) VALUES('1','0','Unresolved','10')
INSERT gemini_issueresolutions(seq,isfinal,resdesc,templateid) VALUES('5','1','Complete','10')
INSERT gemini_issueresolutions(seq,isfinal,resdesc,templateid) VALUES('1','1','Back to Client','11')
INSERT gemini_issueresolutions(seq,isfinal,resdesc,templateid) VALUES('2','1','Complete','11')
INSERT gemini_issueresolutions(seq,isfinal,resdesc,templateid) VALUES('1','0','Unresolved','12')
INSERT gemini_issueresolutions(seq,isfinal,resdesc,templateid) VALUES('2','1','Complete','12')
INSERT gemini_issueresolutions(seq,isfinal,resdesc,templateid) VALUES('1','0','Unresolved','13')
INSERT gemini_issueresolutions(seq,isfinal,resdesc,templateid) VALUES('5','1','Complete','13')
INSERT gemini_issueresolutions(seq,isfinal,resdesc,templateid) VALUES('1','0','Unresolved','14')
INSERT gemini_issueresolutions(seq,isfinal,resdesc,templateid) VALUES('3','1','Complete','14')
INSERT gemini_issueresolutions(seq,isfinal,resdesc,templateid) VALUES('2','0','Parked','15')
INSERT gemini_issueresolutions(seq,isfinal,resdesc,templateid) VALUES('4','1','Complete','15')
INSERT gemini_issueresolutions(seq,isfinal,resdesc,templateid) VALUES('4','1','Rejected','10')
INSERT gemini_issueresolutions(seq,isfinal,resdesc,templateid) VALUES('3','1','Duplicate','10')
INSERT gemini_issueresolutions(seq,isfinal,resdesc,templateid) VALUES('2','0','Parked','10')
INSERT gemini_issueresolutions(seq,isfinal,resdesc,templateid) VALUES('3','1','Duplicate','13')
INSERT gemini_issueresolutions(seq,isfinal,resdesc,templateid) VALUES('4','1','Rejected','13')
INSERT gemini_issueresolutions(seq,isfinal,resdesc,templateid) VALUES('2','0','Parked','13')
INSERT gemini_issueresolutions(seq,isfinal,resdesc,templateid) VALUES('0','0','Unresolved','11')
INSERT gemini_issueresolutions(seq,isfinal,resdesc,templateid) VALUES('0','1','Rejected','12')
INSERT gemini_issueresolutions(seq,isfinal,resdesc,templateid) VALUES('2','0','Parked','14')
INSERT gemini_issueresolutions(seq,isfinal,resdesc,templateid) VALUES('1','0','Unresolved','15')
INSERT gemini_issueresolutions(seq,isfinal,resdesc,templateid) VALUES('3','0','Rejected','15')
INSERT gemini_issueresolutions(seq,isfinal,resdesc,templateid) VALUES('1','0','Unresolved','16')
INSERT gemini_issueresolutions(seq,isfinal,resdesc,templateid) VALUES('2','1','Rejected','16')
INSERT gemini_issueresolutions(seq,isfinal,resdesc,templateid) VALUES('3','1','Resolved','16')

INSERT gemini_issuelinktypes(linkname,linkdesc,templateid) VALUES('Duplicated','','10')
INSERT gemini_issuelinktypes(linkname,linkdesc,templateid) VALUES('Related','','10')
INSERT gemini_issuelinktypes(linkname,linkdesc,templateid) VALUES('Duplicated','','11')
INSERT gemini_issuelinktypes(linkname,linkdesc,templateid) VALUES('Related','','11')
INSERT gemini_issuelinktypes(linkname,linkdesc,templateid) VALUES('Duplicated','','12')
INSERT gemini_issuelinktypes(linkname,linkdesc,templateid) VALUES('Related','','12')
INSERT gemini_issuelinktypes(linkname,linkdesc,templateid) VALUES('Duplicated','','13')
INSERT gemini_issuelinktypes(linkname,linkdesc,templateid) VALUES('Related','','13')
INSERT gemini_issuelinktypes(linkname,linkdesc,templateid) VALUES('Duplicated','','14')
INSERT gemini_issuelinktypes(linkname,linkdesc,templateid) VALUES('Related','','14')
INSERT gemini_issuelinktypes(linkname,linkdesc,templateid) VALUES('Duplicated','','15')
INSERT gemini_issuelinktypes(linkname,linkdesc,templateid) VALUES('Related','','15')
INSERT gemini_issuelinktypes(linkname,linkdesc,templateid) VALUES('Duplicated','','16')
INSERT gemini_issuelinktypes(linkname,linkdesc,templateid) VALUES('Related','','16')

INSERT gemini_issuetimetype(timetypename,timetypedesc,templateid) VALUES('Billable','','10')
INSERT gemini_issuetimetype(timetypename,timetypedesc,templateid) VALUES('Non-Billable','','10')
INSERT gemini_issuetimetype(timetypename,timetypedesc,templateid) VALUES('Internal','','10')
INSERT gemini_issuetimetype(timetypename,timetypedesc,templateid) VALUES('Scrum Master PM','','10')
INSERT gemini_issuetimetype(timetypename,timetypedesc,templateid) VALUES('Testing','','10')
INSERT gemini_issuetimetype(timetypename,timetypedesc,templateid) VALUES('Billable','','11')
INSERT gemini_issuetimetype(timetypename,timetypedesc,templateid) VALUES('Non-Billable','','11')
INSERT gemini_issuetimetype(timetypename,timetypedesc,templateid) VALUES('Internal','','11')
INSERT gemini_issuetimetype(timetypename,timetypedesc,templateid) VALUES('Billable','','12')
INSERT gemini_issuetimetype(timetypename,timetypedesc,templateid) VALUES('Non-Billable','','12')
INSERT gemini_issuetimetype(timetypename,timetypedesc,templateid) VALUES('Internal','','12')
INSERT gemini_issuetimetype(timetypename,timetypedesc,templateid) VALUES('Scrum Master PM','','12')
INSERT gemini_issuetimetype(timetypename,timetypedesc,templateid) VALUES('Testing','','12')
INSERT gemini_issuetimetype(timetypename,timetypedesc,templateid) VALUES('Billable','','13')
INSERT gemini_issuetimetype(timetypename,timetypedesc,templateid) VALUES('Non-Billable','','13')
INSERT gemini_issuetimetype(timetypename,timetypedesc,templateid) VALUES('Internal','','13')
INSERT gemini_issuetimetype(timetypename,timetypedesc,templateid) VALUES('Scrum Master PM','','13')
INSERT gemini_issuetimetype(timetypename,timetypedesc,templateid) VALUES('Testing','','13')
INSERT gemini_issuetimetype(timetypename,timetypedesc,templateid) VALUES('Billable','','14')
INSERT gemini_issuetimetype(timetypename,timetypedesc,templateid) VALUES('non-billable','','14')
INSERT gemini_issuetimetype(timetypename,timetypedesc,templateid) VALUES('Internal','','14')
INSERT gemini_issuetimetype(timetypename,timetypedesc,templateid) VALUES('Billable','','15')
INSERT gemini_issuetimetype(timetypename,timetypedesc,templateid) VALUES('Non-Billable','','15')
INSERT gemini_issuetimetype(timetypename,timetypedesc,templateid) VALUES('Internal','','15')
INSERT gemini_issuetimetype(timetypename,timetypedesc,templateid) VALUES('Scrum Master PM','','15')
INSERT gemini_issuetimetype(timetypename,timetypedesc,templateid) VALUES('Testing','','15')
INSERT gemini_issuetimetype(timetypename,timetypedesc,templateid) VALUES('Billable','','16')
INSERT gemini_issuetimetype(timetypename,timetypedesc,templateid) VALUES('Non-billable','','16')
INSERT gemini_issuetimetype(timetypename,timetypedesc,templateid) VALUES('Internal','','16')

INSERT gemini_customfielddefinitions(customfieldname,screenorder,screenlabel,screendescription,screentooltip,maxlen,canmultiselect,regularexp,customfieldtype,lookupname,lookupvaluefield,lookuptextfield,lookupsortfield,projectidfilter,showinline,lookupdata,maxvalue,minvalue,listlimiter,usedin,templateid,usestaticdata) VALUES('Customer','0','Customer','Our Client name','Select the customer from the list','0','1','','C','','','','','1','1','Customer A
Customer B
Customer C
','0.00000','0.00000','','0','10','1')
INSERT gemini_customfielddefinitions(customfieldname,screenorder,screenlabel,screendescription,screentooltip,maxlen,canmultiselect,regularexp,customfieldtype,lookupname,lookupvaluefield,lookuptextfield,lookupsortfield,projectidfilter,showinline,lookupdata,maxvalue,minvalue,listlimiter,usedin,templateid,usestaticdata) VALUES('Customer','0','Customer','','Select the customer from the list','0','1','','L','','','','','1','1','Customer A
Customer B
Customer C
','0.00000','0.00000','','0','11','1')
INSERT gemini_customfielddefinitions(customfieldname,screenorder,screenlabel,screendescription,screentooltip,maxlen,canmultiselect,regularexp,customfieldtype,lookupname,lookupvaluefield,lookuptextfield,lookupsortfield,projectidfilter,showinline,lookupdata,maxvalue,minvalue,listlimiter,usedin,templateid,usestaticdata) VALUES('Product Platform','0','Product Platform (OS)','','Product Platform (OS)','0','0','','C','','','','','0','1','Windows 7
Windows 8
Linux
Ubuntu
Mac OS
iPad
','0.00000','0.00000','','0','11','1')
INSERT gemini_customfielddefinitions(customfieldname,screenorder,screenlabel,screendescription,screentooltip,maxlen,canmultiselect,regularexp,customfieldtype,lookupname,lookupvaluefield,lookuptextfield,lookupsortfield,projectidfilter,showinline,lookupdata,maxvalue,minvalue,listlimiter,usedin,templateid,usestaticdata) VALUES('Needs Approval','0','Needs Approval','','Needs Approval','0','0','','Y','','','','','0','1','                                              ','0.00000','0.00000','','0','11','1')
INSERT gemini_customfielddefinitions(customfieldname,screenorder,screenlabel,screendescription,screentooltip,maxlen,canmultiselect,regularexp,customfieldtype,lookupname,lookupvaluefield,lookuptextfield,lookupsortfield,projectidfilter,showinline,lookupdata,maxvalue,minvalue,listlimiter,usedin,templateid,usestaticdata) VALUES('Customer','0','Customer','Our Client name','Select the customer from the list','0','1','','L','','','','','1','1','Customer A
Customer B
Customer C
','0.00000','0.00000','','0','12','1')
INSERT gemini_customfielddefinitions(customfieldname,screenorder,screenlabel,screendescription,screentooltip,maxlen,canmultiselect,regularexp,customfieldtype,lookupname,lookupvaluefield,lookuptextfield,lookupsortfield,projectidfilter,showinline,lookupdata,maxvalue,minvalue,listlimiter,usedin,templateid,usestaticdata) VALUES('User Application','0','User Application','','User Application','0','0','','C','','','','','0','1','Peoplesoft Financials
Peoplesoft HR
Dynamics CRM
OURSYS
Inventory Management
Other
','0.00000','0.00000','','0','12','1')
INSERT gemini_customfielddefinitions(customfieldname,screenorder,screenlabel,screendescription,screentooltip,maxlen,canmultiselect,regularexp,customfieldtype,lookupname,lookupvaluefield,lookuptextfield,lookupsortfield,projectidfilter,showinline,lookupdata,maxvalue,minvalue,listlimiter,usedin,templateid,usestaticdata) VALUES('Product Platform','0','Product Platform (OS)','','Product Platform (OS)','0','0','','C','','','','','0','1','Windows 7
Windows 8
Linux
Ubuntu
Mac OS
iPad
','0.00000','0.00000','','0','12','1')
INSERT gemini_customfielddefinitions(customfieldname,screenorder,screenlabel,screendescription,screentooltip,maxlen,canmultiselect,regularexp,customfieldtype,lookupname,lookupvaluefield,lookuptextfield,lookupsortfield,projectidfilter,showinline,lookupdata,maxvalue,minvalue,listlimiter,usedin,templateid,usestaticdata) VALUES('Customer','0','Customer','Our Client name','Select the customer from the list','0','1','','L','','','','','1','1','Customer A
Customer B
Customer C
','0.00000','0.00000','','0','13','1')
INSERT gemini_customfielddefinitions(customfieldname,screenorder,screenlabel,screendescription,screentooltip,maxlen,canmultiselect,regularexp,customfieldtype,lookupname,lookupvaluefield,lookuptextfield,lookupsortfield,projectidfilter,showinline,lookupdata,maxvalue,minvalue,listlimiter,usedin,templateid,usestaticdata) VALUES('User Application','0','User Application','','User Application','0','0','','C','','','','','0','1','Peoplesoft Financials
Peoplesoft HR
Dynamics CRM
OURSYS
Inventory Management
Other
','0.00000','0.00000','','0','13','1')
INSERT gemini_customfielddefinitions(customfieldname,screenorder,screenlabel,screendescription,screentooltip,maxlen,canmultiselect,regularexp,customfieldtype,lookupname,lookupvaluefield,lookuptextfield,lookupsortfield,projectidfilter,showinline,lookupdata,maxvalue,minvalue,listlimiter,usedin,templateid,usestaticdata) VALUES('Product Platform','0','Product Platform (OS)','','Product Platform (OS)','0','0','','C','','','','','0','1','Windows 7
Windows 8
Linux
Ubuntu
Mac OS
iPad
','0.00000','0.00000','','0','13','1')
INSERT gemini_customfielddefinitions(customfieldname,screenorder,screenlabel,screendescription,screentooltip,maxlen,canmultiselect,regularexp,customfieldtype,lookupname,lookupvaluefield,lookuptextfield,lookupsortfield,projectidfilter,showinline,lookupdata,maxvalue,minvalue,listlimiter,usedin,templateid,usestaticdata) VALUES('Owner','0','Owner','Who ownes it?','Select the owner from the list','0','0','','C','','','','','0','1','N/A
Me
You
Some Else
','0.00000','0.00000','','0','14','1')
INSERT gemini_customfielddefinitions(customfieldname,screenorder,screenlabel,screendescription,screentooltip,maxlen,canmultiselect,regularexp,customfieldtype,lookupname,lookupvaluefield,lookuptextfield,lookupsortfield,projectidfilter,showinline,lookupdata,maxvalue,minvalue,listlimiter,usedin,templateid,usestaticdata) VALUES('Agencies','0','Agencies','','Agencies Involved','0','0','','C','','','','','0','1','N/A
MRM Worldwide
Mcann
Ogilvy and Mather
Saatchi and Saatchi
Jalebi
Jellbean Creative
Other
','0.00000','0.00000','','0','14','1')
INSERT gemini_customfielddefinitions(customfieldname,screenorder,screenlabel,screendescription,screentooltip,maxlen,canmultiselect,regularexp,customfieldtype,lookupname,lookupvaluefield,lookuptextfield,lookupsortfield,projectidfilter,showinline,lookupdata,maxvalue,minvalue,listlimiter,usedin,templateid,usestaticdata) VALUES('Budget Approved','0','Budget Approved','','Budget Approver','0','0','','Y','','','','','0','1','                                              ','0.00000','0.00000','','0','14','1')
INSERT gemini_customfielddefinitions(customfieldname,screenorder,screenlabel,screendescription,screentooltip,maxlen,canmultiselect,regularexp,customfieldtype,lookupname,lookupvaluefield,lookuptextfield,lookupsortfield,projectidfilter,showinline,lookupdata,maxvalue,minvalue,listlimiter,usedin,templateid,usestaticdata) VALUES('Customer','0','Customer','Our Client name','Select the customer from the list','0','1','','C','','','','','1','1','Customer A
Customer B
Customer C
','0.00000','0.00000','','0','15','1')
INSERT gemini_customfielddefinitions(customfieldname,screenorder,screenlabel,screendescription,screentooltip,maxlen,canmultiselect,regularexp,customfieldtype,lookupname,lookupvaluefield,lookuptextfield,lookupsortfield,projectidfilter,showinline,lookupdata,maxvalue,minvalue,listlimiter,usedin,templateid,usestaticdata) VALUES('Client/Project','0','Client/Project','','Client/Project','0','0','','C','','','','','0','1','N/A
Client A - Project A
Client A - Project B
Client A - Project C
Client B - Project 1
Client B - Project 2
Client C  Client D
','0.00000','0.00000','','0','16','1')

INSERT gemini_customfielddefinitions(customfieldname,screenorder,screenlabel,screendescription,screentooltip,maxlen,canmultiselect,regularexp,customfieldtype,lookupname,lookupvaluefield,lookuptextfield,lookupsortfield,projectidfilter,showinline,lookupdata,maxvalue,minvalue,listlimiter,usedin,templateid,usestaticdata) VALUES('Affected User(s)','0','Affected User(s)','','Affected User(s)','0','1','','U','','','','','0','1','                                              ','0.00000','0.00000','4|','1','16','0')
INSERT gemini_customfielddefinitions(customfieldname,screenorder,screenlabel,screendescription,screentooltip,maxlen,canmultiselect,regularexp,customfieldtype,lookupname,lookupvaluefield,lookuptextfield,lookupsortfield,projectidfilter,showinline,lookupdata,maxvalue,minvalue,listlimiter,usedin,templateid,usestaticdata) VALUES('Test Type','0','Test Type','','Test Type','0','0','','C','gemini_testing_types','typeid','typedesc','typedesc','0','1','','0.00000','0.00000','','1','16','0')
INSERT gemini_customfielddefinitions(customfieldname,screenorder,screenlabel,screendescription,screentooltip,maxlen,canmultiselect,regularexp,customfieldtype,lookupname,lookupvaluefield,lookuptextfield,lookupsortfield,projectidfilter,showinline,lookupdata,maxvalue,minvalue,listlimiter,usedin,templateid,usestaticdata) VALUES('Preconditions','0','Pre-Conditions','','Pre-Conditions','250','0','','R','','','','','0','0','                                              ','0.00000','0.00000','','1','16','0')
INSERT gemini_customfielddefinitions(customfieldname,screenorder,screenlabel,screendescription,screentooltip,maxlen,canmultiselect,regularexp,customfieldtype,lookupname,lookupvaluefield,lookuptextfield,lookupsortfield,projectidfilter,showinline,lookupdata,maxvalue,minvalue,listlimiter,usedin,templateid,usestaticdata) VALUES('Expected Results','0','Expected Results','','Expected Results','250','0','','R','','','','','0','0','                                              ','0.00000','0.00000','','1','16','0')
GO

UPDATE gemini_issuetypes SET screen='{"ReferenceId":0,"Create":[],"Edit":[],"View":[{"ProjectGroups":[1],"Required":false,"Sequence":0,"Label":"","Id":"IssueKey","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"Title","Type":0}],"IsNew":true,"IsExisting":false,"Active":true,"Archived":false,"Deleted":false,"Created":"2012-08-01T17:17:29.9185046Z","Revised":"2012-08-01T17:17:29.9185046Z","Errors":[],"HasErrors":false,"Id":0}'
	WHERE templateid=1
GO

/**********************************************************************************************************************
* End
**********************************************************************************************************************/

/*********************************************************************************************************************
* GROUPS MIGRATION
*********************************************************************************************************************/
ALTER TABLE gemini_projectgroupmembership ALTER COLUMN projectid NUMERIC(10,0) NULL
GO
ALTER TABLE gemini_admappings DROP CONSTRAINT gemini_admappings_globalgroups_fk 
GO
ALTER TABLE gemini_globalgroupmembership DROP CONSTRAINT gemini_globalgroupmembership_userid_fk
GO

IF EXISTS(SELECT name FROM sysobjects WHERE name COLLATE DATABASE_DEFAULT ='gemini_backupV5_projectgroups' COLLATE DATABASE_DEFAULT ) DROP TABLE gemini_backupV5_projectgroups
GO
IF EXISTS(SELECT name FROM sysobjects WHERE name COLLATE DATABASE_DEFAULT = 'gemini_backupV5_projectgroupmembership' COLLATE DATABASE_DEFAULT ) DROP TABLE gemini_backupV5_projectgroupmembership 
GO
SELECT * INTO gemini_backupV5_projectgroups FROM gemini_projectgroups
SELECT * INTO gemini_backupV5_projectgroupmembership FROM gemini_projectgroupmembership

BEGIN TRANSACTION

DECLARE @ID NUMERIC(10,0)
DECLARE @TargetGroupId NUMERIC(10,0)

SELECT @ID=MAX(projectgroupid)+1 FROM gemini_projectgroups
SET @TargetGroupId = 1

IF(EXISTS(SELECT projectgroupid FROM gemini_projectgroups WHERE projectgroupid= @TargetGroupId))
BEGIN
	
	SET IDENTITY_INSERT gemini_projectgroups ON
	INSERT INTO gemini_projectgroups(projectgroupid, projectgroupname, projectgroupdesc)
	SELECT @ID, projectgroupname, projectgroupdesc FROM gemini_projectgroups WHERE projectgroupid=@TargetGroupId
	SELECT @ID = SCOPE_IDENTITY()
	SET IDENTITY_INSERT gemini_projectgroups OFF
	UPDATE gemini_issues SET visibility=@ID WHERE visibilitymembertype = 3 AND visibility = @TargetGroupId
	UPDATE gemini_issuecomments SET visibility=@ID WHERE visibilitymembertype = 3 AND visibility = @TargetGroupId
	UPDATE gemini_projectgroupmembership SET projectgroupid = @ID WHERE projectgroupid = @TargetGroupId
	SET @ID = @ID + 1
END

SET @TargetGroupId = 2

IF(EXISTS(SELECT projectgroupid FROM gemini_projectgroups WHERE projectgroupid= @TargetGroupId))
BEGIN
	
	SET IDENTITY_INSERT gemini_projectgroups ON
	INSERT INTO gemini_projectgroups(projectgroupid, projectgroupname, projectgroupdesc)
	SELECT @ID, projectgroupname, projectgroupdesc FROM gemini_projectgroups WHERE projectgroupid=@TargetGroupId
	SELECT @ID = SCOPE_IDENTITY()
	SET IDENTITY_INSERT gemini_projectgroups OFF
	UPDATE gemini_issues SET visibility=@ID WHERE visibilitymembertype = 3 AND visibility = @TargetGroupId
	UPDATE gemini_issuecomments SET visibility=@ID WHERE visibilitymembertype = 3 AND visibility = @TargetGroupId
	UPDATE gemini_projectgroupmembership SET projectgroupid = @ID WHERE projectgroupid = @TargetGroupId
	SET @ID = @ID + 1
END

SET @TargetGroupId = 3

IF(EXISTS(SELECT projectgroupid FROM gemini_projectgroups WHERE projectgroupid= @TargetGroupId))
BEGIN
	
	SET IDENTITY_INSERT gemini_projectgroups ON
	INSERT INTO gemini_projectgroups(projectgroupid, projectgroupname, projectgroupdesc)
	SELECT @ID, projectgroupname, projectgroupdesc FROM gemini_projectgroups WHERE projectgroupid=@TargetGroupId
	SELECT @ID = SCOPE_IDENTITY()
	SET IDENTITY_INSERT gemini_projectgroups OFF
	UPDATE gemini_issues SET visibility=@ID WHERE visibilitymembertype = 3 AND visibility = @TargetGroupId
	UPDATE gemini_issuecomments SET visibility=@ID WHERE visibilitymembertype = 3 AND visibility = @TargetGroupId
	UPDATE gemini_projectgroupmembership SET projectgroupid = @ID WHERE projectgroupid = @TargetGroupId
	SET @ID = @ID + 1
END

DELETE FROM gemini_projectgroups WHERE projectgroupid IN (1,2,3)
SET IDENTITY_INSERT gemini_projectgroups ON
	INSERT INTO gemini_projectgroups(projectgroupid, projectgroupname, projectgroupdesc)
	VALUES (1, 'Everyone', 'All users within Gemini')
	INSERT INTO gemini_projectgroups(projectgroupid, projectgroupname, projectgroupdesc)
	VALUES (2, 'Gemini Administrators', 'Gemini administrators')
	INSERT INTO gemini_projectgroups(projectgroupid, projectgroupname, projectgroupdesc)
	VALUES (3, 'Everyone (Authenticated)', 'Authenticated users')
SET IDENTITY_INSERT gemini_projectgroups OFF

INSERT INTO gemini_projectgroupmembership(projectgroupid, projectid, userid)
	SELECT globalgroupid, NULL, userid FROM gemini_globalgroupmembership WHERE globalgroupid IN (1,2,3)

DECLARE groups_cursor CURSOR LOCAL FORWARD_ONLY FOR SELECT globalgroupid, globalgroupname, globalgroupdesc FROM gemini_globalgroups WHERE globalgroupid NOT IN (1,2,3)
OPEN groups_cursor 

DECLARE @GlobalGroupId NUMERIC(10,0)
DECLARE @GlobalName NVARCHAR(1000)
DECLARE @GlobalDesc NVARCHAR(1000)

FETCH NEXT FROM groups_cursor INTO @GlobalGroupId, @GlobalName, @GlobalDesc
WHILE @@FETCH_STATUS =0
BEGIN
	INSERT INTO gemini_projectgroups(projectgroupname, projectgroupdesc)
	VALUES (@GlobalName, @GlobalDesc)
	SELECT @ID = SCOPE_IDENTITY()
	INSERT INTO gemini_projectgroupmembership(projectgroupid, projectid, userid)
		SELECT @ID, NULL, userid FROM gemini_globalgroupmembership WHERE globalgroupid = @GlobalGroupId
		
	UPDATE gemini_issues SET visibility=@ID WHERE visibilitymembertype = 2 AND visibility = @GlobalGroupId
	UPDATE gemini_issuecomments SET visibility=@ID WHERE visibilitymembertype = 2 AND visibility = @GlobalGroupId
	UPDATE gemini_admappings SET globalgroupid = @ID WHERE globalgroupid = @GlobalGroupId
	
	FETCH NEXT FROM groups_cursor INTO @GlobalGroupId, @GlobalName, @GlobalDesc
END
CLOSE groups_cursor 
DEALLOCATE groups_cursor 

UPDATE gemini_customfielddefinitions SET listlimiter= REPLACE(REPLACE(listlimiter, 'P_', ''),'G_','')
COMMIT TRANSACTION

ALTER TABLE gemini_admappings ADD projectgroupid NUMERIC (10,0) NULL CONSTRAINT gemini_admappings_projectgroups_fk FOREIGN KEY REFERENCES gemini_projectgroups(projectgroupid)
GO
UPDATE gemini_admappings SET projectgroupid = globalgroupid
GO
IF EXISTS(SELECT name FROM sysindexes WHERE name COLLATE DATABASE_DEFAULT = 'ind_gemini_admappings_globalgroupid' COLLATE DATABASE_DEFAULT) DROP INDEX ind_gemini_admappings_globalgroupid ON gemini_admappings
GO
ALTER TABLE gemini_admappings DROP COLUMN globalgroupid
GO
ALTER TABLE gemini_admappings ALTER COLUMN projectgroupid NUMERIC (10,0) NOT NULL 
GO
CREATE INDEX ind_gemini_admappings_projectgroupid ON gemini_admappings(projectgroupid)
GO

/**********************************************************************************************************************
* End
**********************************************************************************************************************/

/**********************************************************************************************************************
* Data Changes
**********************************************************************************************************************/
UPDATE i SET i.loggedhours = b.hours, i.loggedminutes = b.minutes FROM gemini_issues i JOIN
(
	SELECT issueid, SUM(hours) AS hours, SUM(minutes) AS minutes FROM gemini_timetracking GROUP BY issueid
) b on i.issueid=b.issueid
GO

UPDATE gemini_issues SET mailboxid = popmailboxid
ALTER TABLE gemini_issues DROP COLUMN popmailboxid 
GO

UPDATE i SET i.votes = v.CNT FROM gemini_issues i JOIN (SELECT issueid,COUNT(*) as CNT FROM gemini_issuevotes GROUP BY issueid) v ON i.issueid=v.issueid
GO

UPDATE gemini_issues SET gemini_issues.components = ISNULL('|' + (SELECT CAST(componentid AS VARCHAR(10))+ '|' FROM gemini_issuecomponents WHERE gemini_issuecomponents.issueid = gemini_issues.issueid FOR XML PATH('')), '')
UPDATE gemini_issues SET gemini_issues.affectedversions = ISNULL('|' + (SELECT CAST(versionid AS VARCHAR(10))+ '|' FROM gemini_affectedversions WHERE gemini_affectedversions.issueid = gemini_issues.issueid FOR XML PATH('')), '')
UPDATE gemini_issues SET gemini_issues.resources = ISNULL('|' + (SELECT CAST(userid AS VARCHAR(10))+ '|' FROM gemini_issueresources WHERE gemini_issueresources.issueid = gemini_issues.issueid FOR XML PATH('')), '')
UPDATE gemini_issues SET gemini_issues.watchers = ISNULL('|' + (SELECT CAST(userid AS VARCHAR(10))+ '|' FROM gemini_watchissues WHERE gemini_watchissues.issueid = gemini_issues.issueid FOR XML PATH('')), '')
GO

UPDATE gemini_testing_run_cases SET caseorder = (SELECT testcaseorder FROM gemini_testing_plan_cases a WHERE a.testcaseid = gemini_testing_run_cases.testcaseid and a.testplanid = gemini_testing_run_cases.testplanid)
GO	

UPDATE gemini_userissuesviews SET issueattribute = NULL where customfieldid IS NOT NULL
GO
DELETE FROM gemini_userissuesviews where issueattribute IN (19, 20)
DELETE FROM gemini_userissuesviews where issueattribute IN (28)
DELETE FROM gemini_userissuesviews where issueattribute IN (36, 37, 38)
GO
UPDATE gemini_userissuesviews SET issueattribute = 1 where issueattribute = 2
UPDATE gemini_userissuesviews SET issueattribute = 26, appid='1' where issueattribute IN (3,4) and appid IS NULL
UPDATE gemini_userissuesviews SET issueattribute = 3, appid='1'  where issueattribute IN (5) and appid IS NULL
UPDATE gemini_userissuesviews SET issueattribute = 2, appid='1'  where issueattribute IN (6) and appid IS NULL
UPDATE gemini_userissuesviews SET issueattribute = 6, appid='1'  where issueattribute IN (7, 8) and appid IS NULL
UPDATE gemini_userissuesviews SET issueattribute = 7, appid='1'  where issueattribute IN (9, 10) and appid IS NULL
UPDATE gemini_userissuesviews SET issueattribute = 8, appid='1'  where issueattribute IN (11, 12) and appid IS NULL
UPDATE gemini_userissuesviews SET issueattribute = 9, appid='1'  where issueattribute IN (13, 14, 46) and appid IS NULL
UPDATE gemini_userissuesviews SET issueattribute = 10, appid='1'  where issueattribute IN (15, 16, 62) and appid IS NULL
UPDATE gemini_userissuesviews SET issueattribute = 11, appid='1'  where issueattribute IN (17, 18) and appid IS NULL
UPDATE gemini_userissuesviews SET issueattribute = 12, appid='1'  where issueattribute IN (21, 22, 23, 24) and appid IS NULL
UPDATE gemini_userissuesviews SET issueattribute = 13, appid='1'  where issueattribute IN (25, 26, 27) and appid IS NULL
UPDATE gemini_userissuesviews SET issueattribute = 14, appid='1'  where issueattribute IN (29, 66) and appid IS NULL
UPDATE gemini_userissuesviews SET issueattribute = 15, appid='1'  where issueattribute IN (30, 31, 32) and appid IS NULL
UPDATE gemini_userissuesviews SET issueattribute = 30, appid='1'  where issueattribute IN (33, 34, 35) and appid IS NULL
UPDATE gemini_userissuesviews SET issueattribute = 16, appid='1'  where issueattribute IN (39) and appid IS NULL
UPDATE gemini_userissuesviews SET issueattribute = 17, appid='1'  where issueattribute IN (40) and appid IS NULL
UPDATE gemini_userissuesviews SET issueattribute = 18, appid='1'  where issueattribute IN (41) and appid IS NULL
UPDATE gemini_userissuesviews SET issueattribute = 19, appid='1'  where issueattribute IN (42) and appid IS NULL
UPDATE gemini_userissuesviews SET issueattribute = 20, appid='1'  where issueattribute IN (43) and appid IS NULL
UPDATE gemini_userissuesviews SET issueattribute = 21, appid='1'  where issueattribute IN (44) and appid IS NULL
UPDATE gemini_userissuesviews SET issueattribute = 22, appid='1'  where issueattribute IN (45) and appid IS NULL
UPDATE gemini_userissuesviews SET issueattribute = 23, appid='1'  where issueattribute IN (47, 48, 49, 50) and appid IS NULL           
UPDATE gemini_userissuesviews SET issueattribute = 7, appid='1'  where issueattribute IN (51, 52, 53) and appid IS NULL           
UPDATE gemini_userissuesviews SET issueattribute = 13, appid='1'  where issueattribute IN (54, 55, 56) and appid IS NULL           
UPDATE gemini_userissuesviews SET issueattribute = 5, appid='1'  where issueattribute IN (57) and appid IS NULL                       
UPDATE gemini_userissuesviews SET issueattribute = 15, appid='1'  where issueattribute IN (58) and appid IS NULL                                   
UPDATE gemini_userissuesviews SET issueattribute = 4, appid='1'  where issueattribute IN (59, 60, 61, 62) and appid IS NULL
UPDATE gemini_userissuesviews SET issueattribute = 29, appid='1'  where issueattribute IN (63) and appid IS NULL
UPDATE gemini_userissuesviews SET issueattribute = 27, appid='1'  where issueattribute IN (64) and appid IS NULL
UPDATE gemini_userissuesviews SET issueattribute = 28, appid='1'  where issueattribute IN (65) and appid IS NULL            
UPDATE gemini_userissuesviews SET issueattribute = 31, appid='1'  where issueattribute IN (67) and appid IS NULL            
UPDATE gemini_userissuesviews SET issueattribute = 32, appid='1'  where issueattribute IN (68) and appid IS NULL            
UPDATE gemini_userissuesviews SET issueattribute = 36, appid='1'  where issueattribute IN (69) and appid IS NULL            
UPDATE gemini_userissuesviews SET issueattribute = 38, appid='1'  where issueattribute IN (70) and appid IS NULL
UPDATE gemini_userissuesviews SET issueattribute = 35, appid='1'  where issueattribute IN (71) and appid IS NULL
UPDATE gemini_userissuesviews SET issueattribute = 39, appid='1'  where issueattribute IN (72) and appid IS NULL
UPDATE gemini_userissuesviews SET issueattribute = 37, appid='1'  where issueattribute IN (73) and appid IS NULL
UPDATE gemini_userissuesviews SET issueattribute = 30, appid='1'  where issueattribute IN (74) and appid IS NULL
UPDATE gemini_userissuesviews SET issueattribute = 41, appid='1'  where issueattribute IN (75) and appid IS NULL
UPDATE gemini_userissuesviews SET issueattribute = 24, appid='1'  where issueattribute IN (76) and appid IS NULL
UPDATE gemini_userissuesviews SET issueattribute = 25, appid='1'  where issueattribute IN (77) and appid IS NULL
UPDATE gemini_userissuesviews SET issueattribute = 25, appid='1'  where issueattribute IN (77) and appid IS NULL
UPDATE gemini_userissuesviews SET viewtype = 1, appid='1' WHERE viewtype = 0 
UPDATE gemini_userissuesviews SET viewtype = 17, appid='1' WHERE viewtype = 1
UPDATE gemini_userissuesviews SET viewtype = 18, appid='2' WHERE viewtype = 2
UPDATE gemini_userissuesviews SET viewtype = 4, appid='2' WHERE viewtype = 3 
UPDATE gemini_userissuesviews SET viewtype = 3, appid='2' WHERE viewtype = 4 
GO
UPDATE gemini_userissuesviews SET appid = NULL
GO
DELETE a FROM gemini_userissuesviews a join 
(SELECT MAX(itemid) maxid, userid,projectid, issueattribute, customfieldid, viewtype from gemini_userissuesviews group by userid,projectid, issueattribute, customfieldid, viewtype
HAVING COUNT(*)>1) b
ON a.userid = b.userid AND a.projectid = b.projectid AND a. issueattribute = b.issueattribute AND (a. customfieldid = b.customfieldid  
OR (a.customfieldid IS NULL and b.customfieldid IS NULL))
AND a.viewtype = b.viewtype
WHERE a.itemid != maxid
GO

-- Default Values
DELETE FROM gemini_projectdefaultvalues
GO

ALTER TABLE gemini_projectdefaultvalues DROP CONSTRAINT gemini_projectdefaultvalues_defaultvalue_def
GO
ALTER TABLE gemini_projectdefaultvalues DROP COLUMN fieldid
GO
ALTER TABLE gemini_projectdefaultvalues DROP COLUMN defaultvalue
GO

-- Custom Fields
UPDATE gemini_customfielddata set revised=created
GO
UPDATE gemini_customfielddefinitions SET usestaticdata = 1 WHERE (lookupname IS NULL OR lookupname COLLATE DATABASE_DEFAULT = '' COLLATE DATABASE_DEFAULT) AND lookupdata IS NOT NULL
GO

--UTC
UPDATE gemini_issues SET 
	created = DATEADD(MI, DATEDIFF(mi, GETDATE(), GETUTCDATE()), created), 
	revised = DATEADD(MI, DATEDIFF(mi, GETDATE(), GETUTCDATE()), revised),
	closeddate = DATEADD(MI, DATEDIFF(mi, GETDATE(), GETUTCDATE()), closeddate),
	resolveddate = DATEADD(MI, DATEDIFF(mi, GETDATE(), GETUTCDATE()), resolveddate),
	originatorprocessed = DATEADD(MI, DATEDIFF(mi, GETDATE(), GETUTCDATE()), originatorprocessed)
	
UPDATE gemini_admappings SET lastsyncdate = DATEADD(MI, DATEDIFF(mi, GETDATE(), GETUTCDATE()), lastsyncdate)
UPDATE gemini_users SET lastupdated = DATEADD(MI, DATEDIFF(mi, GETDATE(), GETUTCDATE()), lastupdated)
UPDATE gemini_users SET logindate = DATEADD(MI, DATEDIFF(mi, GETDATE(), GETUTCDATE()), logindate)
UPDATE gemini_users SET previouslogindate = DATEADD(MI, DATEDIFF(mi, GETDATE(), GETUTCDATE()), previouslogindate)
UPDATE gemini_testing_customdata SET created = DATEADD(MI, DATEDIFF(mi, GETDATE(), GETUTCDATE()), created)
UPDATE gemini_issuecomponents SET created = DATEADD(MI, DATEDIFF(mi, GETDATE(), GETUTCDATE()), created)
UPDATE gemini_issuelinks SET created = DATEADD(MI, DATEDIFF(mi, GETDATE(), GETUTCDATE()), created)
UPDATE gemini_projects SET created = DATEADD(MI, DATEDIFF(mi, GETDATE(), GETUTCDATE()), created)
UPDATE gemini_globalgroups SET created = DATEADD(MI, DATEDIFF(mi, GETDATE(), GETUTCDATE()), created)
UPDATE gemini_systemlog SET created = DATEADD(MI, DATEDIFF(mi, GETDATE(), GETUTCDATE()), created)
UPDATE gemini_customfielddefinitions SET created = DATEADD(MI, DATEDIFF(mi, GETDATE(), GETUTCDATE()), created)
UPDATE gemini_testing_history SET created = DATEADD(MI, DATEDIFF(mi, GETDATE(), GETUTCDATE()), created)
UPDATE gemini_projectgroups SET created = DATEADD(MI, DATEDIFF(mi, GETDATE(), GETUTCDATE()), created)
UPDATE gemini_projectdocuments SET created = DATEADD(MI, DATEDIFF(mi, GETDATE(), GETUTCDATE()), created)
UPDATE gemini_projectgroupmembership SET created = DATEADD(MI, DATEDIFF(mi, GETDATE(), GETUTCDATE()), created)
UPDATE gemini_appointments SET created = DATEADD(MI, DATEDIFF(mi, GETDATE(), GETUTCDATE()), created)
UPDATE gemini_versions SET created = DATEADD(MI, DATEDIFF(mi, GETDATE(), GETUTCDATE()), created)
UPDATE gemini_customfieldusage SET created = DATEADD(MI, DATEDIFF(mi, GETDATE(), GETUTCDATE()), created)
UPDATE gemini_customfielddata SET created = DATEADD(MI, DATEDIFF(mi, GETDATE(), GETUTCDATE()), created)
UPDATE gemini_components SET created = DATEADD(MI, DATEDIFF(mi, GETDATE(), GETUTCDATE()), created)
UPDATE gemini_projectdefaultvalues SET created = DATEADD(MI, DATEDIFF(mi, GETDATE(), GETUTCDATE()), created)
UPDATE gemini_testing_plans SET created = DATEADD(MI, DATEDIFF(mi, GETDATE(), GETUTCDATE()), created)
UPDATE gemini_testing_plans SET revised = DATEADD(MI, DATEDIFF(mi, GETDATE(), GETUTCDATE()), revised)
UPDATE gemini_testing_cases SET created = DATEADD(MI, DATEDIFF(mi, GETDATE(), GETUTCDATE()), created)
UPDATE gemini_testing_cases SET revised = DATEADD(MI, DATEDIFF(mi, GETDATE(), GETUTCDATE()), revised)
UPDATE gemini_globalsecurityschemes SET created = DATEADD(MI, DATEDIFF(mi, GETDATE(), GETUTCDATE()), created)
UPDATE gemini_globalsecurityschemeroles SET created = DATEADD(MI, DATEDIFF(mi, GETDATE(), GETUTCDATE()), created)
UPDATE gemini_users SET created = DATEADD(MI, DATEDIFF(mi, GETDATE(), GETUTCDATE()), created)
UPDATE gemini_testing_plan_cases SET created = DATEADD(MI, DATEDIFF(mi, GETDATE(), GETUTCDATE()), created)
UPDATE gemini_testing_plan_cases SET revised = DATEADD(MI, DATEDIFF(mi, GETDATE(), GETUTCDATE()), revised)
UPDATE gemini_adgroups SET created = DATEADD(MI, DATEDIFF(mi, GETDATE(), GETUTCDATE()), created)
UPDATE gemini_admappings SET created = DATEADD(MI, DATEDIFF(mi, GETDATE(), GETUTCDATE()), created)
UPDATE gemini_testing_case_issues SET created = DATEADD(MI, DATEDIFF(mi, GETDATE(), GETUTCDATE()), created)
UPDATE gemini_testing_case_steps SET created = DATEADD(MI, DATEDIFF(mi, GETDATE(), GETUTCDATE()), created)
UPDATE gemini_testing_case_steps SET revised = DATEADD(MI, DATEDIFF(mi, GETDATE(), GETUTCDATE()), revised)
/*UPDATE gemini_timetracking SET created = DATEADD(MI, DATEDIFF(mi, GETDATE(), GETUTCDATE()), created)
UPDATE gemini_timetracking SET timeentrydate = DATEADD(MI, DATEDIFF(mi, GETDATE(), GETUTCDATE()), timeentrydate)*/
UPDATE gemini_issuecomments SET created = DATEADD(MI, DATEDIFF(mi, GETDATE(), GETUTCDATE()), created)
UPDATE gemini_testing_runs SET created = DATEADD(MI, DATEDIFF(mi, GETDATE(), GETUTCDATE()), created)
UPDATE gemini_testing_runs SET revised = DATEADD(MI, DATEDIFF(mi, GETDATE(), GETUTCDATE()), revised)
UPDATE gemini_projectlabels SET created = DATEADD(MI, DATEDIFF(mi, GETDATE(), GETUTCDATE()), created)
UPDATE gemini_issuehistory SET created = DATEADD(MI, DATEDIFF(mi, GETDATE(), GETUTCDATE()), created)
UPDATE gemini_testing_run_cases SET created = DATEADD(MI, DATEDIFF(mi, GETDATE(), GETUTCDATE()), created)
UPDATE gemini_versionmilestone SET created = DATEADD(MI, DATEDIFF(mi, GETDATE(), GETUTCDATE()), created)
UPDATE gemini_issueattachments SET created = DATEADD(MI, DATEDIFF(mi, GETDATE(), GETUTCDATE()), created)
UPDATE gemini_testing_run_steps SET created = DATEADD(MI, DATEDIFF(mi, GETDATE(), GETUTCDATE()), created)
UPDATE gemini_testing_attachments SET created = DATEADD(MI, DATEDIFF(mi, GETDATE(), GETUTCDATE()), created)
UPDATE gemini_issuetimetype SET created = DATEADD(MI, DATEDIFF(mi, GETDATE(), GETUTCDATE()), created)
UPDATE gemini_watchissues SET created = DATEADD(MI, DATEDIFF(mi, GETDATE(), GETUTCDATE()), created)
UPDATE gemini_testing_run_times SET created = DATEADD(MI, DATEDIFF(mi, GETDATE(), GETUTCDATE()), created)
UPDATE gemini_affectedversions SET created = DATEADD(MI, DATEDIFF(mi, GETDATE(), GETUTCDATE()), created)
UPDATE gemini_issueresources SET created = DATEADD(MI, DATEDIFF(mi, GETDATE(), GETUTCDATE()), created)
UPDATE gemini_issuelinktypes SET created = DATEADD(MI, DATEDIFF(mi, GETDATE(), GETUTCDATE()), created)
GO

ALTER TABLE gemini_install DROP CONSTRAINT gemini_install_created_def  ALTER TABLE gemini_install ADD CONSTRAINT gemini_install_created_def DEFAULT (getutcdate()) FOR created
ALTER TABLE gemini_projectlabels DROP CONSTRAINT gemini_projectlabels_created_def  ALTER TABLE gemini_projectlabels ADD CONSTRAINT gemini_projectlabels_created_def DEFAULT (getutcdate()) FOR created
ALTER TABLE gemini_testing_case_issues DROP CONSTRAINT gemini_testing_case_issues_def  ALTER TABLE gemini_testing_case_issues ADD CONSTRAINT gemini_testing_case_issues_def DEFAULT (getutcdate()) FOR created
ALTER TABLE gemini_components DROP CONSTRAINT gemini_components_created_def  ALTER TABLE gemini_components ADD CONSTRAINT gemini_components_created_def DEFAULT (getutcdate()) FOR created
ALTER TABLE gemini_issuelinks DROP CONSTRAINT gemini_issuelinks_created_def  ALTER TABLE gemini_issuelinks ADD CONSTRAINT gemini_issuelinks_created_def DEFAULT (getutcdate()) FOR created
ALTER TABLE gemini_converse_mailboxprocessor_message DROP CONSTRAINT gemini_converse_mailboxprocessor_message_created_def  ALTER TABLE gemini_converse_mailboxprocessor_message ADD CONSTRAINT gemini_converse_mailboxprocessor_message_created_def DEFAULT (getutcdate()) FOR created
ALTER TABLE gemini_testing_customdata DROP CONSTRAINT gemini_testing_customdata_created_def  ALTER TABLE gemini_testing_customdata ADD CONSTRAINT gemini_testing_customdata_created_def DEFAULT (getutcdate()) FOR created
ALTER TABLE gemini_issueattachments DROP CONSTRAINT gemini_issueattachments_created_def  ALTER TABLE gemini_issueattachments ADD CONSTRAINT gemini_issueattachments_created_def DEFAULT (getutcdate()) FOR created
ALTER TABLE gemini_converse_mailboxprocessor_string DROP CONSTRAINT gemini_converse_mailboxprocessor_string_created_def  ALTER TABLE gemini_converse_mailboxprocessor_string ADD CONSTRAINT gemini_converse_mailboxprocessor_string_created_def DEFAULT (getutcdate()) FOR created
ALTER TABLE gemini_adgroups DROP CONSTRAINT gemini_adgroups_created_def  ALTER TABLE gemini_adgroups ADD CONSTRAINT gemini_adgroups_created_def DEFAULT (getutcdate()) FOR created
ALTER TABLE gemini_testing_attachments DROP CONSTRAINT gemini_testing_attachments_created_def  ALTER TABLE gemini_testing_attachments ADD CONSTRAINT gemini_testing_attachments_created_def DEFAULT (getutcdate()) FOR created
ALTER TABLE gemini_globalsecurityschemeroles DROP CONSTRAINT gemini_globalsecurityschemeroles_created_def  ALTER TABLE gemini_globalsecurityschemeroles ADD CONSTRAINT gemini_globalsecurityschemeroles_created_def DEFAULT (getutcdate()) FOR created
ALTER TABLE gemini_admappings DROP CONSTRAINT gemini_admappings_created_def  ALTER TABLE gemini_admappings ADD CONSTRAINT gemini_admappings_created_def DEFAULT (getutcdate()) FOR created
ALTER TABLE gemini_appointments DROP CONSTRAINT gemini_appointments_created_def  ALTER TABLE gemini_appointments ADD CONSTRAINT gemini_appointments_created_def DEFAULT (getutcdate()) FOR created
ALTER TABLE gemini_testing_cases DROP CONSTRAINT gemini_testing_cases_created_def  ALTER TABLE gemini_testing_cases ADD CONSTRAINT gemini_testing_cases_created_def DEFAULT (getutcdate()) FOR created
ALTER TABLE gemini_affectedversions DROP CONSTRAINT gemini_affectedversions_created_def  ALTER TABLE gemini_affectedversions ADD CONSTRAINT gemini_affectedversions_created_def DEFAULT (getutcdate()) FOR created
ALTER TABLE gemini_projectgroups DROP CONSTRAINT gemini_projectgroups_created_def  ALTER TABLE gemini_projectgroups ADD CONSTRAINT gemini_projectgroups_created_def DEFAULT (getutcdate()) FOR created
ALTER TABLE gemini_watchissues DROP CONSTRAINT gemini_watchissues_created_def  ALTER TABLE gemini_watchissues ADD CONSTRAINT gemini_watchissues_created_def DEFAULT (getutcdate()) FOR created
ALTER TABLE gemini_issuevotes DROP CONSTRAINT gemini_issuevotes_created_def  ALTER TABLE gemini_issuevotes ADD CONSTRAINT gemini_issuevotes_created_def DEFAULT (getutcdate()) FOR created
ALTER TABLE gemini_customfielddata DROP CONSTRAINT gemini_customfielddata_created_def  ALTER TABLE gemini_customfielddata ADD CONSTRAINT gemini_customfielddata_created_def DEFAULT (getutcdate()) FOR created
ALTER TABLE gemini_testing_run_steps DROP CONSTRAINT gemini_testing_run_steps_created_def  ALTER TABLE gemini_testing_run_steps ADD CONSTRAINT gemini_testing_run_steps_created_def DEFAULT (getutcdate()) FOR created
ALTER TABLE gemini_systemlog DROP CONSTRAINT gemini_systemlog_created_def  ALTER TABLE gemini_systemlog ADD CONSTRAINT gemini_systemlog_created_def DEFAULT (getutcdate()) FOR created
ALTER TABLE gemini_testing_run_cases DROP CONSTRAINT gemini_testing_run_cases_created_def  ALTER TABLE gemini_testing_run_cases ADD CONSTRAINT gemini_testing_run_cases_created_def DEFAULT (getutcdate()) FOR created
ALTER TABLE gemini_users DROP CONSTRAINT gemini_users_created_def  ALTER TABLE gemini_users ADD CONSTRAINT gemini_users_created_def DEFAULT (getutcdate()) FOR created
ALTER TABLE gemini_timetracking DROP CONSTRAINT gemini_timetracking_created_def  ALTER TABLE gemini_timetracking ADD CONSTRAINT gemini_timetracking_created_def DEFAULT (getutcdate()) FOR created
ALTER TABLE gemini_testing_history DROP CONSTRAINT gemini_testing_history_created_def  ALTER TABLE gemini_testing_history ADD CONSTRAINT gemini_testing_history_created_def DEFAULT (getutcdate()) FOR created
ALTER TABLE gemini_projectgroupmembership DROP CONSTRAINT gemini_projectgroupmembership_created_def  ALTER TABLE gemini_projectgroupmembership ADD CONSTRAINT gemini_projectgroupmembership_created_def DEFAULT (getutcdate()) FOR created
ALTER TABLE gemini_testing_runs DROP CONSTRAINT gemini_testing_runs_created_def  ALTER TABLE gemini_testing_runs ADD CONSTRAINT gemini_testing_runs_created_def DEFAULT (getutcdate()) FOR created
ALTER TABLE gemini_issueresources DROP CONSTRAINT gemini_issueresources_created_def  ALTER TABLE gemini_issueresources ADD CONSTRAINT gemini_issueresources_created_def DEFAULT (getutcdate()) FOR created
ALTER TABLE gemini_versions DROP CONSTRAINT gemini_versions_created_def  ALTER TABLE gemini_versions ADD CONSTRAINT gemini_versions_created_def DEFAULT (getutcdate()) FOR created
ALTER TABLE gemini_globalsecurityschemes DROP CONSTRAINT gemini_globalsecurityschemes_created_def  ALTER TABLE gemini_globalsecurityschemes ADD CONSTRAINT gemini_globalsecurityschemes_created_def DEFAULT (getutcdate()) FOR created
ALTER TABLE gemini_projectdocuments DROP CONSTRAINT ggemini_projectdocuments_created_def  ALTER TABLE gemini_projectdocuments ADD CONSTRAINT ggemini_projectdocuments_created_def DEFAULT (getutcdate()) FOR created
ALTER TABLE gemini_testing_run_times DROP CONSTRAINT gemini_testing_run_times_created_def  ALTER TABLE gemini_testing_run_times ADD CONSTRAINT gemini_testing_run_times_created_def DEFAULT (getutcdate()) FOR created
ALTER TABLE gemini_versionmilestone DROP CONSTRAINT gemini_versionmilestone_created_def  ALTER TABLE gemini_versionmilestone ADD CONSTRAINT gemini_versionmilestone_created_def DEFAULT (getutcdate()) FOR created
ALTER TABLE gemini_customfielddefinitions DROP CONSTRAINT gemini_customfielddefinitions_created_def  ALTER TABLE gemini_customfielddefinitions ADD CONSTRAINT gemini_customfielddefinitions_created_def DEFAULT (getutcdate()) FOR created
ALTER TABLE gemini_issuelinktypes DROP CONSTRAINT gemini_issuelinktypes_created_def  ALTER TABLE gemini_issuelinktypes ADD CONSTRAINT gemini_issuelinktypes_created_def DEFAULT (getutcdate()) FOR created
ALTER TABLE gemini_projectdefaultvalues DROP CONSTRAINT gemini_projectdefaultvalues_created_def  ALTER TABLE gemini_projectdefaultvalues ADD CONSTRAINT gemini_projectdefaultvalues_created_def DEFAULT (getutcdate()) FOR created
ALTER TABLE gemini_projects DROP CONSTRAINT gemini_projects_creared_def  ALTER TABLE gemini_projects ADD CONSTRAINT gemini_projects_creared_def DEFAULT (getutcdate()) FOR created
ALTER TABLE gemini_testing_plans DROP CONSTRAINT gemini_testing_plans_created_def  ALTER TABLE gemini_testing_plans ADD CONSTRAINT gemini_testing_plans_created_def DEFAULT (getutcdate()) FOR created
ALTER TABLE gemini_issuecomments DROP CONSTRAINT gemini_issuecomments_created_def  ALTER TABLE gemini_issuecomments ADD CONSTRAINT gemini_issuecomments_created_def DEFAULT (getutcdate()) FOR created
ALTER TABLE gemini_issuetimetype DROP CONSTRAINT gemini_issuetimetype_created_def  ALTER TABLE gemini_issuetimetype ADD CONSTRAINT gemini_issuetimetype_created_def DEFAULT (getutcdate()) FOR created
ALTER TABLE gemini_testing_case_steps DROP CONSTRAINT gemini_testing_case_steps_created_def  ALTER TABLE gemini_testing_case_steps ADD CONSTRAINT gemini_testing_case_steps_created_def DEFAULT (getutcdate()) FOR created
ALTER TABLE gemini_testing_plan_cases DROP CONSTRAINT gemini_testing_plan_cases_created_def  ALTER TABLE gemini_testing_plan_cases ADD CONSTRAINT gemini_testing_plan_cases_created_def DEFAULT (getutcdate()) FOR created
ALTER TABLE gemini_issuecomponents DROP CONSTRAINT gemini_issuecomponents_created_def  ALTER TABLE gemini_issuecomponents ADD CONSTRAINT gemini_issuecomponents_created_def DEFAULT (getutcdate()) FOR created
ALTER TABLE gemini_users DROP CONSTRAINT gemini_users_lastupdated_def  ALTER TABLE gemini_users ADD CONSTRAINT gemini_users_lastupdated_def DEFAULT (getutcdate()) FOR lastupdated
ALTER TABLE gemini_users DROP CONSTRAINT gemini_users_logindate_def  ALTER TABLE gemini_users ADD CONSTRAINT gemini_users_logindate_def DEFAULT (getutcdate()) FOR logindate
ALTER TABLE gemini_versionmilestone DROP CONSTRAINT gemini_versionmilestone_milestonedate_def  ALTER TABLE gemini_versionmilestone ADD CONSTRAINT gemini_versionmilestone_milestonedate_def DEFAULT (getutcdate()) FOR milestonedate
ALTER TABLE gemini_users DROP CONSTRAINT gemini_users_previouslogindate_def  ALTER TABLE gemini_users ADD CONSTRAINT gemini_users_previouslogindate_def DEFAULT (getutcdate()) FOR previouslogindate
ALTER TABLE gemini_versions DROP CONSTRAINT gemini_versions_releasedate_def  ALTER TABLE gemini_versions ADD CONSTRAINT gemini_versions_releasedate_def DEFAULT (getutcdate()) FOR releasedate
ALTER TABLE gemini_testing_cases DROP CONSTRAINT gemini_testing_cases_revised_def  ALTER TABLE gemini_testing_cases ADD CONSTRAINT gemini_testing_cases_revised_def DEFAULT (getutcdate()) FOR revised
ALTER TABLE gemini_testing_runs DROP CONSTRAINT gemini_testing_runs_revised_def  ALTER TABLE gemini_testing_runs ADD CONSTRAINT gemini_testing_runs_revised_def DEFAULT (getutcdate()) FOR revised
ALTER TABLE gemini_testing_plans DROP CONSTRAINT gemini_testing_plans_revised_def  ALTER TABLE gemini_testing_plans ADD CONSTRAINT gemini_testing_plans_revised_def DEFAULT (getutcdate()) FOR revised
ALTER TABLE gemini_testing_case_steps DROP CONSTRAINT gemini_testing_case_steps_revised_def  ALTER TABLE gemini_testing_case_steps ADD CONSTRAINT gemini_testing_case_steps_revised_def DEFAULT (getutcdate()) FOR revised
ALTER TABLE gemini_testing_plan_cases DROP CONSTRAINT gemini_testing_plan_cases_revised_def  ALTER TABLE gemini_testing_plan_cases ADD CONSTRAINT gemini_testing_plan_cases_revised_def DEFAULT (getutcdate()) FOR revised
ALTER TABLE gemini_versions DROP CONSTRAINT gemini_versions_startdate_def  ALTER TABLE gemini_versions ADD CONSTRAINT gemini_versions_startdate_def DEFAULT (getutcdate()) FOR startdate
GO
ALTER TABLE gemini_issues DROP CONSTRAINT gemini_issues_created_def
GO
ALTER TABLE gemini_issues ADD CONSTRAINT gemini_issues_created_def DEFAULT (getutcdate()) FOR created
GO
ALTER TABLE gemini_issues DROP CONSTRAINT gemini_issues_revised_def
GO
ALTER TABLE gemini_issues ADD CONSTRAINT gemini_issues_revised_def DEFAULT (getutcdate()) FOR revised
GO

/**** IMAGES ****/
UPDATE gemini_testing_types SET imagepath='assets/images/meta/'+lower(REPLACE(imagepath,'images/','')) 
UPDATE gemini_issuetypes SET imagepath='assets/images/meta/'+lower(REPLACE(REPLACE(imagepath, '_','-'),'images/','')) WHERE templateid = 1 AND imagepath IN (
'images/Type_Bug.png', 
'images/Type_ChangeRequest.png',
'images/Type_Enhancement.png',
'images/Type_Investigation.png', 
'images/Type_NewFeature.png', 
'images/Type_QA.png', 
'images/Type_Requirement.png',
'images/Type_Subtask.png',
'images/Type_Support.png',
'images/Type_Task.png'
)

UPDATE gemini_issuetypes SET imagepath='assets/images/meta/type-qualitycheck.png' WHERE templateid=1 AND imagepath LIKE '%Type_QA.png%'

UPDATE gemini_issuepriorities SET imagepath='assets/images/meta/'+lower(REPLACE(REPLACE(imagepath, '_','-'),'images/','')) WHERE templateid = 1 AND imagepath IN (
'images/Priority_Low.png', 
'images/Priority_Medium.png',
'images/Priority_High.png'
)

UPDATE gemini_issuestatus SET imagepath='assets/images/meta/'+lower(REPLACE(REPLACE(REPLACE(imagepath, '_','-'),'images/',''),'issues','s')) WHERE templateid = 1 AND imagepath IN (
'images/IssueStatus_Unassigned.png', 
'images/IssueStatus_assigned.png',
'images/IssueStatus_InProgress.png',
'images/IssueStatus_closed.png',
'images/IssueStatus_Reopened.png',
'images/IssueStatus_Check.png',
'images/IssueStatus.png'
)

UPDATE gemini_issueseverity SET imagepath='assets/images/meta/'+lower(REPLACE(REPLACE(imagepath, '_','-'),'images/','')) WHERE templateid = 1 AND imagepath IN (
'images/Severity_Trivial.png', 
'images/Severity_Minor.png',
'images/Severity_Major.png',
'images/Severity_ShowStopper.png'
)
GO

DELETE FROM gemini_applicationsettings where settingname='EnabledEmailAlertTypes' AND settingid=(SELECT MIN(settingid) FROM gemini_applicationsettings where settingname COLLATE DATABASE_DEFAULT ='EnabledEmailAlertTypes' COLLATE DATABASE_DEFAULT )
GO

-- Move history
TRUNCATE TABLE gemini_projectaudit
TRUNCATE TABLE gemini_issueaudit
GO

-- PROJECT AUDIT
INSERT INTO gemini_projectaudit(projectarea,projectid, touched, useraction, userid, username, created)
SELECT  13, projectid, a.issueid, 1, NULL, a.username, created  FROM gemini_issuehistory a where history COLLATE DATABASE_DEFAULT ='created' COLLATE DATABASE_DEFAULT order by created

INSERT INTO gemini_projectaudit(projectarea,projectid, touched, useraction, userid, username, created)
SELECT  13, projectid, a.issueid , 5, NULL, a.username, created  FROM gemini_issuehistory a where history COLLATE DATABASE_DEFAULT ='issue moved' COLLATE DATABASE_DEFAULT order by created

INSERT INTO gemini_projectaudit(projectarea,projectid, touched, useraction, userid, username, created)
SELECT  13, projectid, a.issueid , 6, NULL, a.username, created  FROM gemini_issuehistory a where history COLLATE DATABASE_DEFAULT ='issue copied' COLLATE DATABASE_DEFAULT order by created

-- Update user id
UPDATE gemini_projectaudit SET userid= (SELECT TOP 1 userid FROM gemini_users WHERE firstname + ' ' + surname COLLATE DATABASE_DEFAULT = gemini_projectaudit.username COLLATE DATABASE_DEFAULT)

-- ISSUE AUDIT
INSERT INTO gemini_issueaudit(projectid, issueid, userid, username, attributechanged, fieldchanged, iscustom, idafter, idbefore, valueafter, valuebefore, created)
SELECT projectid, issueid, NULL, username, 13, 'Component', 0, REPLACE(history, 'Component changed to ', ''), '', REPLACE(history, 'Component changed to ', ''), '', created FROM gemini_issuehistory WHERE history LIKE 'Component changed to %' order by created
UPDATE a SET a.idafter= b.componentid FROM gemini_issueaudit a JOIN gemini_components b ON a.idafter COLLATE DATABASE_DEFAULT = b.componentname COLLATE DATABASE_DEFAULT AND a.projectid = b.projectid WHERE fieldchanged COLLATE DATABASE_DEFAULT ='Component' COLLATE DATABASE_DEFAULT 
GO

INSERT INTO gemini_issueaudit(projectid, issueid, userid, username, attributechanged, fieldchanged, iscustom, idafter, idbefore, valueafter, valuebefore, created)
SELECT projectid, issueid, NULL, username, 12, 'FixedInVersion', 0, REPLACE(history, 'Fixed version changed to ', ''), '', REPLACE(history, 'Fixed version changed to ', ''), '', created FROM gemini_issuehistory WHERE history LIKE 'Fixed version changed to %' order by created
UPDATE a SET a.idafter= b.versionname FROM gemini_issueaudit a JOIN gemini_versions b ON a.idafter COLLATE DATABASE_DEFAULT = b.versionname COLLATE DATABASE_DEFAULT AND a.projectid = b.projectid WHERE fieldchanged COLLATE DATABASE_DEFAULT ='FixedInVersion' COLLATE DATABASE_DEFAULT 
GO

INSERT INTO gemini_issueaudit(projectid, issueid, userid, username, attributechanged, fieldchanged, iscustom, idafter, idbefore, valueafter, valuebefore, created)
SELECT projectid, issueid, NULL, username, 7, 'AssignedTo', 0, REPLACE(history, 'Assigned resources changed to ', ''), '', REPLACE(history, 'Assigned resources changed to ', ''), '', created FROM gemini_issuehistory WHERE history LIKE 'Assigned resources changed to %' order by created
UPDATE a SET a.idafter= b.userid FROM gemini_issueaudit a JOIN gemini_users b ON a.idafter COLLATE DATABASE_DEFAULT = b.firstname + ' '+ b.surname COLLATE DATABASE_DEFAULT WHERE fieldchanged COLLATE DATABASE_DEFAULT ='AssignedTo' COLLATE DATABASE_DEFAULT 
GO

INSERT INTO gemini_issueaudit(projectid, issueid, userid, username, attributechanged, fieldchanged, iscustom, idafter, idbefore, valueafter, valuebefore, created)
SELECT projectid, issueid, NULL, username, 3, 'Title',0, '', '', REPLACE(history, 'Issue title changed to ', ''), '', created FROM gemini_issuehistory WHERE history LIKE 'Issue title changed to %' order by created
GO

INSERT INTO gemini_issueaudit(projectid, issueid, userid, username, attributechanged, fieldchanged, iscustom, idafter, idbefore, valueafter, valuebefore, created)
SELECT projectid, issueid, NULL, username, 8, 'Type', 0, REPLACE(history, 'Issue type changed to ', ''), '', REPLACE(history, 'Issue type changed to ', ''), '', created FROM gemini_issuehistory WHERE history LIKE 'Issue type changed to %' order by created
UPDATE a SET a.idafter= b.typeid FROM gemini_issueaudit a JOIN gemini_issuetypes b ON a.idafter COLLATE DATABASE_DEFAULT = b.typedesc COLLATE DATABASE_DEFAULT WHERE fieldchanged COLLATE DATABASE_DEFAULT ='Type' COLLATE DATABASE_DEFAULT 
GO

INSERT INTO gemini_issueaudit(projectid, issueid, userid, username, attributechanged, fieldchanged, iscustom, idafter, idbefore, valueafter, valuebefore, created)
SELECT projectid, issueid, NULL, username, 9, 'Priority', 0, REPLACE(history, 'Issue priority changed to ', ''), '', REPLACE(history, 'Issue priority changed to ', ''), '', created FROM gemini_issuehistory WHERE history LIKE 'Issue priority changed to %' order by created
UPDATE a SET a.idafter= b.priorityid FROM gemini_issueaudit a JOIN gemini_issuepriorities b ON a.idafter COLLATE DATABASE_DEFAULT = b.prioritydesc COLLATE DATABASE_DEFAULT WHERE fieldchanged COLLATE DATABASE_DEFAULT ='Priority' COLLATE DATABASE_DEFAULT 
GO

INSERT INTO gemini_issueaudit(projectid, issueid, userid, username, attributechanged, fieldchanged, iscustom, idafter, idbefore, valueafter, valuebefore, created)
SELECT projectid, issueid, NULL, username, 4, 'Severity', 0, REPLACE(history, 'Issue severity changed to ', ''), '', REPLACE(history, 'Issue severity changed to ', ''), '', created FROM gemini_issuehistory WHERE history LIKE 'Issue severity changed to %' order by created
UPDATE a SET a.idafter= b.severityid FROM gemini_issueaudit a JOIN gemini_issueseverity b ON a.idafter COLLATE DATABASE_DEFAULT = b.severitydesc COLLATE DATABASE_DEFAULT WHERE fieldchanged COLLATE DATABASE_DEFAULT ='Severity' COLLATE DATABASE_DEFAULT 
GO

INSERT INTO gemini_issueaudit(projectid, issueid, userid, username, attributechanged, fieldchanged, iscustom, idafter, idbefore, valueafter, valuebefore, created)
SELECT projectid, issueid, NULL, username, 23, 'AffectedVersions', 0, REPLACE(history, 'Affected versions changed to ', ''), '', REPLACE(history, 'Affected versions changed to ', ''), '', created FROM gemini_issuehistory WHERE history LIKE 'Affected versions changed to %' order by created
UPDATE a SET a.idafter= b.versionname FROM gemini_issueaudit a JOIN gemini_versions b ON a.idafter COLLATE DATABASE_DEFAULT = b.versionname COLLATE DATABASE_DEFAULT AND a.projectid = b.projectid WHERE fieldchanged COLLATE DATABASE_DEFAULT  ='AffectedVersions' COLLATE DATABASE_DEFAULT 
GO

INSERT INTO gemini_issueaudit(projectid, issueid, userid, username, attributechanged, fieldchanged, iscustom, idafter, idbefore, valueafter, valuebefore, created)
SELECT projectid, issueid, NULL, username, 10, 'Status', 0, REPLACE(history, 'Issue status changed to ', ''), '', REPLACE(history, 'Issue status changed to ', ''), '', created FROM gemini_issuehistory WHERE history LIKE 'Issue status changed to %' order by created
UPDATE a SET a.idafter= b.statusid FROM gemini_issueaudit a JOIN gemini_issuestatus b ON a.idafter COLLATE DATABASE_DEFAULT = b.statusdesc COLLATE DATABASE_DEFAULT WHERE fieldchanged COLLATE DATABASE_DEFAULT ='Status' COLLATE DATABASE_DEFAULT 
GO

INSERT INTO gemini_issueaudit(projectid, issueid, userid, username, attributechanged, fieldchanged, iscustom, idafter, idbefore, valueafter, valuebefore, created)
SELECT projectid, issueid, NULL, username, 11, 'Resolution', 0, REPLACE(history, 'Issue resolution changed to ', ''), '', REPLACE(history, 'Issue resolution changed to ', ''), '', created FROM gemini_issuehistory WHERE history LIKE 'Issue resolution changed to %' order by created
UPDATE a SET a.idafter= b.resolutionid FROM gemini_issueaudit a JOIN gemini_issueresolutions b ON a.idafter COLLATE DATABASE_DEFAULT = b.resdesc COLLATE DATABASE_DEFAULT WHERE fieldchanged COLLATE DATABASE_DEFAULT ='Resolution' COLLATE DATABASE_DEFAULT 
GO

INSERT INTO gemini_issueaudit(projectid, issueid, userid, username, attributechanged, fieldchanged, iscustom, idafter, idbefore, valueafter, valuebefore, created)
SELECT projectid, issueid, NULL, username, 16, 'StartDate', 0, '', '', REPLACE(history, 'Start date changed to ', ''), '', created FROM gemini_issuehistory WHERE history LIKE 'Start date changed to %' order by created
GO

INSERT INTO gemini_issueaudit(projectid, issueid, userid, username, attributechanged, fieldchanged, iscustom, idafter, idbefore, valueafter, valuebefore, created)
SELECT projectid, issueid, NULL, username, 17, 'DueDate', 0, '', '', REPLACE(history, 'Due date changed to ', ''), '', created FROM gemini_issuehistory WHERE history LIKE 'Due date changed to %' order by created
GO

INSERT INTO gemini_issueaudit(projectid, issueid, userid, username, attributechanged, fieldchanged, iscustom, idafter, idbefore, valueafter, valuebefore, created)
SELECT projectid, issueid, NULL, username, 14, 'PercentComplete', 0, REPLACE(history, 'Percent complete changed to ', ''), '', REPLACE(history, 'Percent complete changed to ', ''), '', created FROM gemini_issuehistory WHERE history LIKE 'Percent complete changed to %' order by created
GO

INSERT INTO gemini_issueaudit(projectid, issueid, userid, username, attributechanged, fieldchanged, iscustom, idafter, idbefore, valueafter, valuebefore, created)
SELECT projectid, issueid, NULL, username, 15, 'EstimatedEffort', 0, '', '', REPLACE(history, 'Estimated hours effort changed to ', '')+'h 0m', '', created FROM gemini_issuehistory WHERE history LIKE 'Estimated hours effort changed to %' order by created
GO
INSERT INTO gemini_issueaudit(projectid, issueid, userid, username, attributechanged, fieldchanged, iscustom, idafter, idbefore, valueafter, valuebefore, created)
SELECT projectid, issueid, NULL, username, 15, 'EstimatedEffort', 0, '', '', '0h '+REPLACE(history, 'Estimated minutes effort changed to ', '')+'m', '', created FROM gemini_issuehistory WHERE history LIKE 'Estimated minutes effort changed to %' order by created
GO

INSERT INTO gemini_issueaudit(projectid, issueid, userid, username, attributechanged, fieldchanged, iscustom, idafter, idbefore, valueafter, valuebefore, created)
SELECT projectid, issueid, NULL, username, 2, 'Description', 0, '', '', '', '', created FROM gemini_issuehistory WHERE history COLLATE DATABASE_DEFAULT = 'Issue description changed' COLLATE DATABASE_DEFAULT order by created
INSERT INTO gemini_issueaudit(projectid, issueid, userid, username, attributechanged, fieldchanged, iscustom, idafter, idbefore, valueafter, valuebefore, created)
SELECT projectid, issueid, NULL, username, 2, 'Description', 0, '', '', REPLACE(history, 'Issue Description Updated ', ''),'', created FROM gemini_issuehistory WHERE history LIKE 'Issue Description Updated %' order by created
GO
INSERT INTO gemini_issueaudit(projectid, issueid, userid, username, attributechanged, fieldchanged, iscustom, idafter, idbefore, valueafter, valuebefore, created)
SELECT projectid, issueid, NULL, username, 5, 'Visibility', 0, '', '', '', '', created FROM gemini_issuehistory WHERE history COLLATE DATABASE_DEFAULT = 'Issue visibility changed' COLLATE DATABASE_DEFAULT order by created
GO

INSERT INTO gemini_issueaudit(projectid, issueid, userid, username, attributechanged, fieldchanged, iscustom, idafter, idbefore, valueafter, valuebefore, created)
SELECT projectid, issueid, NULL, username, 36, 'AssociatedComments', 0, '', '', REPLACE(history, '[Updated Comment] ', ''), '', created FROM gemini_issuehistory WHERE history LIKE '[[]updated comment]%'  order by created
GO
INSERT INTO gemini_issueaudit(projectid, issueid, userid, username, attributechanged, fieldchanged, iscustom, idafter, idbefore, valueafter, valuebefore, created)
SELECT projectid, issueid, NULL, username, 36, 'AssociatedComments', 0, '', '', REPLACE(history, '[Comment Added] ', ''), '', created FROM gemini_issuehistory WHERE history LIKE '[[]Comment Added]%'  order by created
GO
INSERT INTO gemini_issueaudit(projectid, issueid, userid, username, attributechanged, fieldchanged, iscustom, idafter, idbefore, valueafter, valuebefore, created)
SELECT projectid, issueid, NULL, username, 38, 'AssociatedAttachments', 0, '', '', REPLACE(history, 'Added attachment: ', ''), '', created FROM gemini_issuehistory WHERE history LIKE 'Added attachment: %'  order by created
GO
INSERT INTO gemini_issueaudit(projectid, issueid, userid, username, attributechanged, fieldchanged, iscustom, idafter, idbefore, valueafter, valuebefore, created)
SELECT projectid, issueid, NULL, username, 38, 'AssociatedAttachments', 0, '', '', REPLACE(history, 'Added comment attachment: ', ''), '', created FROM gemini_issuehistory WHERE history LIKE 'Added comment attachment: %'  order by created
GO
INSERT INTO gemini_issueaudit(projectid, issueid, userid, username, attributechanged, fieldchanged, iscustom, idafter, idbefore, valueafter, valuebefore, created)
SELECT projectid, issueid, NULL, username, 35, 'AssociatedWatchers', 0, '', '', REPLACE(history, 'Issue watcher added: ', ''), '', created FROM gemini_issuehistory WHERE history LIKE 'Issue watcher added: %'  order by created
GO

INSERT INTO gemini_issueaudit(projectid, issueid, userid, username, attributechanged, fieldchanged, iscustom, idafter, idbefore, valueafter, valuebefore, created)
SELECT projectid, issueid, NULL, username, 43, 'AssociatedCustomFields', 1, SUBSTRING(history, 14, CHARINDEX(' changed from', history, 14) - 14), SUBSTRING(history, 14, CHARINDEX(' changed from', history, 14) - 14), SUBSTRING(history, CHARINDEX(' to ', history)+4, 7999), SUBSTRING(SUBSTRING(history, CHARINDEX('changed from ', history)+1+LEN('changed from '), 7999), 1, CHARINDEX(' to ', SUBSTRING(history, CHARINDEX('changed from ', history)+1+LEN('changed from '), 7999))), created FROM gemini_issuehistory WHERE history LIKE 'custom field%' and history not like '%changed' order by created
UPDATE a SET a.idafter= b.customfieldid, a.idbefore = b.customfieldid FROM gemini_issueaudit a JOIN gemini_customfielddefinitions b ON a.idafter COLLATE DATABASE_DEFAULT = b.customfieldname COLLATE DATABASE_DEFAULT WHERE fieldchanged COLLATE DATABASE_DEFAULT  ='AssociatedCustomFields' COLLATE DATABASE_DEFAULT 
GO

-- Permissions reset
update gemini_projects set globalschemeid=null
GO
DELETE FROM gemini_globalsecurityschemeroles
GO
DELETE FROM gemini_globalsecurityschemes
GO

-- Update user id
UPDATE gemini_issueaudit SET userid= (SELECT TOP 1 userid FROM gemini_users WHERE firstname + ' ' + surname COLLATE DATABASE_DEFAULT= gemini_issueaudit.username COLLATE DATABASE_DEFAULT)
/**********************************************************************************************************************
* End
**********************************************************************************************************************/

/**********************************************************************************************************************
* Discontinued 
**********************************************************************************************************************/
TRUNCATE TABLE gemini_issuealerts
GO
ALTER TABLE gemini_watchproject DROP CONSTRAINT gemini_watchproject_userid_fk
GO
ALTER TABLE gemini_watchproject DROP CONSTRAINT gemini_watchproject_projectid_fk
GO
ALTER TABLE gemini_mailboxprocessor DROP CONSTRAINT gemini_mailboxprocessor_visibilitymembertype_def
ALTER TABLE gemini_mailboxprocessor DROP COLUMN visibilitymembertype
GO
ALTER TABLE gemini_projects DROP CONSTRAINT	gemini_projects_issuetypeschemeid_fk 
ALTER TABLE gemini_projects DROP COLUMN issuetypeschemeid
ALTER TABLE gemini_projects DROP CONSTRAINT	gemini_projects_issuepriorityschemeid_fk 
ALTER TABLE gemini_projects DROP COLUMN issuepriorityschemeid
ALTER TABLE gemini_projects DROP CONSTRAINT	gemini_projects_issueworkflowid_fk 
ALTER TABLE gemini_projects DROP COLUMN issueworkflowid
ALTER TABLE gemini_projects DROP CONSTRAINT	gemini_projects_fieldvisibilityschemeid_def 
ALTER TABLE gemini_projects DROP COLUMN fieldvisibilityschemeid

IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'gemini_issues_issueseverityschemeid_fk') AND parent_object_id = OBJECT_ID(N'gemini_projects'))
ALTER TABLE gemini_projects DROP CONSTRAINT gemini_issues_issueseverityschemeid_fk
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'gemini_projects_issueseverityschemeid_fk') AND parent_object_id = OBJECT_ID(N'gemini_projects'))
ALTER TABLE gemini_projects DROP CONSTRAINT gemini_projects_issueseverityschemeid_fk

ALTER TABLE gemini_projects DROP COLUMN issueseverityschemeid 
ALTER TABLE gemini_projects DROP CONSTRAINT	gemini_projects_settingdescription_def
GO
ALTER TABLE gemini_projects DROP COLUMN settingdescription
GO
ALTER TABLE gemini_issuetypes DROP CONSTRAINT gemini_issuetypes_createstagefieldvisibilityschemeid_def 
GO
ALTER TABLE gemini_issuetypes DROP COLUMN createstagefieldvisibilityschemeid 
GO
ALTER TABLE gemini_issuetypes DROP CONSTRAINT gemini_issuetypes_editstagefieldvisibilityschemeid_def 
GO
ALTER TABLE gemini_issuetypes DROP COLUMN editstagefieldvisibilityschemeid 
GO
ALTER TABLE gemini_issuetypes DROP CONSTRAINT gemini_issuetypes_viewstagefieldvisibilityschemeid_def 
GO
ALTER TABLE gemini_issuetypes DROP COLUMN viewstagefieldvisibilityschemeid 
GO
ALTER TABLE gemini_issuetypes DROP CONSTRAINT gemini_issuetypes_commentstagefieldvisibilityschemeid_def 
GO
ALTER TABLE gemini_issuetypes DROP COLUMN commentstagefieldvisibilityschemeid 
GO
ALTER TABLE gemini_issuetypes DROP CONSTRAINT gemini_issuetypes_issueworkflowid_def 
GO
ALTER TABLE gemini_issuetypes DROP COLUMN issueworkflowid 
GO
ALTER TABLE gemini_mailboxprocessor DROP CONSTRAINT gemini_mailboxprocessor_issuerisklevelid_fk
GO
ALTER TABLE gemini_mailboxprocessor DROP COLUMN issuerisklevelid 
GO
ALTER TABLE gemini_issuecomments DROP CONSTRAINT gemini_issuecomments_isprivate_def
GO
ALTER TABLE gemini_issuecomments DROP COLUMN isprivate 
GO
ALTER TABLE gemini_usersettings DROP CONSTRAINT gemini_usesettings_userid_fk
GO
ALTER TABLE gemini_issues DROP CONSTRAINT gemini_issues_isprivate_def
ALTER TABLE gemini_issues DROP COLUMN isprivate 
IF EXISTS( SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_NAME COLLATE DATABASE_DEFAULT = 'gemini_issues' COLLATE DATABASE_DEFAULT AND  COLUMN_NAME COLLATE DATABASE_DEFAULT = 'estimatedays' COLLATE DATABASE_DEFAULT) ALTER TABLE gemini_issues DROP CONSTRAINT gemini_issues_estimateddays_def 
IF EXISTS( SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_NAME COLLATE DATABASE_DEFAULT = 'gemini_issues' COLLATE DATABASE_DEFAULT AND  COLUMN_NAME COLLATE DATABASE_DEFAULT = 'estimatedays' COLLATE DATABASE_DEFAULT) ALTER TABLE gemini_issues DROP COLUMN estimatedays
GO
IF EXISTS(SELECT name FROM sysindexes WHERE name COLLATE DATABASE_DEFAULT = 'ind_gemini_issues_issuerisklevelid' COLLATE DATABASE_DEFAULT ) DROP INDEX ind_gemini_issues_issuerisklevelid ON gemini_issues
GO
ALTER TABLE gemini_issues DROP CONSTRAINT gemini_issues_issuerisklevelid_fk 
GO
ALTER TABLE gemini_issues DROP COLUMN issuerisklevelid 
GO
IF  EXISTS (SELECT * FROM sys.default_constraints WHERE object_id = OBJECT_ID(N'gemini_issues_visibilitymembertype_def') AND parent_object_id = OBJECT_ID(N'gemini_issues'))
ALTER TABLE gemini_issues DROP CONSTRAINT gemini_issues_visibilitymembertype_def
IF  EXISTS (SELECT * FROM sys.default_constraints WHERE object_id = OBJECT_ID(N'gemini_issuesvisibilitymembertype_def') AND parent_object_id = OBJECT_ID(N'gemini_issues'))
ALTER TABLE gemini_issues DROP CONSTRAINT gemini_issuesvisibilitymembertype_def

IF EXISTS(SELECT name FROM sysindexes WHERE name COLLATE DATABASE_DEFAULT ='ind_gemini_issues_visibilitymembertype' COLLATE DATABASE_DEFAULT ) DROP INDEX ind_gemini_issues_visibilitymembertype ON gemini_issues
GO
ALTER TABLE gemini_issues DROP COLUMN visibilitymembertype
GO
DROP TABLE gemini_issuerisklevels
GO

ALTER TABLE gemini_customfielddefinitions DROP CONSTRAINT gemini_customfielddefinitions_isactive_def 
GO
ALTER TABLE gemini_customfielddefinitions DROP COLUMN isactive 
GO
ALTER TABLE gemini_customfielddefinitions DROP CONSTRAINT gemini_customfielddefinitions_numcols_def 
GO
ALTER TABLE gemini_customfielddefinitions DROP COLUMN numcols 
GO
ALTER TABLE gemini_customfielddefinitions DROP CONSTRAINT gemini_customfielddefinitions_numrows_def 
GO
ALTER TABLE gemini_customfielddefinitions DROP COLUMN  numrows 
GO
ALTER TABLE gemini_customfielddefinitions DROP CONSTRAINT gemini_customfielddefinitions_cancreate_def 
GO
ALTER TABLE gemini_customfielddefinitions DROP COLUMN cancreate 
GO
ALTER TABLE gemini_customfielddefinitions DROP CONSTRAINT gemini_customfielddefinitions_canedit_def 
GO
ALTER TABLE gemini_customfielddefinitions DROP COLUMN canedit 
GO
ALTER TABLE gemini_customfielddefinitions DROP CONSTRAINT gemini_customfielddefinitions_canview_def 
GO
ALTER TABLE gemini_customfielddefinitions DROP COLUMN canview 
GO
ALTER TABLE gemini_customfielddefinitions DROP CONSTRAINT gemini_customfielddefinitions_requiredfield_def 
GO
ALTER TABLE gemini_customfielddefinitions DROP COLUMN requiredfield 
GO
ALTER TABLE gemini_customfielddefinitions DROP CONSTRAINT gemini_customfielddefinitions_defaultvalue_def 
GO
ALTER TABLE gemini_customfielddefinitions DROP COLUMN defaultvalue 
GO
ALTER TABLE gemini_customfielddefinitions DROP CONSTRAINT gemini_customfielddefinitions_tag_def 
GO
ALTER TABLE gemini_customfielddefinitions DROP COLUMN tag 
GO
IF EXISTS(SELECT name FROM sysindexes WHERE name COLLATE DATABASE_DEFAULT ='ind_gemini_customfielddefinitions_customfieldid' COLLATE DATABASE_DEFAULT ) DROP INDEX ind_gemini_customfielddefinitions_customfieldid ON gemini_customfielddefinitions
GO

ALTER TABLE gemini_issuetypeschemeitems DROP CONSTRAINT gemini_issuetypeschemeitems_issuetypeschemeid_fk
ALTER TABLE gemini_issuepriorityschemeitems DROP CONSTRAINT gemini_issuepriorityschemeitems_issuepriorityschemeid_fk
ALTER TABLE gemini_issueseverityschemeitems DROP CONSTRAINT gemini_issueseverityschemeitems_issueseverityschemeid_fk
GO
ALTER TABLE gemini_issuetypeschemeitems DROP CONSTRAINT gemini_issuetypeschemeitems_typeid_fk
ALTER TABLE gemini_issuepriorityschemeitems DROP CONSTRAINT gemini_issuepriorityschemeitems_priorityid_fk
ALTER TABLE gemini_issueseverityschemeitems DROP CONSTRAINT gemini_issueseverityschemeitems_severityid_fk
GO

ALTER TABLE gemini_issuecomments DROP CONSTRAINT gemini_issuecomments_visibilitymembertype_def
ALTER TABLE gemini_issuecomments DROP COLUMN visibilitymembertype
GO
DROP VIEW gemini_issuesview
GO

ALTER TABLE gemini_testing_case_issues DROP CONSTRAINT gemini_testing_case_issues_testcaseid_fk
GO
ALTER TABLE gemini_testing_run_steps DROP CONSTRAINT gemini_testing_run_steps_testcaseid_fk
GO
ALTER TABLE gemini_testing_run_cases DROP CONSTRAINT gemini_testing_run_cases_testcaseid_fk
GO
ALTER TABLE gemini_testing_run_cases DROP CONSTRAINT gemini_testing_run_cases_testplanid_fk
GO
ALTER TABLE gemini_testing_plan_cases DROP CONSTRAINT gemini_testing_plan_cases_testcaseid_fk
GO
ALTER TABLE gemini_testing_plan_cases DROP CONSTRAINT gemini_testing_plan_cases_testplanid_fk
GO
ALTER TABLE gemini_testing_case_steps DROP CONSTRAINT gemini_testing_case_steps_testcaseid_fk
GO
ALTER TABLE gemini_testing_runs DROP CONSTRAINT gemini_testing_runs_testcaseid_fk
GO
ALTER TABLE gemini_testing_runs DROP CONSTRAINT gemini_testing_runs_testplanid_fk
GO
/**********************************************************************************************************************
* End
**********************************************************************************************************************/

DECLARE @templateid INT
SET @templateid = 1
DECLARE @typePlan INT
DECLARE @typeCase INT
DECLARE @expectedResult INT
DECLARE @preConditions INT
DECLARE @type INT
--DECLARE @projectid INT

DECLARE @defStatus INT
DECLARE @defPriority INT
DECLARE @defSeverity INT
DECLARE @defResolution INT


/*INSERT INTO gemini_projects(color,componentmode, projectcode, projectdesc, projectname,resourcemode, templateid)
VALUES ('#f5f5f5', 0, 'QA', 'QA - Test Scripts', 'QA', 0, 16)
SELECT @projectid = SCOPE_IDENTITY()*/
INSERT gemini_customfielddefinitions(customfieldname,screenorder,screenlabel,screendescription,screentooltip,maxlen,canmultiselect,regularexp,customfieldtype,lookupname,lookupvaluefield,lookuptextfield,lookupsortfield,projectidfilter,showinline,lookupdata,maxvalue,minvalue,listlimiter,usedin,templateid,usestaticdata) VALUES('Test Type','0','Test Type','','Test Type','0','0','','C','gemini_testing_types','typeid','typedesc','typedesc','0','1','','0.00000','0.00000','','1','1','0')
INSERT gemini_customfielddefinitions(customfieldname,screenorder,screenlabel,screendescription,screentooltip,maxlen,canmultiselect,regularexp,customfieldtype,lookupname,lookupvaluefield,lookuptextfield,lookupsortfield,projectidfilter,showinline,lookupdata,maxvalue,minvalue,listlimiter,usedin,templateid,usestaticdata) VALUES('Preconditions','0','Pre-Conditions','','Pre-Conditions','250','0','','R','','','','','0','0','                                              ','0.00000','0.00000','','1','1','0')
INSERT gemini_customfielddefinitions(customfieldname,screenorder,screenlabel,screendescription,screentooltip,maxlen,canmultiselect,regularexp,customfieldtype,lookupname,lookupvaluefield,lookuptextfield,lookupsortfield,projectidfilter,showinline,lookupdata,maxvalue,minvalue,listlimiter,usedin,templateid,usestaticdata) VALUES('Expected Results','0','Expected Results','','Expected Results','250','0','','R','','','','','0','0','                                              ','0.00000','0.00000','','1','1','0')

INSERT gemini_issuetypes(seq,typedesc,imagepath,color,screen,workflow,tag,templateid) VALUES('1','Test Plan','assets/images/meta/SENTRY/type-investigation.png','#374fe8','{"ReferenceId":0,"Create":[{"ProjectGroups":[1],"Required":false,"Sequence":0,"Label":"","Id":"Type","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"Title","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":2,"Label":"","Id":"Description","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":3,"Label":"","Id":"Priority","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":4,"Label":"","Id":"Severity","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":5,"Label":"","Id":"ReportedBy","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":6,"Label":"","Id":"PercentComplete","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":7,"Label":"","Id":"AssociatedComments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":8,"Label":"","Id":"StartDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":9,"Label":"","Id":"DueDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":10,"Label":"","Id":"AssociatedAttachments","Type":0}],"Edit":[{"ProjectGroups":[1],"Required":false,"Sequence":0,"Label":"","Id":"Type","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"Title","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":2,"Label":"","Id":"Priority","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":3,"Label":"","Id":"Severity","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":4,"Label":"","Id":"ReportedBy","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":5,"Label":"","Id":"PercentComplete","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":6,"Label":"","Id":"AssociatedComments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":7,"Label":"","Id":"StartDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":8,"Label":"","Id":"DueDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":9,"Label":"","Id":"AssociatedAttachments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":10,"Label":"","Id":"Source","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":11,"Label":"","Id":"Status","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":12,"Label":"","Id":"Resolution","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":13,"Label":"","Id":"AssignedTo","Type":0}],"View":[{"ProjectGroups":[1],"Required":false,"Sequence":0,"Label":"","Id":"Type","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"Title","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":2,"Label":"","Id":"Priority","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":3,"Label":"","Id":"Severity","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":4,"Label":"","Id":"ReportedBy","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":5,"Label":"","Id":"PercentComplete","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":6,"Label":"","Id":"StartDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":7,"Label":"","Id":"DueDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":8,"Label":"","Id":"Source","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":9,"Label":"","Id":"Status","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":10,"Label":"","Id":"Resolution","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":11,"Label":"","Id":"AssignedTo","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":12,"Label":"","Id":"DateCreated","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":13,"Label":"","Id":"ClosedDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":14,"Label":"","Id":"ResolvedDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":15,"Label":"","Id":"CalculatedTimeLogged","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":16,"Label":"","Id":"IssueKey","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":17,"Label":"","Id":"Description","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":18,"Label":"","Id":"AssociatedAttachments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":19,"Label":"","Id":"AssociatedComments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":20,"Label":"","Id":"AssociatedLinks","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":21,"Label":"","Id":"598B4999-4C31-427D-9BBC-C84AD954E55A","Type":2},{"ProjectGroups":[1],"Required":false,"Sequence":22,"Label":"","Id":"EFEE5D50-7DF6-49C3-9D69-E7EFC1AFEF72","Type":2},{"ProjectGroups":[1],"Required":false,"Sequence":23,"Label":"","Id":"AC93C4F9-B512-4A3D-8B37-BC3D5F83429D","Type":2},{"ProjectGroups":[1],"Required":false,"Sequence":24,"Label":"","Id":"D8DECEB7-BA6D-4A3F-A4A7-C79A5A5585BA","Type":2}],"IsNew":true,"IsExisting":false,"Active":true,"Archived":false,"Deleted":false,"Created":"2012-08-13T11:50:23.7683507Z","Revised":"2012-08-13T11:50:23.7683507Z","Errors":[],"HasErrors":false,"Id":0}','','PLAN','1')
INSERT gemini_issuetypes(seq,typedesc,imagepath,color,screen,workflow,tag,templateid) VALUES('2','Test Case','assets/images/meta/SENTRY/type-task.png','#20700c','{"ReferenceId":0,"Create":[{"ProjectGroups":[1],"Required":false,"Sequence":0,"Label":"","Id":"Type","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"Title","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":2,"Label":"","Id":"41","Type":1},{"ProjectGroups":[1],"Required":false,"Sequence":3,"Label":"","Id":"Description","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":4,"Label":"","Id":"42","Type":1},{"ProjectGroups":[1],"Required":false,"Sequence":5,"Label":"","Id":"43","Type":1},{"ProjectGroups":[1],"Required":false,"Sequence":6,"Label":"","Id":"Status","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":7,"Label":"","Id":"Component","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":8,"Label":"","Id":"AssignedTo","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":9,"Label":"","Id":"StartDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":10,"Label":"","Id":"DueDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":11,"Label":"","Id":"PercentComplete","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":12,"Label":"","Id":"AffectedVersions","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":13,"Label":"","Id":"AssociatedAttachments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":14,"Label":"","Id":"AssociatedComments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":15,"Label":"","Id":"EstimatedEffort","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":16,"Label":"","Id":"Priority","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":17,"Label":"","Id":"Resolution","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":18,"Label":"","Id":"Severity","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":19,"Label":"","Id":"Source","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":20,"Label":"","Id":"AssociatedTime","Type":0}],"Edit":[{"ProjectGroups":[1],"Required":false,"Sequence":0,"Label":"","Id":"Type","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"Title","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":2,"Label":"","Id":"41","Type":1},{"ProjectGroups":[1],"Required":false,"Sequence":3,"Label":"","Id":"Description","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":4,"Label":"","Id":"42","Type":1},{"ProjectGroups":[1],"Required":false,"Sequence":5,"Label":"","Id":"43","Type":1},{"ProjectGroups":[1],"Required":false,"Sequence":6,"Label":"","Id":"Status","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":7,"Label":"","Id":"Component","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":8,"Label":"","Id":"AssignedTo","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":9,"Label":"","Id":"StartDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":10,"Label":"","Id":"DueDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":11,"Label":"","Id":"PercentComplete","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":12,"Label":"","Id":"AffectedVersions","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":13,"Label":"","Id":"AssociatedAttachments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":14,"Label":"","Id":"AssociatedComments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":15,"Label":"","Id":"EstimatedEffort","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":16,"Label":"","Id":"Priority","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":17,"Label":"","Id":"Resolution","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":18,"Label":"","Id":"Severity","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":19,"Label":"","Id":"Source","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":20,"Label":"","Id":"AssociatedTime","Type":0}],"View":[{"ProjectGroups":[1],"Required":false,"Sequence":0,"Label":"","Id":"Type","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":1,"Label":"","Id":"Title","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":2,"Label":"","Id":"41","Type":1},{"ProjectGroups":[1],"Required":false,"Sequence":3,"Label":"","Id":"Status","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":4,"Label":"","Id":"Component","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":5,"Label":"","Id":"AssignedTo","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":6,"Label":"","Id":"StartDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":7,"Label":"","Id":"DueDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":8,"Label":"","Id":"PercentComplete","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":9,"Label":"","Id":"AffectedVersions","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":10,"Label":"","Id":"EstimatedEffort","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":11,"Label":"","Id":"Priority","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":12,"Label":"","Id":"Resolution","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":13,"Label":"","Id":"Severity","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":14,"Label":"","Id":"Source","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":15,"Label":"","Id":"ResolvedDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":16,"Label":"","Id":"ClosedDate","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":17,"Label":"","Id":"DateCreated","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":18,"Label":"","Id":"DateRevised","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":19,"Label":"","Id":"IssueKey","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":20,"Label":"","Id":"CalculatedTimeLogged","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":21,"Label":"","Id":"Description","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":22,"Label":"","Id":"42","Type":1},{"ProjectGroups":[1],"Required":false,"Sequence":23,"Label":"","Id":"43","Type":1},{"ProjectGroups":[1],"Required":false,"Sequence":24,"Label":"","Id":"AssociatedComments","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":25,"Label":"","Id":"AssociatedLinks","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":26,"Label":"","Id":"AssociatedTime","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":27,"Label":"","Id":"AssociatedHistory","Type":0},{"ProjectGroups":[1],"Required":false,"Sequence":28,"Label":"","Id":"617F1435-C0AA-48E6-B6A9-2C1F756E02A6","Type":2},{"ProjectGroups":[1],"Required":false,"Sequence":29,"Label":"","Id":"15627A0F-5AD2-4D35-BF24-60D678F59354","Type":2},{"ProjectGroups":[1],"Required":false,"Sequence":30,"Label":"","Id":"ACE2F8B3-C724-444F-9B72-B8B2137DDA16","Type":2}],"IsNew":true,"IsExisting":false,"Active":true,"Archived":false,"Deleted":false,"Created":"2012-08-13T11:22:02.4410404Z","Revised":"2012-08-13T11:22:02.4410404Z","Errors":[],"HasErrors":false,"Id":0}','','CASE','1')

SELECT @typePlan = typeid FROM gemini_issuetypes WHERE typedesc='Test Plan' AND templateid=@templateid
SELECT @typeCase = typeid FROM gemini_issuetypes WHERE typedesc='Test Case' AND templateid=@templateid
SELECT @expectedResult = customfieldid FROM gemini_customfielddefinitions WHERE customfieldname='Expected Results' AND templateid=@templateid
SELECT @preConditions = customfieldid FROM gemini_customfielddefinitions WHERE customfieldname='Preconditions' AND templateid=@templateid
SELECT @type = customfieldid FROM gemini_customfielddefinitions WHERE customfieldname='Test Type' AND templateid=@templateid
SELECT @defStatus = MIN(statusid) FROM gemini_issuestatus WHERE templateid=@templateid
SELECT @defPriority = MIN(priorityid) FROM gemini_issuepriorities WHERE templateid=@templateid
SELECT @defSeverity = MIN(severityid) FROM gemini_issueseverity WHERE templateid=@templateid
SELECT @defResolution = MIN(resolutionid) FROM gemini_issueresolutions WHERE templateid=@templateid

-- Migrate Plans to issues table
INSERT INTO gemini_issues(created, issuepriorityid, issueresolutionid, issueseverityid, issuestatusid, issuetypeid, longdesc, projectid, reportedby, summary, userdata1, userdata2, visibility)
SELECT created, @defPriority, @defResolution, @defSeverity, @defStatus, @typePlan, plandescription, projectid, createdby, summary, testplanid, 'Migrated Plan', 1  FROM gemini_testing_plans WHERE isfolder=0

-- Migrate Cases to issues table
INSERT INTO gemini_issues(created, issuepriorityid, issueresolutionid, issueseverityid, issuestatusid, issuetypeid, longdesc, projectid, reportedby, summary, userdata1, userdata2, visibility, resources, revised)
SELECT created, @defPriority, @defResolution, @defSeverity, @defStatus, @typeCase, testcasedescription , projectid, createdby, summary, testcaseid, 'Migrated Case', 1,  CASE WHEN assignedto IS NULL THEN '' ELSE '|' + CAST(assignedto AS VARCHAR(10)) + '|' END, revised FROM gemini_testing_cases

INSERT INTO gemini_issueresources(issueid, userid)
SELECT i.issueid, c.assignedto FROM gemini_testing_cases c JOIN gemini_issues i ON i.userdata2='Migrated Case' AND i.userdata1=c.testcaseid WHERE c.assignedto IS NOT NULL
-- Move Case associate data to custom fields
INSERT INTO gemini_customfielddata(customfieldid, fielddata, issueid, projectid, userid)
SELECT @expectedResult, c.expectedresult, i.issueid, i.projectid, c.createdby FROM gemini_testing_cases c JOIN gemini_issues i ON i.userdata2='Migrated Case' AND i.userdata1=c.testcaseid
INSERT INTO gemini_customfielddata(customfieldid, fielddata, issueid, projectid, userid)
SELECT @preConditions, c.preconditions, i.issueid, i.projectid, c.createdby FROM gemini_testing_cases c JOIN gemini_issues i ON i.userdata2='Migrated Case' AND i.userdata1=c.testcaseid
INSERT INTO gemini_customfielddata(customfieldid, fielddata, issueid, projectid, userid)
SELECT @type, c.typeid, i.issueid, i.projectid, c.createdby FROM gemini_testing_cases c JOIN gemini_issues i ON i.userdata2='Migrated Case' AND i.userdata1=c.testcaseid

-- Update plan ids on related testing tables
UPDATE a SET a.testplanid = i.issueid FROM gemini_testing_plan_cases a JOIN gemini_issues i ON i.userdata2='Migrated Plan' AND i.userdata1=a.testplanid
UPDATE a SET a.testplanid = i.issueid FROM gemini_testing_run_cases a JOIN gemini_issues i ON i.userdata2='Migrated Plan' AND i.userdata1=a.testplanid
UPDATE a SET a.testplanid = i.issueid FROM gemini_testing_runs a JOIN gemini_issues i ON i.userdata2='Migrated Plan' AND i.userdata1=a.testplanid
-- DELETE RUNs FOR CASES ONLY.....

-- Update case ids on related testing tables
UPDATE a SET a.testcaseid = i.issueid FROM gemini_testing_plan_cases a JOIN gemini_issues i ON i.userdata2='Migrated Case' AND i.userdata1=a.testcaseid
UPDATE a SET a.testcaseid = i.issueid FROM gemini_testing_run_cases a JOIN gemini_issues i ON i.userdata2='Migrated Case' AND i.userdata1=a.testcaseid
UPDATE a SET a.testcaseid = i.issueid FROM gemini_testing_run_steps a JOIN gemini_issues i ON i.userdata2='Migrated Case' AND i.userdata1=a.testcaseid
UPDATE a SET a.testcaseid = i.issueid FROM gemini_testing_case_steps a JOIN gemini_issues i ON i.userdata2='Migrated Case' AND i.userdata1=a.testcaseid
UPDATE a SET a.testcaseid = i.issueid FROM gemini_testing_case_issues a JOIN gemini_issues i ON i.userdata2='Migrated Case' AND i.userdata1=a.testcaseid

/*UPDATE gemini_testing_run_cases SET projectid = @projectid
UPDATE gemini_testing_runs SET projectid = @projectid*/

UPDATE gemini_issues SET originatordata = (SELECT CAST(testplanid AS VARCHAR(10)) + '|' + originatordata FROM gemini_testing_runs WHERE testrunid = originatordata) WHERE originatortype = 3

-- Now move custom fields?

